@@ -19,6 +19,18 @@ from site import USER_BASE
 from site import USER_SITE
 HAS_USER_SITE = True
 
+
+# See also: site.py/sysconfig.py
+def _get_implementation():
+    if '__pypy__' in sys.builtin_module_names:
+        return 'PyPy'
+    return 'Python'
+
+
+IMPLEMENTATION = _get_implementation()
+IMPLEMENTATION_LOWER = IMPLEMENTATION.lower()
+
+
 WINDOWS_SCHEME = {
     'purelib': '$base/Lib/site-packages',
     'platlib': '$base/Lib/site-packages',
@@ -29,16 +41,16 @@ WINDOWS_SCHEME = {
 
 INSTALL_SCHEMES = {
     'unix_prefix': {
-        'purelib': '$base/lib/$implementation_lower$py_version_short/site-packages',
-        'platlib': '$platbase/lib/$implementation_lower$py_version_short/site-packages',
-        'headers': '$base/include/$implementation_lower$py_version_short$abiflags/$dist_name',
+        'purelib': '$base/lib/' + IMPLEMENTATION_LOWER + '$py_version_short/site-packages',
+        'platlib': '$platbase/$platlibdir/' + IMPLEMENTATION_LOWER + '$py_version_short/site-packages',
+        'headers': '$base/include/' + IMPLEMENTATION_LOWER + '$py_version_short$abiflags/$dist_name',
         'scripts': '$base/bin',
         'data'   : '$base',
         },
     'unix_home': {
-        'purelib': '$base/lib/$implementation_lower',
-        'platlib': '$base/lib/$implementation_lower',
-        'headers': '$base/include/$implementation_lower/$dist_name',
+        'purelib': '$base/lib/' + IMPLEMENTATION_LOWER,
+        'platlib': '$base/$platlibdir/' + IMPLEMENTATION_LOWER,
+        'headers': '$base/include/' + IMPLEMENTATION_LOWER + '/$dist_name',
         'scripts': '$base/bin',
         'data'   : '$base',
         },
@@ -50,8 +62,8 @@ if HAS_USER_SITE:
     INSTALL_SCHEMES['nt_user'] = {
         'purelib': '$usersite',
         'platlib': '$usersite',
-        'headers': '$userbase/$implementation$py_version_nodot/Include/$dist_name',
-        'scripts': '$userbase/$implementation$py_version_nodot/Scripts',
+        'headers': '$userbase/' + IMPLEMENTATION + '$py_version_nodot/Include/$dist_name',
+        'scripts': '$userbase/' + IMPLEMENTATION + '$py_version_nodot/Scripts',
         'data'   : '$userbase',
         }
 
@@ -59,7 +71,7 @@ if HAS_USER_SITE:
         'purelib': '$usersite',
         'platlib': '$usersite',
         'headers':
-            '$userbase/include/$implementation_lower$py_version_short$abiflags/$dist_name',
+            '$userbase/include/' + IMPLEMENTATION_LOWER + '$py_version_short$abiflags/$dist_name',
         'scripts': '$userbase/bin',
         'data'   : '$userbase',
         }
@@ -303,8 +315,7 @@ class install(Command):
                             'sys_exec_prefix': exec_prefix,
                             'exec_prefix': exec_prefix,
                             'abiflags': abiflags,
-                            'implementation_lower': _get_implementation().lower(),
-                            'implementation': _get_implementation(),
+                            'platlibdir': sys.platlibdir,
                            }
 
         if HAS_USER_SITE:
