@@ -344,16 +344,24 @@ def getsitepackages(prefixes=None):
         if not prefix or prefix in seen:
             continue
         seen.add(prefix)
+        libdirs = [sys.platlibdir]
+        if sys.platlibdir != "lib":
+            libdirs.append("lib")
 
         implementation = _get_implementation().lower()
         ver = sys.version_info
         if os.sep == '/':
-            sitepackages.append(os.path.join(prefix, "lib",
-                                        f"{implementation}{ver[0]}.{ver[1]}",
-                                        "site-packages"))
+            for libdir in libdirs:
+                path = os.path.join(prefix, libdir,
+                                            f"{implementation}{ver[0]}.{ver[1]}",
+                                            "site-packages")
+                sitepackages.append(path)
         else:
             sitepackages.append(prefix)
-            sitepackages.append(os.path.join(prefix, "lib", "site-packages"))
+
+            for libdir in libdirs:
+                path = os.path.join(prefix, libdir, "site-packages")
+                sitepackages.append(path)
     return sitepackages
 
 def addsitepackages(known_paths, prefixes=None):
@@ -385,9 +393,8 @@ def setcopyright():
     builtins.copyright = _sitebuiltins._Printer("copyright", sys.copyright)
     licenseargs = None
     if is_pypy:
-        credits = "PyPy is maintained by the PyPy developers: http://pypy.org/"
+        credits = "PyPy is maintained by the PyPy developers\nhttp://pypy.org/"
         license = "See https://foss.heptapod.net/pypy/pypy/src/default/LICENSE"
-        licenseargs = (license,)
     elif sys.platform[:4] == 'java':
         credits = ("Jython is maintained by the Jython developers "
                    "(www.jython.org).")
