@@ -140,6 +140,7 @@ testmap = [
     RegrTest('test_cgi.py'),
     RegrTest('test_cgitb.py'),
     RegrTest('test_charmapcodec.py', core=True),
+    RegrTest('test_check_c_globals.py'),
     RegrTest('test_class.py', core=True),
     RegrTest('test_clinic.py'),
     RegrTest('test_cmath.py', core=True),
@@ -207,8 +208,6 @@ testmap = [
     RegrTest('test_doctest2.py'),
     RegrTest('test_docxmlrpc.py'),
     RegrTest('test_dtrace.py'),
-    RegrTest('test_dummy_thread.py', core=True),
-    RegrTest('test_dummy_threading.py', core=True),
     RegrTest('test_dynamic.py'),
     RegrTest('test_dynamicclassattribute.py'),
     RegrTest('test_eintr.py'),
@@ -252,6 +251,7 @@ testmap = [
     RegrTest('test_gdb.py', skip="not applicable"),
     RegrTest('test_generator_stop.py'),
     RegrTest('test_generators.py', core=True, usemodules='thread _weakref'),
+    RegrTest('test_genericalias.py'),
     RegrTest('test_genericclass.py'),
     RegrTest('test_genericpath.py'),
     RegrTest('test_genexps.py', core=True, usemodules='_weakref'),
@@ -262,6 +262,7 @@ testmap = [
     RegrTest('test_glob.py', core=True),
     RegrTest('test_global.py', core=True),
     RegrTest('test_grammar.py', core=True),
+    RegrTest('test_graphlib.py'),
     RegrTest('test_grp.py'),
     RegrTest('test_gzip.py', usemodules='zlib'),
     RegrTest('test_hash.py', core=True),
@@ -329,7 +330,6 @@ testmap = [
     RegrTest('test_netrc.py'),
     RegrTest('test_nis.py'),
     RegrTest('test_nntplib.py'),
-    RegrTest('test_normalization.py'),
     RegrTest('test_ntpath.py'),
     RegrTest('test_numeric_tower.py'),
     RegrTest('test_opcodes.py', core=True),
@@ -344,12 +344,12 @@ testmap = [
     RegrTest('test_pathlib.py'),
     RegrTest('test_pdb.py'),
     RegrTest('test_peepholer.py'),
+    RegrTest('test_peg_parser.py'),
     RegrTest('test_pickle.py', core=True),
     RegrTest('test_picklebuffer.py'),
     RegrTest('test_pickletools.py', core=False),
     RegrTest('test_pipes.py'),
     RegrTest('test_pkg.py', core=True),
-    RegrTest('test_pkgimport.py', core=True),
     RegrTest('test_pkgutil.py'),
     RegrTest('test_platform.py'),
     RegrTest('test_plistlib.py'),
@@ -447,7 +447,6 @@ testmap = [
     RegrTest('test_tempfile.py'),
     RegrTest('test_textwrap.py'),
     RegrTest('test_thread.py', usemodules="thread", core=True),
-    RegrTest('test_threaded_import.py', usemodules="thread", core=True),
     RegrTest('test_threadedtempfile.py', usemodules="thread", core=False),
     RegrTest('test_threading.py', usemodules="thread", core=True),
     RegrTest('test_threading_local.py', usemodules="thread", core=True),
@@ -481,6 +480,7 @@ testmap = [
     RegrTest('test_univnewlines.py'),
     RegrTest('test_unpack.py', core=True),
     RegrTest('test_unpack_ex.py', core=True),
+    RegrTest('test_unparse.py'),
     RegrTest('test_urllib.py'),
     RegrTest('test_urllib2.py'),
     RegrTest('test_urllib2_localnet.py', usemodules="thread"),
@@ -522,7 +522,6 @@ testmap = [
     RegrTest('test_zipimport.py', usemodules='zlib zipimport'),
     RegrTest('test_zipimport_support.py', usemodules='zlib zipimport'),
     RegrTest('test_zlib.py', usemodules='zlib'),
-
 ]
 
 def check_testmap_complete():
