@@ -23,14 +23,15 @@
 #
 # Note: This software has been modified for use in PyPy.
 
-from collections import OrderedDict
-from functools import wraps
 import datetime
+import os
 import string
 import sys
-import weakref
 import threading
-import os
+import types
+import weakref
+from collections import OrderedDict
+from functools import wraps
 
 try:
     from __pypy__ import newlist_hint, add_memory_pressure
@@ -104,14 +105,11 @@ PARSE_DECLTYPES = 2
 # SQLite version information
 sqlite_version = str(_ffi.string(_lib.sqlite3_libversion()).decode('ascii'))
 
-_STMT_TYPE_UPDATE = 0
-_STMT_TYPE_DELETE = 1
-_STMT_TYPE_INSERT = 2
-_STMT_TYPE_REPLACE = 3
-_STMT_TYPE_OTHER = 4
-_STMT_TYPE_SELECT = 5
-_STMT_TYPE_INVALID = 6
+# flag that signals if base types need adaption
+BASE_TYPE_ADAPTED = False
 
+# set of base types that are supported by SQLite3
+BASE_TYPES = {bytearray, bytes, float, int, str, NoneType}
 
 class Error(StandardError):
     pass
@@ -274,6 +272,8 @@ class Connection(object):
             _lib.sqlite3_close(self._db)
 
     def close(self):
+        if not self.__initialized:
+            raise ProgrammingError("Base Connection.__init__ not called.")
         self._check_thread()
 
         self.__do_all_statements(Statement._finalize, True)
@@ -434,6 +434,7 @@ class Connection(object):
 
     def iterdump(self):
         from sqlite3.dump import _iterdump
+        self._check_closed()
         return _iterdump(self)
 
     def _begin(self):
@@ -532,7 +533,7 @@ class Connection(object):
             if _lib.sqlite3_libversion_number() < 3008003:
                 raise NotSupportedError(
                         "deterministic=True requires SQLite 3.8.3 or higher")
-                flags |= _lib.SQLITE_DETERMINISTIC
+            flags |= _lib.SQLITE_DETERMINISTIC
         ret = _lib.sqlite3_create_function(self._db, name, nargs,
                                            flags, _ffi.NULL,
                                            closure, _ffi.NULL, _ffi.NULL)
@@ -711,6 +712,8 @@ class Connection(object):
     total_changes = property(__get_total_changes)
 
     def __get_isolation_level(self):
+        if not self.__initialized:
+            raise ProgrammingError("Base Connection.__init__ not called")
         return self._isolation_level
 
     def __set_isolation_level(self, val):
@@ -750,6 +753,7 @@ class Connection(object):
             bck_conn = target._db
             if not bck_conn or not self._db:
                 raise ProgrammingError("cannot operate on closed connection")
+            self._check_closed()
             bck_handle = _lib.sqlite3_backup_init(bck_conn, b"main", self._db, name.encode("utf-8"))
             if not bck_handle:
                 raise target._get_exception()
@@ -786,8 +790,11 @@ class Connection(object):
 class Cursor(object):
     __initialized = False
     __statement = None
+    __locked = False
 
     def __init__(self, con):
+        if self.__locked:
+            raise ProgrammingError("Recursive use of cursors not allowed.")
         if not isinstance(con, Connection):
             raise TypeError
         self.__connection = con
@@ -817,6 +824,8 @@ class Cursor(object):
     def close(self):
         if not self.__initialized:
             raise ProgrammingError("Base Cursor.__init__ not called.")
+        if self.__locked:
+            raise ProgrammingError("Recursive use of cursors not allowed.")
         self.__connection._check_thread()
         self.__connection._check_closed()
         if self.__statement:
@@ -939,7 +948,7 @@ class Cursor(object):
             pass
         try:
             if not isinstance(sql, basestring):
-                raise ValueError("operation parameter must be str or unicode")
+                raise TypeError("operation parameter must be str or unicode, not %s" % (type(sql).__name__, ))
             try:
                 del self.__description
             except AttributeError:
@@ -1066,7 +1075,12 @@ class Cursor(object):
 
         ret = _lib.sqlite3_step(self.__statement._statement)
         if ret == _lib.SQLITE_ROW:
-            self.__next_row = self.__fetch_one_row()
+            assert not self.__locked
+            self.__locked = True
+            try:
+                self.__next_row = self.__fetch_one_row()
+            finally:
+                self.__locked = False
         else:
             self.__statement._reset(self.__in_use_token)
             if ret != _lib.SQLITE_DONE:
@@ -1106,9 +1120,26 @@ class Cursor(object):
         try:
             return self.__description
         except AttributeError:
-            if self.__statement:
-                self.__description = self.__statement._get_description()
-                return self.__description
+            if not self.__statement:
+                return None
+            statement = self.__statement
+            if not statement._valid:
+                return None
+
+            numcols = _lib.sqlite3_column_count(statement._statement)
+            if numcols <= 0:
+                self.__description = None
+                return None
+            desc = []
+            for i in xrange(numcols):
+                name = _lib.sqlite3_column_name(statement._statement, i)
+                if name:
+                    name = _ffi.string(name).decode('utf-8')
+                    if self.__connection._detect_types & PARSE_COLNAMES:
+                        name = name.split("[", 1)[0].strip()
+                desc.append((name, None, None, None, None, None, None))
+            self.__description = tuple(desc)
+            return self.__description
     description = property(__get_description)
 
     def __get_lastrowid(self):
@@ -1133,7 +1164,7 @@ class Statement(object):
         self._in_use_token = None
 
         if not isinstance(sql, basestring):
-            raise Warning("SQL is of wrong type. Must be string or unicode.")
+            raise TypeError("sql argument must be str, not %s" % (type(sql).__name__, ))
         if '\0' in sql:
             raise ValueError("the query contains a null character")
 
@@ -1186,28 +1217,17 @@ class Statement(object):
             _lib.sqlite3_reset(self._statement)
             self._in_use_token = None
 
-    if sys.version_info[0] < 3:
-        def __check_decodable(self, param):
-            if self.__con.text_factory in (unicode, OptimizedUnicode,
-                                           _unicode_text_factory):
-                for c in param:
-                    if ord(c) & 0x80 != 0:
-                        raise self.__con.ProgrammingError(
-                            "You must not use 8-bit bytestrings unless "
-                            "you use a text_factory that can interpret "
-                            "8-bit bytestrings (like text_factory = str). "
-                            "It is highly recommended that you instead "
-                            "just switch your application to Unicode strings.")
-
     def __set_param(self, idx, param):
-        cvt = converters.get(type(param))
-        if cvt is not None:
-            param = cvt(param)
-
-        try:
+        typ = type(param)
+        if BASE_TYPE_ADAPTED or (
+            typ is not bytearray
+            and typ is not bytes
+            and typ is not float
+            and typ is not int
+            and typ is not str
+            and typ is not NoneType
+        ):
             param = adapt(param)
-        except Exception:
-            pass  # And use previous value
 
         if param is None:
             rc = _lib.sqlite3_bind_null(self._statement, idx)
@@ -1215,18 +1235,19 @@ class Statement(object):
             if -2147483648 <= param <= 2147483647:
                 rc = _lib.sqlite3_bind_int(self._statement, idx, param)
             else:
-                rc = _lib.sqlite3_bind_int64(self._statement, idx, param)
+                try:
+                    rc = _lib.sqlite3_bind_int64(self._statement, idx, param)
+                except OverflowError:
+                    raise OverflowError(
+                        "Python int too large to convert to SQLite INTEGER"
+                    )
         elif isinstance(param, float):
             rc = _lib.sqlite3_bind_double(self._statement, idx, param)
-        elif isinstance(param, unicode):
+        elif isinstance(param, str):
             param = param.encode("utf-8")
             rc = _lib.sqlite3_bind_text(self._statement, idx, param,
                                         len(param), _SQLITE_TRANSIENT)
-        elif isinstance(param, str):
-            self.__check_decodable(param)
-            rc = _lib.sqlite3_bind_text(self._statement, idx, param,
-                                        len(param), _SQLITE_TRANSIENT)
-        elif isinstance(param, (buffer, bytes)):
+        elif isinstance(param, (buffer, bytes, bytearray)):
             param = bytes(param)
             rc = _lib.sqlite3_bind_blob(self._statement, idx, param,
                                         len(param), _SQLITE_TRANSIENT)
@@ -1281,19 +1302,6 @@ class Statement(object):
         else:
             raise ValueError("parameters are of unsupported type")
 
-    def _get_description(self):
-        if self._is_dml or not self._valid:
-            return None
-        desc = []
-        for i in xrange(_lib.sqlite3_column_count(self._statement)):
-            name = _lib.sqlite3_column_name(self._statement, i)
-            if name:
-                name = _ffi.string(name).decode('utf-8')
-                if self.__con._detect_types & PARSE_COLNAMES:
-                    name = name.split("[")[0].strip()
-            desc.append((name, None, None, None, None, None, None))
-        return desc
-
 
 class Row(object):
     def __init__(self, cursor, values):
@@ -1397,7 +1405,8 @@ def _convert_params(con, nargs, params):
             val = _lib.sqlite3_value_double(params[i])
         elif typ == _lib.SQLITE_TEXT:
             val = _lib.sqlite3_value_text(params[i])
-            val = _ffi.string(val).decode('utf-8')
+            length = _lib.sqlite3_value_bytes(params[i])
+            val = _ffi.buffer(val, length)[:].decode('utf-8')
         elif typ == _lib.SQLITE_BLOB:
             blob = _lib.sqlite3_value_blob(params[i])
             blob_len = _lib.sqlite3_value_bytes(params[i])
@@ -1415,10 +1424,8 @@ def _convert_result(con, val):
         _lib.sqlite3_result_int64(con, int(val))
     elif isinstance(val, float):
         _lib.sqlite3_result_double(con, val)
-    elif isinstance(val, unicode):
-        val = val.encode('utf-8')
-        _lib.sqlite3_result_text(con, val, len(val), _SQLITE_TRANSIENT)
     elif isinstance(val, str):
+        val = val.encode('utf-8')
         _lib.sqlite3_result_text(con, val, len(val), _SQLITE_TRANSIENT)
     elif isinstance(val, (buffer, bytes)):
         _lib.sqlite3_result_blob(con, bytes(val), len(val), _SQLITE_TRANSIENT)
@@ -1434,7 +1441,11 @@ def _function_callback(real_cb, context, nargs, c_params):
         msg = b"user-defined function raised exception"
         _lib.sqlite3_result_error(context, msg, len(msg))
     else:
-        _convert_result(context, val)
+        try:
+            _convert_result(context, val)
+        except Exception:
+            msg = b"user-defined function raised exception"
+            _lib.sqlite3_result_error(context, msg, len(msg))
 
 converters = {}
 adapters = {}
@@ -1445,6 +1456,11 @@ class PrepareProtocol(object):
 
 
 def register_adapter(typ, callable):
+    global BASE_TYPE_ADAPTED
+
+    if typ in BASE_TYPES:
+        BASE_TYPE_ADAPTED = True
+
     adapters[typ, PrepareProtocol] = callable
 
 
