@@ -1,4 +1,4 @@
-import imp
+from importlib.util import spec_from_file_location, module_from_spec
 import os
 
 try:
@@ -10,12 +10,11 @@ else:
     cfile = '_testcapimodule.c'
     thisdir = os.path.dirname(__file__)
     output_dir = _pypy_testcapi.get_hashed_dir(os.path.join(thisdir, cfile))
-
+    modfile = '_testcapi' + _pypy_testcapi._get_c_extension_suffix()
+    spec = spec_from_file_location('_testcapi',
+                                   os.path.join(output_dir, modfile))
     try:
-        fp, filename, description = imp.find_module('_testcapi',
-                                                    path=[output_dir])
-        with fp:
-            imp.load_module('_testcapi', fp, filename, description)
+        module_from_spec(spec)
     except ImportError:
         if os.name == 'nt':
             # hack around finding compilers on win32
@@ -23,8 +22,7 @@ else:
                 import setuptools
             except ImportError:
                 pass
-        _pypy_testcapi.compile_shared(cfile, '_testcapi', output_dir)
-
+        mod = _pypy_testcapi.compile_shared(cfile, '_testcapi', output_dir)
 
 class awaitType:
     def __init__(self, iterator):
