@@ -10,7 +10,8 @@ import os
 import sys
 import time
 
-sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', '..'))
+pypy_path = os.path.join(os.path.dirname(__file__), '..', '..')
+sys.path.insert(0, os.path.abspath(pypy_path))
 
 from pypy.tool import option
 from pypy.interpreter import main, interactive, error, gateway
@@ -41,17 +42,26 @@ cmdline_optiondescr = OptionDescription("interactive", "the options of pyinterac
     StrOption("warn",
               "warning control (arg is action:message:category:module:lineno)",
               default=None, cmdline="-W"),
- 
+
     ])
 
 pypy_init = gateway.applevel('''
 def pypy_init(import_site):
     if import_site:
+        import os, sys
+        _MACOSX = sys.platform == 'darwin'
+        if _MACOSX:
+            # __PYVENV_LAUNCHER__, used by CPython on macOS, should be ignored
+            # since it (possibly) results in a wrong sys.prefix and
+            # sys.exec_prefix (and consequently sys.path).
+            old_pyvenv_launcher = os.environ.pop('__PYVENV_LAUNCHER__', None)
         try:
             import site
         except:
             import sys
-            print >> sys.stderr, "import site\' failed"
+            print("'import site' failed", file=sys.stderr)
+        if _MACOSX and old_pyvenv_launcher:
+            os.environ['__PYVENV_LAUNCHER__'] = old_pyvenv_launcher
 ''').interphook('pypy_init')
 
 
@@ -80,6 +90,7 @@ def main_(argv=None):
         error.RECORD_INTERPLEVEL_TRACEBACK = True
     # --allworkingmodules takes really long to start up, but can be forced on
     config.objspace.suggest(allworkingmodules=False)
+    config.objspace.usemodules.struct = True
     if config.objspace.allworkingmodules:
         pypyoption.enable_allworkingmodules(config)
     if config.objspace.usemodules._continuation:
@@ -100,10 +111,9 @@ def main_(argv=None):
         space.appexec([], """():
             import sys
             flags = list(sys.flags)
-            flags[6] = 2
+            flags[3] = 2
             sys.flags = type(sys.flags)(flags)
-            import __pypy__
-            __pypy__.set_debug(False)
+            __builtins__.__dict__['__debug__'] = False
         """)
 
     # call pypy_find_stdlib: the side-effect is that it sets sys.prefix and
@@ -117,7 +127,7 @@ def main_(argv=None):
     # set warning control options (if any)
     warn_arg = interactiveconfig.warn
     if warn_arg is not None:
-        space.appexec([space.wrap(warn_arg)], """(arg): 
+        space.appexec([space.wrap(warn_arg)], """(arg):
         import sys
         sys.warnoptions.append(arg)""")
 
@@ -142,7 +152,8 @@ def main_(argv=None):
     if interactiveconfig.runmodule:
         command = args.pop(0)
     for arg in args:
-        space.call_method(space.sys.get('argv'), 'append', space.wrap(arg))
+        space.call_method(space.sys.get('argv'), 'append',
+                          space.newfilename(arg))
 
     # load the source of the program given as command-line argument
     if interactiveconfig.runcommand:
@@ -152,11 +163,12 @@ def main_(argv=None):
         def doit():
             main.run_module(command, args, space=space)
     elif args:
-        scriptdir = os.path.dirname(os.path.abspath(args[0]))
+        abspath_file = os.path.abspath(args[0])
+        scriptdir = os.path.dirname(abspath_file)
         space.call_method(space.sys.get('path'), 'insert',
                           space.wrap(0), space.wrap(scriptdir))
         def doit():
-            main.run_file(args[0], space=space)
+            main.run_file(abspath_file, space=space)
     else:
         def doit():
             pass
@@ -199,6 +211,6 @@ def main_(argv=None):
 
 if __name__ == '__main__':
     if hasattr(sys, 'setrecursionlimit'):
-        # for running "python -i pyinteractive.py -Si -- py.py -Si" 
+        # for running "python -i pyinteractive.py -Si -- py.py -Si"
         sys.setrecursionlimit(3000)
     sys.exit(main_(sys.argv))
