@@ -8,14 +8,17 @@ import textwrap
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.pyparser import pyparse
 from pypy.interpreter.pyparser.error import SyntaxError
-from pypy.interpreter.astcompiler.astbuilder import ast_from_node
+from pypy.interpreter.error import OperationError
 from pypy.interpreter.astcompiler import ast, consts
 
 
-class TestAstBuilder:
+class TestAstBuilding:
 
     def setup_class(cls):
-        cls.parser = pyparse.PythonParser(cls.space)
+        cls.parser = pyparse.PegParser(cls.space)
+
+    def setup_method(self, method):
+        self.info = None
 
     def get_ast(self, source, p_mode=None, flags=None, with_async_hacks=False):
         if p_mode is None:
@@ -25,8 +28,9 @@ class TestAstBuilder:
         if with_async_hacks:
             flags |= consts.PyCF_ASYNC_HACKS
         info = pyparse.CompileInfo("<test>", p_mode, flags)
-        tree = self.parser.parse_source(source, info)
-        ast_node = ast_from_node(self.space, tree, info, self.parser)
+        self.info = info
+        ast_node = self.parser.parse_source(source, info)
+        ast_node.to_object(self.space) # does not crash
         return ast_node
 
     def get_first_expr(self, source, p_mode=None, flags=None):
@@ -67,6 +71,9 @@ class TestAstBuilder:
         for stmt in mod.body:
             assert isinstance(stmt, ast.Assign)
 
+    def test_constant_kind_bug(self):
+        d = self.get_first_expr("None") # used to crash in to_object
+
     def test_del(self):
         d = self.get_first_stmt("del x")
         assert isinstance(d, ast.Delete)
@@ -195,8 +202,8 @@ class TestAstBuilder:
         input = "from x import a, b,"
         with pytest.raises(SyntaxError) as excinfo:
             self.get_ast(input)
-        assert excinfo.value.msg == "trailing comma is only allowed with surronding " \
-            "parenthesis"
+        assert excinfo.value.msg == "trailing comma not allowed without surrounding " \
+            "parentheses"
         assert excinfo.value.text == input + "\n"
 
     def test_global(self):
@@ -568,7 +575,7 @@ class TestAstBuilder:
         assert isinstance(fn.args.kw_defaults[0], ast.Constant)
         input = "def f(p1, *, **k1):  pass"
         exc = pytest.raises(SyntaxError, self.get_ast, input).value
-        assert exc.msg == "named arguments must follows bare *"
+        assert exc.msg == "named arguments must follow bare *"
 
     def test_posonly_arguments(self):
         fn = self.get_first_stmt("def f(a, b, c, /, arg): pass")
@@ -869,12 +876,16 @@ class TestAstBuilder:
                 input = template % (expr,)
                 with pytest.raises(SyntaxError) as excinfo:
                     self.get_ast(input)
-                assert excinfo.value.msg == "cannot %s %s" % (ctx_type, type_str)
+                assert excinfo.value.msg.startswith("cannot %s %s" % (ctx_type, type_str))
 
     def test_assignment_to_forbidden_names(self):
         invalid = (
             "%s = x",
             "%s, x = y",
+            "[%s, x] = y",
+            "[%s, x] = y",
+            "*%s, x = y",
+            "[*%s, x] = y",
             "def %s(): pass",
             "class %s(): pass",
             "def f(%s): pass",
@@ -888,13 +899,65 @@ class TestAstBuilder:
             "from x import %s",
             "from x import y as %s",
             "for %s in x: pass",
+            "x.%s = y",
+            "x.%s += y",
+            "%s += 1",
         )
         for name in "__debug__",:
             for template in invalid:
                 input = template % (name,)
-                with pytest.raises(SyntaxError) as excinfo:
-                    self.get_ast(input)
-                assert excinfo.value.msg == "cannot assign to %s" % (name,)
+                ast = self.get_ast(input) # error now caught during codegen!
+                ec = self.space.getexecutioncontext()
+                with pytest.raises(OperationError) as excinfo:
+                    ec.compiler.compile_ast(ast, "", "exec")
+                msg = self.space.text_w(self.space.repr(excinfo.value.get_w_value(self.space)))
+                assert ("cannot assign to %s" % (name,)) in msg
+
+    def test_delete_forbidden_name(self):
+        invalid = (
+            "del %s",
+            "del %s, a",
+            "del [%s, a]",
+            "del a.%s",
+        )
+        for name in "__debug__",:
+            for template in invalid:
+                input = template % (name,)
+                ast = self.get_ast(input) # error now caught during codegen!
+                ec = self.space.getexecutioncontext()
+                with pytest.raises(OperationError) as excinfo:
+                    ec.compiler.compile_ast(ast, "", "exec")
+                msg = self.space.text_w(self.space.repr(excinfo.value.get_w_value(self.space)))
+                assert ("cannot delete %s" % (name,)) in msg
+
+    def test_cannot_delete_messages(self):
+        invalid = [
+            ("del (1, x)", "cannot delete literal"),
+            ("del [x, z, a, {1, 2}]", "cannot delete set display"),
+            ("del [a, (b, *c)]", "cannot delete starred"),
+            ("del (a := 5)", "cannot delete named expression"),
+        ]
+        for wrong, msg in invalid:
+            with pytest.raises(SyntaxError) as excinfo:
+                self.get_ast(wrong)
+            assert msg in excinfo.value.msg
+
+    def test_cannot_assign_messages(self):
+        invalid = [
+            ("(1, x) = 5", "cannot assign to literal"),
+            ("[x, z, a, {1, 2}] = 12", "cannot assign to set display"),
+            ("for (1, x) in []: pass", "cannot assign to literal"),
+            ("with foo as (1, 2): pass", "cannot assign to literal"),
+            ("(a, *True, c) = (1, 2, 3)", "cannot assign to True"),
+            ("for (x, *(y, z.d())) in b: pass", "cannot assign to function call"),
+        ]
+        for wrong, msg in invalid:
+            with pytest.raises(SyntaxError) as excinfo:
+                self.get_ast(wrong)
+            assert msg in excinfo.value.msg
+
+    def test_assign_bug(self):
+        self.get_ast("direct = (__debug__ and optimize == 0)") # used to crash
 
     def test_lambda(self):
         lam = self.get_first_expr("lambda x: expr")
@@ -1100,26 +1163,26 @@ class TestAstBuilder:
         assert isinstance(call.args[0], ast.GeneratorExp)
         input = "f(x for x in y, 1)"
         exc = pytest.raises(SyntaxError, self.get_ast, input).value
-        assert exc.msg == "Generator expression must be parenthesized if not " \
-            "sole argument"
+        assert exc.msg == "Generator expression must be parenthesized"
+
         input = "f(x for x in y, )"
         exc = pytest.raises(SyntaxError, self.get_ast, input).value
-        assert exc.msg == "Generator expression must be parenthesized if not " \
-            "sole argument"
+        assert exc.msg == "Generator expression must be parenthesized"
+
         many_args = ", ".join("x%i" % i for i in range(256))
         input = "f(%s)" % (many_args,)
         self.get_ast(input) # doesn't crash any more
         exc = pytest.raises(SyntaxError, self.get_ast, "f((a+b)=c)").value
         assert exc.msg == 'expression cannot contain assignment, perhaps you meant "=="?'
-        exc = pytest.raises(SyntaxError, self.get_ast, "f(a=c, a=d)").value
-        assert exc.msg == "keyword argument repeated: 'a'"
+        exc = pytest.raises(SyntaxError, self.get_ast, "deff f(a=1, a=2): pass").value
+        assert  'invalid syntax' in exc.msg # used to be "keyword argument repeated: 'a'"
         with pytest.raises(SyntaxError) as excinfo:
             self.get_ast("f((x)=1)")
         assert excinfo.value.msg == 'expression cannot contain assignment, perhaps you meant "=="?'
         with pytest.raises(SyntaxError) as excinfo:
             self.get_ast("f(True=1)")
         assert excinfo.value.msg == 'cannot assign to True'
-        assert excinfo.value.offset == 2
+        assert excinfo.value.offset == 3
 
 
     def test_attribute(self):
@@ -1143,8 +1206,7 @@ class TestAstBuilder:
         assert isinstance(sub.value, ast.Name)
         assert sub.value.ctx == ast.Load
         assert sub.ctx == ast.Load
-        assert isinstance(sub.slice, ast.Index)
-        assert isinstance(sub.slice.value, ast.Name)
+        assert isinstance(sub.slice, ast.Name)
         slc = self.get_first_expr("x[:]").slice
         assert slc.upper is None
         assert slc.lower is None
@@ -1194,14 +1256,13 @@ class TestAstBuilder:
             assert isinstance(field, ast.Constant)
         sub = self.get_first_expr("x[1,2,3]")
         slc = sub.slice
-        assert isinstance(slc, ast.Index)
-        assert isinstance(slc.value, ast.Tuple)
-        assert len(slc.value.elts) == 3
-        assert slc.value.ctx == ast.Load
+        assert isinstance(slc, ast.Tuple)
+        assert len(slc.elts) == 3
+        assert slc.ctx == ast.Load
         slc = self.get_first_expr("x[1,3:4]").slice
-        assert isinstance(slc, ast.ExtSlice)
-        assert len(slc.dims) == 2
-        complex_slc = slc.dims[1]
+        assert isinstance(slc, ast.Tuple)
+        assert len(slc.elts) == 2
+        complex_slc = slc.elts[1]
         assert isinstance(complex_slc, ast.Slice)
         assert isinstance(complex_slc.lower, ast.Constant)
         assert isinstance(complex_slc.upper, ast.Constant)
@@ -1213,8 +1274,8 @@ class TestAstBuilder:
         assert self.space.is_w(e.value, self.space.w_Ellipsis)
 
         sub = self.get_first_expr("x[...]")
-        assert isinstance(sub.slice.value, ast.Constant)
-        assert self.space.is_w(sub.slice.value.value, self.space.w_Ellipsis)
+        assert isinstance(sub.slice, ast.Constant)
+        assert self.space.is_w(sub.slice.value, self.space.w_Ellipsis)
 
     def test_string(self):
         space = self.space
@@ -1227,26 +1288,25 @@ class TestAstBuilder:
         s = self.get_first_expr("b'hi' b' implicitly' b' extra'")
         assert isinstance(s, ast.Constant)
         assert space.eq_w(s.value, space.newbytes("hi implicitly extra"))
-        pytest.raises(SyntaxError, self.get_first_expr, "b'hello' 'world'")
+        excinfo = pytest.raises(SyntaxError, self.get_first_expr, "b'hello' 'world'")
+        assert excinfo.value.offset == 1
+        excinfo = pytest.raises(SyntaxError, self.get_first_expr, "'foo' b'hello' 'world'")
+        assert excinfo.value.offset == 7
         sentence = u"Die Männer ärgern sich!"
         source = u"# coding: utf-7\nstuff = '%s'" % (sentence,)
-        info = pyparse.CompileInfo("<test>", "exec")
-        tree = self.parser.parse_source(source.encode("utf-7"), info)
-        assert info.encoding == "utf-7"
-        s = ast_from_node(space, tree, info).body[0].value
-        assert isinstance(s, ast.Constant)
-        assert space.eq_w(s.value, space.wrap(sentence))
+        s = self.get_first_stmt(source.encode("utf-7"))
+        assert self.info.encoding == "utf-7"
+        assert isinstance(s.value, ast.Constant)
+        assert space.eq_w(s.value.value, space.wrap(sentence))
 
     def test_string_pep3120(self):
         space = self.space
         japan = u'日本'
         source = u"foo = '%s'" % japan
-        info = pyparse.CompileInfo("<test>", "exec")
-        tree = self.parser.parse_source(source.encode("utf-8"), info)
-        assert info.encoding == "utf-8"
-        s = ast_from_node(space, tree, info).body[0].value
-        assert isinstance(s, ast.Constant)
-        assert space.eq_w(s.value, space.wrap(japan))
+        s = self.get_first_stmt(source.encode("utf-8"))
+        assert self.info.encoding == "utf-8"
+        assert isinstance(s.value, ast.Constant)
+        assert space.eq_w(s.value.value, space.wrap(japan))
 
     def test_name_pep3131(self):
         assign = self.get_first_stmt("日本 = 32")
@@ -1276,19 +1336,16 @@ class TestAstBuilder:
         space = self.space
         source = u'# coding: Latin-1\nu = "Ç"\n'
         info = pyparse.CompileInfo("<test>", "exec")
-        tree = self.parser.parse_source(source.encode("Latin-1"), info)
-        assert info.encoding == "iso-8859-1"
-        s = ast_from_node(space, tree, info).body[0].value
+        s = self.get_first_stmt(source.encode("Latin-1")).value
+        assert self.info.encoding == "iso-8859-1"
         assert isinstance(s, ast.Constant)
         assert space.eq_w(s.value, space.wrap(u'Ç'))
 
     def test_string_bug(self):
         space = self.space
         source = '# -*- encoding: utf8 -*-\nstuff = "x \xc3\xa9 \\n"\n'
-        info = pyparse.CompileInfo("<test>", "exec")
-        tree = self.parser.parse_source(source, info)
-        assert info.encoding == "utf8"
-        s = ast_from_node(space, tree, info).body[0].value
+        s = self.get_first_stmt(source).value
+        assert self.info.encoding == "utf8"
         assert isinstance(s, ast.Constant)
         assert space.eq_w(s.value, space.wrap(u'x \xe9 \n'))
 
@@ -1435,7 +1492,7 @@ class TestAstBuilder:
         assert isinstance(expr.left, ast.Name)
         assert isinstance(expr.right, ast.Name)
         # imatmul is tested earlier search for @=
-    
+
     @pytest.mark.parametrize('with_async_hacks', [False, True])
     def test_asyncFunctionDef(self, with_async_hacks):
         mod = self.get_ast("async def f():\n await something()", with_async_hacks=with_async_hacks)
@@ -1456,7 +1513,7 @@ class TestAstBuilder:
         assert isinstance(func, ast.Name)
         assert func.id == 'something'
         assert func.ctx == ast.Load
-    
+
     @pytest.mark.parametrize('with_async_hacks', [False, True])
     def test_asyncFor(self, with_async_hacks):
         mod = self.get_ast("async def f():\n async for e in i: 1\n else: 2", with_async_hacks=with_async_hacks)
@@ -1477,7 +1534,7 @@ class TestAstBuilder:
         assert len(asyncfor.orelse) == 1
         assert isinstance(asyncfor.orelse[0], ast.Expr)
         assert isinstance(asyncfor.orelse[0].value, ast.Constant)
-    
+
     @pytest.mark.parametrize('with_async_hacks', [False, True])
     def test_asyncWith(self, with_async_hacks):
         mod = self.get_ast("async def f():\n async with a as b: 1", with_async_hacks=with_async_hacks)
@@ -1562,7 +1619,7 @@ class TestAstBuilder:
         assert exc.msg == ("(unicode error) 'unicodeescape' codec can't decode"
                            " bytes in position 0-1: truncated \\xXX escape")
         assert exc.lineno == 2
-        assert exc.offset == 6
+        assert exc.offset == 7
 
     def test_fstring_lineno(self):
         mod = self.get_ast('x=1\nf"{    x + 1}"')
@@ -1785,7 +1842,7 @@ class TestAstBuilder:
         for function in module.body:
             args = function.args
             all_args = []
-            all_args.extend(args.args)
+            all_args.extend(args.args or [])
             all_args.extend(args.kwonlyargs or [])
             if args.vararg:
                 all_args.append(args.vararg)
@@ -1831,7 +1888,7 @@ class TestAstBuilder:
 
     def test_func_type(self):
         func = self.get_ast("() -> int", p_mode="func_type")
-        assert len(func.argtypes) == 0
+        assert not func.argtypes
         assert isinstance(func.returns, ast.Name)
         assert func.returns.id == 'int'
 
@@ -1893,8 +1950,8 @@ class TestAstBuilder:
         assert tree.end_col_offset == len(s)
         assert tree.col_offset == 0
         gen = tree.args[0]
-        assert gen.end_col_offset == len(s) - 1
-        assert gen.col_offset == 2
+        assert gen.end_col_offset in (len(s), len(s) - 1)
+        assert gen.col_offset in (1, 2)
         assert fdef.get_source_segment(s) == s
 
         s = "(x for x in y)"
@@ -1981,4 +2038,10 @@ class TestAstBuilder:
             tree = self.get_ast("f'{((}')")
         assert excinfo.value.msg == "unmatched ')'"
 
+    def test_keyword_position(self):
+        tree = self.get_first_expr("f(a=1, **kwarg)")
+        assert tree.keywords[0].col_offset == 2
+        assert tree.keywords[0].end_col_offset == 5
+        assert tree.keywords[1].col_offset == 7
+        assert tree.keywords[1].end_col_offset == 14
 
