@@ -4,7 +4,7 @@ import py
 from rpython.rlib.cache import Cache
 from rpython.tool.uid import HUGEVAL_BYTES
 from rpython.rlib import jit, types, rutf8
-from rpython.rlib.debug import make_sure_not_resized
+from rpython.rlib.debug import make_sure_not_resized, check_not_access_directly
 from rpython.rlib.objectmodel import (we_are_translated, newlist_hint,
      compute_unique_id, specialize, not_rpython)
 from rpython.rlib.signature import signature
@@ -72,7 +72,11 @@ class W_Root(object):
 
     # to be used directly only by space.type implementations
     def getclass(self, space):
-        return space.gettypeobject(self.typedef)
+        # make sure that we never annotate this with access_directly set to
+        # True. otherwise every call to getclass (and other methods) has an
+        # extra indirection due to a much more complicated function set
+        check_not_access_directly(self)
+        return space.gettypefor(self.__class__)
 
     def setclass(self, space, w_subtype):
         raise oefmt(space.w_TypeError,
@@ -946,28 +950,16 @@ class ObjSpace(object):
             return None
         if not isinstance(w_obj, RequiredClass):   # or obj is None
             raise oefmt(self.w_TypeError,
-                        "'%s' object expected, got '%N' instead",
+                        "'%s' object expected, got '%T' instead",
                         wrappable_class_name(RequiredClass),
-                        w_obj.getclass(self))
+                        w_obj)
         return w_obj
 
-    def _iter_unpackiterable(self, w_iterable):
-        try:
-            return self.iter(w_iterable)
-        except OperationError as e:
-            if e.got_any_traceback():
-                raise
-            if not e.match(self, self.w_TypeError):
-                raise
-            raise oefmt(self.w_TypeError,
-                    "cannot unpack non-iterable %T object",
-                    w_iterable)
-
     def unpackiterable(self, w_iterable, expected_length=-1):
         """Unpack an iterable into a real (interpreter-level) list.
 
         Raise an OperationError(w_ValueError) if the length is wrong."""
-        w_iterator = self._iter_unpackiterable(w_iterable)
+        w_iterator = self.iter(w_iterable)
         if expected_length == -1:
             if self.is_generator(w_iterator):
                 # special hack for speed
@@ -1039,7 +1031,7 @@ class ObjSpace(object):
         # Like unpackiterable(), but for the cases where we have
         # an expected_length and want to unroll when JITted.
         # Returns a fixed-size list.
-        w_iterator = self._iter_unpackiterable(w_iterable)
+        w_iterator = self.iter(w_iterable)
         assert expected_length != -1
         return self._unpackiterable_known_length_jitlook(w_iterator,
                                                          expected_length)
