@@ -1,5 +1,5 @@
 from pypy.interpreter.error import OperationError
-from pypy.interpreter.pyparser import future, parser, pytokenizer, pygram, error
+from pypy.interpreter.pyparser import future, parser, pytokenizer, pygram, error, pytoken
 from pypy.interpreter.astcompiler import consts
 from pypy.module.sys.version import CPYTHON_VERSION
 from rpython.rlib import rstring
@@ -103,7 +103,7 @@ _targets = {
 
 class PythonParser(parser.Parser):
 
-    def __init__(self, space, future_flags=future.futureFlags_3_8,
+    def __init__(self, space, future_flags=future.futureFlags_3_9,
                  grammar=pygram.python_grammar):
         parser.Parser.__init__(self, grammar)
         self.space = space
@@ -119,7 +119,12 @@ class PythonParser(parser.Parser):
         Everything from decoding the source to tokenizing to building the parse
         tree is handled here.
         """
-        # Detect source encoding.
+        textsrc = self._handle_encoding(bytessrc, compile_info, self.space)
+        return self._parse(textsrc, compile_info)
+
+    @staticmethod
+    def _handle_encoding(bytessrc, compile_info, space):
+        # Detect source encoding. also updates compile_info.flags
         explicit_encoding = False
         enc = None
         if compile_info.flags & consts.PyCF_SOURCE_IS_UTF8:
@@ -143,12 +148,11 @@ class PythonParser(parser.Parser):
             if enc is None:
                 enc = 'utf-8'
             try:
-                textsrc = recode_to_utf8(self.space, bytessrc, enc)
+                textsrc = recode_to_utf8(space, bytessrc, enc)
             except OperationError as e:
                 # if the codec is not found, LookupError is raised.  we
                 # check using 'is_w' not to mask potential IndexError or
                 # KeyError
-                space = self.space
                 if e.match(space, space.w_LookupError):
                     raise error.SyntaxError("Unknown encoding: %s" % enc,
                                             filename=compile_info.filename)
@@ -162,7 +166,7 @@ class PythonParser(parser.Parser):
             compile_info.encoding = enc
         if explicit_encoding:
             compile_info.flags |= consts.PyCF_FOUND_ENCODING
-        return self._parse(textsrc, compile_info)
+        return textsrc
 
     def _get_possible_arcs(self, arcs):
         # Handle func_body_suite separately for expecting an IndentationError
@@ -213,6 +217,7 @@ class PythonParser(parser.Parser):
             try:
                 tokens_stream = iter(tokens)
 
+                token = None
                 for token in tokens_stream:
                     next_token_seen = token
                     # Special handling for TYPE_IGNOREs
@@ -225,22 +230,7 @@ class PythonParser(parser.Parser):
                 next_token_seen = None
 
                 if compile_info.mode == 'single':
-                    for token in tokens_stream:
-                        if token.token_type == pygram.tokens.ENDMARKER:
-                            break
-                        if token.token_type == pygram.tokens.NEWLINE:
-                            continue
-
-                        if token.token_type == pygram.tokens.COMMENT:
-                            for token in tokens_stream:
-                                if token.token_type == pygram.tokens.NEWLINE:
-                                    break
-                        else:
-                            new_err = error.SyntaxError
-                            msg = ("multiple statements found while "
-                                   "compiling a single statement")
-                            raise new_err(msg, token.lineno, token.column,
-                                          token.line, compile_info.filename)
+                    self._check_token_stream_single(compile_info, tokens_stream)
 
             except error.TokenError as e:
                 e.filename = compile_info.filename
@@ -252,7 +242,7 @@ class PythonParser(parser.Parser):
                 # Catch parse errors, pretty them up and reraise them as a
                 # SyntaxError.
                 new_err = error.IndentationError
-                if token.token_type == pygram.tokens.INDENT:
+                if e.token.token_type == pygram.tokens.INDENT:
                     msg = "unexpected indent"
                 elif e.expected == pygram.tokens.INDENT:
                     # hack a bit to find the line that starts the block. should
@@ -282,6 +272,25 @@ class PythonParser(parser.Parser):
             self.root = None
         return tree
 
+    @staticmethod
+    def _check_token_stream_single(compile_info, tokens_stream):
+        for token in tokens_stream:
+            if token.token_type == pygram.tokens.ENDMARKER:
+                break
+            if token.token_type == pygram.tokens.NEWLINE:
+                continue
+
+            if token.token_type == pygram.tokens.COMMENT:
+                for token in tokens_stream:
+                    if token.token_type == pygram.tokens.NEWLINE:
+                        break
+            else:
+                new_err = error.SyntaxError
+                msg = ("multiple statements found while "
+                       "compiling a single statement")
+                raise new_err(msg, token.lineno, token.column,
+                              token.line, compile_info.filename)
+
 SUITE_STARTERS = dict.fromkeys("def if elif else while for try except finally with class".split())
 
 def _compute_indentation_error_msg(stack):
@@ -306,3 +315,82 @@ def _compute_indentation_error_msg(stack):
                     return "expected an indented block after '%s' statement on line %s" % (
                             child.value, child.lineno)
     return "expected an indented block"
+
+
+class PegParser(object):
+    def __init__(self, space, future_flags=future.futureFlags_3_9):
+        self.space = space
+        self.future_flags = future_flags
+        self.type_ignores = []
+
+    def reset(self):
+        pass
+
+    def parse_source(self, bytessrc, compile_info):
+        """Main entry point for parsing Python source.
+
+        Everything from decoding the source to tokenizing to building the parse
+        tree is handled here.
+        """
+        textsrc = PythonParser._handle_encoding(bytessrc, compile_info, self.space)
+        return self._parse(textsrc, compile_info)
+
+    def _parse(self, textsrc, compile_info):
+        from pypy.interpreter.pyparser.rpypegparse import PythonParser
+        # XXX too much copy-paste
+        flags = compile_info.flags
+
+        # The tokenizer is very picky about how it wants its input.
+        source_lines = textsrc.splitlines(True)
+        if source_lines and not source_lines[-1].endswith("\n"):
+            source_lines[-1] += '\n'
+        if textsrc and textsrc[-1] == "\n" or compile_info.mode != "single":
+            flags &= ~consts.PyCF_DONT_IMPLY_DEDENT
+
+        try:
+            # Note: we no longer pass the CO_FUTURE_* to the tokenizer,
+            # which is expected to work independently of them.  It's
+            # certainly the case for all futures in Python <= 2.7.
+            tokens = pytokenizer.generate_tokens(source_lines, flags)
+        except error.TokenError as e:
+            e.filename = compile_info.filename
+            raise
+        except error.TokenIndentationError as e:
+            e.filename = compile_info.filename
+            raise
+
+
+        newflags, last_future_import = (
+            future.add_future_flags(self.future_flags, tokens))
+        compile_info.last_future_import = last_future_import
+        compile_info.flags |= newflags
+
+        mode = compile_info.mode
+        if mode != "single":
+            assert tokens[-2].token_type == pytoken.python_tokens['NEWLINE']
+            del tokens[-2]
+        pp = PythonParser(self.space, tokens, compile_info)
+        try:
+            for token in tokens:
+                # Special handling for TYPE_IGNOREs
+                if token.token_type == pygram.tokens.TYPE_IGNORE:
+                    self.type_ignores.append(token)
+            if mode == "exec":
+                meth = PythonParser.file
+            elif mode == "single":
+                meth = PythonParser.interactive
+            elif mode == "eval":
+                meth = PythonParser.eval
+            elif mode == "func_type":
+                meth = PythonParser.func_type
+            else:
+                assert 0, "unknown mode"
+            return pp.parse_meth_or_raise(meth)
+        except error.TokenError as e:
+            e.filename = compile_info.filename
+            raise
+        except error.TokenIndentationError as e:
+            e.filename = compile_info.filename
+            raise
+
+
