@@ -7,7 +7,7 @@ from pypy.interpreter.pyparser.error import SyntaxError, IndentationError, TabEr
 from pypy.interpreter.astcompiler import consts
 
 
-class TestPythonParser:
+class TestPythonParserOnly: # not subclassed for Peg
     spaceconfig = {}
 
     def setup_class(self):
@@ -18,26 +18,11 @@ class TestPythonParser:
             info = pyparse.CompileInfo("<test>", mode, flags=flags)
         return self.parser.parse_source(source, info)
 
-    def test_with_and_as(self):
-        py.test.raises(SyntaxError, self.parse, "with = 23")
-        py.test.raises(SyntaxError, self.parse, "as = 2")
-
-    def test_dont_imply_dedent(self):
-        info = pyparse.CompileInfo("<test>", "single",
-                                   consts.PyCF_DONT_IMPLY_DEDENT)
-        self.parse('if 1:\n  x\n', info=info)
-        self.parse('x = 5 ', info=info)
-
     def test_clear_state(self):
         assert self.parser.root is None
         tree = self.parse("name = 32")
         assert self.parser.root is None
 
-    def test_encoding_pep3120(self):
-        info = pyparse.CompileInfo("<test>", "exec")
-        tree = self.parse("""foo = '日本'""", info=info)
-        assert info.encoding == 'utf-8'
-
     def test_encoding(self):
         info = pyparse.CompileInfo("<test>", "exec")
         tree = self.parse("""# coding: latin-1
@@ -68,6 +53,54 @@ stuff = "nothing"
         assert exc.msg == ("'ascii' codec can't decode byte 0xc3 "
                            "in position 16: ordinal not in range(128)")
 
+    def test_mode(self):
+        assert self.parse("x = 43*54").type == syms.file_input
+        tree = self.parse("43**54", "eval")
+        assert tree.type == syms.eval_input
+        py.test.raises(SyntaxError, self.parse, "x = 54", "eval")
+        tree = self.parse("x = 43", "single")
+        assert tree.type == syms.single_input
+
+    def test_universal_newlines(self):
+        fmt = 'stuff = """hello%sworld"""'
+        expected_tree = self.parse(fmt % '\n')
+        for linefeed in ["\r\n","\r"]:
+            tree = self.parse(fmt % linefeed)
+            assert expected_tree == tree
+
+    def test_end_positions(self):
+        tree = self.parse("45 * a", "eval").get_child(0)
+        assert tree.get_end_column() == 6
+
+
+class TestPythonParser:
+    spaceconfig = {}
+
+    def setup_class(self):
+        self.parser = pyparse.PythonParser(self.space)
+
+    def parse(self, source, mode="exec", info=None, flags=0):
+        if info is None:
+            info = pyparse.CompileInfo("<test>", mode, flags=flags)
+        return self.parser.parse_source(source, info)
+
+    def test_with_and_as(self):
+        py.test.raises(SyntaxError, self.parse, "with = 23")
+        py.test.raises(SyntaxError, self.parse, "as = 2")
+
+    def test_dont_imply_dedent(self):
+        info = pyparse.CompileInfo("<test>", "single",
+                                   consts.PyCF_DONT_IMPLY_DEDENT)
+        self.parse('if 1:\n  x\n', info=info)
+        self.parse('x = 5 ', info=info)
+        py.test.raises(SyntaxError, self.parse, "if 1:\n  x", info=info)
+        excinfo = py.test.raises(SyntaxError, self.parse, "if 1:\n  x x\n", info=info)
+
+    def test_encoding_pep3120(self):
+        info = pyparse.CompileInfo("<test>", "exec")
+        tree = self.parse("""foo = '日本'""", info=info)
+        assert info.encoding == 'utf-8'
+
     def test_unicode_identifier(self):
         tree = self.parse("a日本 = 32")
         tree = self.parse("日本 = 32")
@@ -75,9 +108,9 @@ stuff = "nothing"
     def test_syntax_error(self):
         parse = self.parse
         exc = py.test.raises(SyntaxError, parse, "name another for").value
-        assert exc.msg == "invalid syntax"
+        assert exc.msg.startswith("invalid syntax")
         assert exc.lineno == 1
-        assert exc.offset == 6
+        assert exc.offset in (1, 6)
         assert exc.text.startswith("name another for")
         exc = py.test.raises(SyntaxError, parse, "x = \"blah\n\n\n").value
         assert exc.msg == "end of line (EOL) while scanning string literal"
@@ -99,6 +132,8 @@ stuff = "nothing"
         assert exc.msg == "unmatched ')'"
         assert exc.lineno == 1
         assert exc.offset == 4
+        exc = py.test.raises(SyntaxError, parse, "\\").value
+        assert exc.msg == "end of file (EOF) in multi-line statement"
 
     def test_is(self):
         self.parse("x is y")
@@ -160,14 +195,6 @@ if 1:
     def test_mac_newline(self):
         self.parse("this_is\ra_mac\rfile")
 
-    def test_mode(self):
-        assert self.parse("x = 43*54").type == syms.file_input
-        tree = self.parse("43**54", "eval")
-        assert tree.type == syms.eval_input
-        py.test.raises(SyntaxError, self.parse, "x = 54", "eval")
-        tree = self.parse("x = 43", "single")
-        assert tree.type == syms.single_input
-
     def test_multiline_string(self):
         self.parse("''' \n '''")
         self.parse("r''' \n '''")
@@ -194,13 +221,6 @@ if 1:
     def test_print_function(self):
         self.parse("from __future__ import print_function\nx = print\n")
 
-    def test_universal_newlines(self):
-        fmt = 'stuff = """hello%sworld"""'
-        expected_tree = self.parse(fmt % '\n')
-        for linefeed in ["\r\n","\r"]:
-            tree = self.parse(fmt % linefeed)
-            assert expected_tree == tree
-
     def test_revdb_dollar_num(self):
         assert not self.space.config.translation.reverse_debugger
         py.test.raises(SyntaxError, self.parse, '$0')
@@ -384,9 +404,8 @@ if 1:
             print x
             py.test.raises(SyntaxError, self.parse, "x = %s" % x)
 
-    def test_end_positions(self):
-        tree = self.parse("45 * a", "eval").get_child(0)
-        assert tree.get_end_column() == 6
+    def test_relaxed_decorators(self):
+        self.parse("@(1 + 2)\ndef f(x): pass") # does not crash
 
 
 
@@ -459,6 +478,12 @@ stuff = "nothing"
         info = py.test.raises(SyntaxError, self.parse, "print 1)")
         assert "unmatched" in info.value.msg
 
+    def test_error_exec_without_parens_bug(self):
+        info = py.test.raises(SyntaxError, self.parse, "exec {1:(foo.)}")
+        assert "Missing parentheses in call to 'exec'" in info.value.msg
+        assert info.value.offset == 6
+
+
 class TestPythonParserRevDB(TestPythonParser):
     spaceconfig = {"translation.reverse_debugger": True}
 
@@ -473,3 +498,42 @@ class TestPythonParserRevDB(TestPythonParser):
         py.test.raises(SyntaxError, self.parse, '$.5')
 
 
+class TestPythonPegParser(TestPythonParser):
+    spaceconfig = {}
+
+    def setup_class(self):
+        self.parser = pyparse.PegParser(self.space)
+
+    def test_crash_with(self):
+        # used to crash
+        py.test.raises(SyntaxError, self.parse,
+                "async with a:\n    pass", "single",
+                flags=consts.PyCF_DONT_IMPLY_DEDENT | consts.PyCF_ALLOW_TOP_LEVEL_AWAIT)
+
+    def test_crash_eval_empty(self):
+        # used to crash
+        py.test.raises(SyntaxError, self.parse,
+                       '', 'eval', flags=consts.PyCF_DONT_IMPLY_DEDENT)
+
+
+    def test_dont_imply_dedent_ignored_on_exec(self):
+        self.parse(
+            "if 1: \n pass", "exec",
+            flags=consts.PyCF_DONT_IMPLY_DEDENT)
+
+    def test_nonparen_genexp_in_call(self):
+        with py.test.raises(SyntaxError) as info:
+            self.parse("""\
+f(x for x in l)
+if 1:
+pass
+""")
+        assert info.value.msg == "expected an indented block after 'if' statement on line 2"
+        with py.test.raises(SyntaxError) as info:
+            self.parse("""f(1, x for x in y for z in a if
+
+b)""")
+        assert info.value.msg == 'Generator expression must be parenthesized'
+        assert info.value.lineno == 1
+        assert info.value.offset == 6
+        assert info.value.lastlineno == 3
