@@ -672,6 +672,14 @@ def test_method_identity():
     assert x is B.m
     assert id(x) == id(B.m)
 
+def test_method_ne_NotImplemented():
+    class A(object):
+        def __ne__(self, other):
+            return "ABC"
+    meth = A.__ne__
+    assert meth.__ne__(1) is NotImplemented
+    assert (meth != A()) == "ABC"
+
 def test_posonly():
     def posonlyfunc(a, b, c, /, d):
         return (a, b, c, d)
@@ -700,10 +708,74 @@ def global_inner_has_pos_only():
 def test_posonly_annotations_crash():
     assert global_inner_has_pos_only().__annotations__ == {"x": int}
 
+def test_classmethod_of_random_callable():
+    class Callable:
+        def __call__(self, cls):
+            print(cls)
+            assert cls is Class
+            return "foo"
+    class Class:
+        f = classmethod(Callable())
+    assert Class().f() == "foo"
+
+
+def test_classmethod_of_other_descriptor():
+    class BoundWrapper:
+        def __init__(self, wrapped):
+            self.__wrapped__ = wrapped
+
+        def __call__(self, *args, **kwargs):
+            return self.__wrapped__(*args, **kwargs)
+
+    class Wrapper:
+        def __init__(self, wrapped):
+            self.__wrapped__ = wrapped
+
+        def __get__(self, instance, owner):
+            bound_function = self.__wrapped__.__get__(instance, owner)
+            return BoundWrapper(bound_function)
+
+    def decorator(wrapped):
+        return Wrapper(wrapped)
+
+    class Class:
+        @decorator
+        @classmethod
+        def inner(cls):
+            # This should already work.
+            assert cls is Class
+            return 'spam'
+
+        @classmethod
+        @decorator
+        def outer(cls):
+            # Raised TypeError with a message saying that the 'Wrapper'
+            # object is not callable.
+            assert cls is Class
+            return 'eggs'
+
+    assert Class.inner() == 'spam'
+    assert Class.outer() == 'eggs'
+    assert Class().inner() == 'spam'
+    assert Class().outer() == 'eggs'
+
 def test_duplicate_key_kwargs():
     def f(**d): pass
     class A:
         def keys(self): return ['a', 'a', 'b']
+        def items(self): return [('a', None), ('a', None), ('b', None)]
         def __getitem__(self, key): 1
+        def __len__(self): return 3
     with pytest.raises(TypeError):
         f(**A())
+
+def test_unpack_dict_without_length():
+    def f(**d): return d
+    class A:
+        def keys(self): return ['a']
+        def items(self): return [('a', 1)]
+        def __getitem__(self, key): return 1
+    assert f(**A()) == {'a': 1}
+    assert f(b=2, **A()) == {'b': 2, 'a': 1}
+    assert f(**A(), b=2) == {'b': 2, 'a': 1}
+
