@@ -613,13 +613,14 @@ class AppTestSyntaxError:
             exec("""if 1:
                 class Foo:
                     bla
-                    a b c d e
+                    a as e
                     bar
             """)
         except SyntaxError as e:
             assert e.lineno == 4
-            assert e.text.endswith('a b c d e\n')
-            assert e.offset == e.text.index('b') + 1 # offset is 1-based
+            assert e.text.endswith('a as e\n')
+            print(e.offset, e.text.index('as'))
+            assert e.offset == e.text.index('as') + 1 # offset is 1-based
         else:
             raise Exception("no SyntaxError??")
 
