@@ -6,7 +6,7 @@ class TestTargetPyPy(object):
     def test_run(self):
         config = get_pypy_config(translating=False)
         entry_point = get_entry_point(config)[0]
-        entry_point(['pypy3-c' , '-S', '-c', 'print 3'])
+        entry_point(['pypy3-c' , '-S', '-c', 'print(3)'])
 
 def test_execute_source(space):
     _, d = create_entry_point(space, None)
