@@ -1,3 +1,4 @@
+# encoding: utf-8
 import pytest
 
 from pypy.interpreter.unicodehelper import (
@@ -6,6 +7,8 @@ from pypy.interpreter.unicodehelper import (
 
 from pypy.interpreter.unicodehelper import str_decode_utf8, utf8_encode_latin_1
 from pypy.interpreter.unicodehelper import utf8_encode_ascii, str_decode_ascii
+from pypy.interpreter.unicodehelper import utf8_encode_latin_1, str_decode_unicode_escape
+from pypy.interpreter.unicodehelper import str_decode_raw_unicode_escape
 from pypy.interpreter import unicodehelper as uh
 from pypy.module._codecs.interp_codecs import CodecState
 
@@ -80,3 +83,33 @@ def test_latin1_shortcut_bug(space):
 
     sin = u"a\xac\u1234\u20ac\u8000"
     assert utf8_encode_latin_1(sin.encode("utf-8"), "backslashreplace", handler) == sin.encode("latin-1", "backslashreplace")
+
+def test_unicode_escape_incremental_bug():
+    class FakeUnicodeDataHandler:
+        def call(self, name):
+            assert name == "QUESTION MARK"
+            return ord("?")
+    unicodedata_handler = FakeUnicodeDataHandler()
+    input = u"äҰ𐀂?"
+    data = b'\\xe4\\u04b0\\U00010002\\N{QUESTION MARK}'
+    for i in range(1, len(data)):
+        result1, _, lgt1, _ = str_decode_unicode_escape(data[:i], 'strict', False, None, unicodedata_handler)
+        result2, _, lgt2, _ = str_decode_unicode_escape(data[lgt1:i] + data[i:], 'strict', True, None, unicodedata_handler)
+        assert lgt1 + lgt2 == len(data)
+        assert input == (result1 + result2).decode("utf-8")
+
+def test_raw_unicode_escape_incremental_bug():
+    input = u"xҰa𐀂"
+    data = b'x\\u04b0a\\U00010002'
+    for i in range(1, len(data)):
+        result1, _, lgt1 = str_decode_raw_unicode_escape(data[:i], 'strict', False, None)
+        result2, _, lgt2 = str_decode_raw_unicode_escape(data[lgt1:i] + data[i:], 'strict', True, None)
+        assert lgt1 + lgt2 == len(data)
+        assert input == (result1 + result2).decode("utf-8")
+
+def test_raw_unicode_escape_backslash_without_escape():
+    data = b'[:/?#[\\]@]\\'
+    result, _, l = str_decode_raw_unicode_escape(data, 'strict', True, None)
+    assert l == len(data)
+    assert result == data
+
