@@ -7,10 +7,19 @@ import subprocess
 
 pypypath = py.path.local(pypy.__file__).dirpath("bin", "pyinteractive.py")
 
-def run(*args):
+def run(*args, **kwds):
+    stdin = kwds.pop('stdin', '')
+    assert not kwds
     argslist = map(str, args)
-    popen = subprocess.Popen(argslist, stdout=subprocess.PIPE)
-    stdout, stderr = popen.communicate()
+    popen = subprocess.Popen(argslist, stdin=subprocess.PIPE,
+                                       stdout=subprocess.PIPE)
+    stdout, stderr = popen.communicate(stdin)
+    print('--- stdout ---')
+    print(stdout)
+    print()
+    print('--- stderr ---')
+    print(stderr)
+    print()
     return stdout
 
 
@@ -18,19 +27,19 @@ def test_executable():
     """Ensures sys.executable points to the py.py script"""
     # TODO : watch out for spaces/special chars in pypypath
     output = run(sys.executable, pypypath, '-S',
-                 "-c", "import sys;print sys.executable")
+                 "-c", "import sys;print(sys.executable)")
     assert output.splitlines()[-1] == pypypath
 
 def test_special_names():
     """Test the __name__ and __file__ special global names"""
-    cmd = "print __name__; print '__file__' in globals()"
+    cmd = "print(__name__); print('__file__' in globals())"
     output = run(sys.executable, pypypath, '-S', '-c', cmd)
     assert output.splitlines()[-2] == '__main__'
     assert output.splitlines()[-1] == 'False'
 
     tmpfilepath = str(udir.join("test_py_script_1.py"))
     tmpfile = file( tmpfilepath, "w" )
-    tmpfile.write("print __name__; print __file__\n")
+    tmpfile.write("print(__name__); print(__file__)\n")
     tmpfile.close()
 
     output = run(sys.executable, pypypath, '-S', tmpfilepath)
@@ -41,22 +50,22 @@ def test_argv_command():
     """Some tests on argv"""
     # test 1 : no arguments
     output = run(sys.executable, pypypath, '-S',
-                 "-c", "import sys;print sys.argv")
+                 "-c", "import sys;print(sys.argv)")
     assert output.splitlines()[-1] == str(['-c'])
 
     # test 2 : some arguments after
     output = run(sys.executable, pypypath, '-S',
-                 "-c", "import sys;print sys.argv", "hello")
+                 "-c", "import sys;print(sys.argv)", "hello")
     assert output.splitlines()[-1] == str(['-c','hello'])
     
     # test 3 : additionnal pypy parameters
     output = run(sys.executable, pypypath, '-S',
-                 "-O", "-c", "import sys;print sys.argv", "hello")
+                 "-O", "-c", "import sys;print(sys.argv)", "hello")
     assert output.splitlines()[-1] == str(['-c','hello'])
 
 SCRIPT_1 = """
 import sys
-print sys.argv
+print(sys.argv)
 """
 def test_scripts():
     tmpfilepath = str(udir.join("test_py_script.py"))
@@ -75,7 +84,28 @@ def test_scripts():
     # test 3 : additionnal pypy parameters
     output = run(sys.executable, pypypath, '-S', "-O", tmpfilepath, "hello")
     assert output.splitlines()[-1] == str([tmpfilepath,'hello'])
-    
+
+def test_optimize_removes_assert():
+    tmpfilepath = str(udir.join("test_assert.py"))
+    tmpfile = file(tmpfilepath, "w")
+    tmpfile.write("""
+try:
+    assert 0
+except AssertionError:
+    print("AssertionError")
+else:
+    print("nothing")
+""")
+    tmpfile.close()
+
+    # no optimization: crashes
+    output = run(sys.executable, pypypath, '-S', tmpfilepath)
+    assert "AssertionError" in output
+
+    # optimization: just works
+    output = run(sys.executable, pypypath, '-SO', tmpfilepath)
+    assert "nothing" in output
+
 
 TB_NORMALIZATION_CHK= """
 class K(object):
@@ -98,4 +128,40 @@ def test_tb_normalization():
     popen = subprocess.Popen([sys.executable, str(pypypath), '-S', tmpfilepath],
                              stderr=subprocess.PIPE)
     _, stderr = popen.communicate()
-    assert stderr.endswith('KeyError: <normalized>\n')
+    assert 'KeyError: <normalized>\n' in stderr
+
+
+def test_pytrace():
+    output = run(sys.executable, pypypath, '-S',
+                 stdin="__pytrace__ = 1\nx = 5\nx")
+    output = output.replace('\r\n', '\n')
+    assert ('\t<module>:           LOAD_CONST    0 (5)\n'
+            '\t<module>:           STORE_NAME    0 (x)\n'
+            '\t<module>:           LOAD_CONST    1 (None)\n'
+            '\t<module>:           RETURN_VALUE    0 \n'
+            '>>>> ') in output
+    # '5\n' --- this line sent to stderr
+    assert ('\t<module>:           LOAD_NAME    0 (x)\n'
+            '\t<module>:           PRINT_EXPR    0 \n') in output
+
+def test_pytrace_dis_bug():
+    output = run(sys.executable, pypypath, '-S',
+                 stdin="__pytrace__ = 1\nfor i in range(1): i += 1\n")
+    output = output.replace('\r\n', '\n')
+    assert """\
+\t<module>:           LOAD_NAME    0 (range)
+\t<module>:           LOAD_CONST    0 (1)
+\t<module>:           CALL_FUNCTION    1 
+\t<module>:           GET_ITER    0 
+\t<module>:           FOR_ITER   12 (to 20)
+\t<module>:           STORE_NAME    1 (i)
+\t<module>:           LOAD_NAME    1 (i)
+\t<module>:           LOAD_CONST    0 (1)
+\t<module>:           INPLACE_ADD    0 
+\t<module>:           STORE_NAME    1 (i)
+\t<module>:           JUMP_ABSOLUTE    8 
+\t<module>:           FOR_ITER   12 (to 20)
+\t<module>:           LOAD_CONST    1 (None)
+\t<module>:           RETURN_VALUE    0 
+""" in output
+
