@@ -537,6 +537,8 @@ def _str_decode_utf8_slowpath(s, errors, final, errorhandler, allow_surrogates):
             pos += 4
 
     r = result.build()
+    # XXX can keep track of the resulting length without calling check_utf8
+    # here
     return r, rutf8.check_utf8(r, True), pos
 
 hexdigits = "0123456789ABCDEFabcdef"
@@ -593,13 +595,15 @@ def str_decode_unicode_escape(s, errors, final, errorhandler, ud_handler):
             continue
 
         # - Escapes
-        pos += 1
-        if pos >= len(s):
+        if pos + 1 >= len(s):
+            if not final:
+                break
             message = "\\ at end of string"
             r, pos, rettype, s = errorhandler(errors, "unicodeescape",
                                     message, s, pos - 1, len(s))
             builder.append(r)
             continue
+        pos += 1
 
         ch = s[pos]
         pos += 1
@@ -646,18 +650,27 @@ def str_decode_unicode_escape(s, errors, final, errorhandler, ud_handler):
         # \xXX
         elif ch == 'x':
             digits = 2
+            if pos + digits > len(s) and not final:
+                pos -= 2
+                break
             message = "truncated \\xXX escape"
             pos, s = hexescape(builder, s, pos, digits,
                             "unicodeescape", errorhandler, message, errors)
         # \uXXXX
         elif ch == 'u':
             digits = 4
+            if pos + digits > len(s) and not final:
+                pos -= 2
+                break
             message = "truncated \\uXXXX escape"
             pos, s = hexescape(builder, s, pos, digits,
                             "unicodeescape", errorhandler, message, errors)
         #  \UXXXXXXXX
         elif ch == 'U':
             digits = 8
+            if pos + digits > len(s) and not final:
+                pos -= 2
+                break
             message = "truncated \\UXXXXXXXX escape"
             pos, s = hexescape(builder, s, pos, digits,
                             "unicodeescape", errorhandler, message, errors)
@@ -684,11 +697,18 @@ def str_decode_unicode_escape(s, errors, final, errorhandler, ud_handler):
                     pos = look + 1
                     builder.append_code(code)
                 else:
+                    if not final:
+                        pos -= 2
+                        break
                     r, pos, rettype, s = errorhandler(errors, "unicodeescape",
                                             message, s, pos - 1, look + 1)
                     builder.append(r)
             else:
+                if not final:
+                    pos -= 2
+                    break
                 r, pos, rettype, s = errorhandler(errors, "unicodeescape",
+
                                         message, s, pos - 1, look + 1)
                 builder.append(r)
         else:
@@ -731,41 +751,40 @@ def str_decode_raw_unicode_escape(s, errors, final=False,
     pos = 0
     while pos < len(s):
         ch = s[pos]
+        pos += 1
 
         # Non-escape characters are interpreted as Unicode ordinals
         if ch != '\\':
             builder.append_code(ord(ch))
-            pos += 1
             continue
 
-        # \u-escapes are only interpreted iff the number of leading
-        # backslashes is odd
-        bs = pos
-        while pos < len(s):
-            pos += 1
-            if pos == len(s) or s[pos] != '\\':
-                break
-            builder.append_char('\\')
-
-        # we have a backslash at the end of the string, stop here
-        if pos >= len(s):
-            builder.append_char('\\')
+        if pos == len(s):
+            if final:
+                # we have a backslash at the end of the string, stop here
+                builder.append_char('\\')
+            else:
+                pos -= 1
             break
-
-        if ((pos - bs) & 1 == 0 or pos >= len(s) or
-                (s[pos] != 'u' and s[pos] != 'U')):
+        ch = s[pos]
+        if ch == "\\":
             builder.append_char('\\')
-            builder.append_code(ord(s[pos]))
             pos += 1
             continue
-
         if s[pos] == 'u':
             digits = 4
             message = "truncated \\uXXXX escape"
-        else:
+        elif s[pos] == 'U':
             digits = 8
             message = "truncated \\UXXXXXXXX escape"
+        else:
+            builder.append_char('\\')
+            builder.append_char(ch)
+            pos += 1
+            continue
         pos += 1
+        if pos + digits > len(s) and not final:
+            pos -= 2
+            break
         pos, s = hexescape(builder, s, pos, digits,
                            "rawunicodeescape", errorhandler, message, errors)
 
@@ -999,6 +1018,12 @@ def str_decode_utf_7(s, errors, final=False,
                 pos += 1
                 result.append('+')
                 outsize += 1
+            elif pos < len(s) and not _utf7_IS_BASE64(ord(s[pos])):
+                msg = "ill-formed sequence"
+                r, pos, rettype, s = errorhandler(errors, 'utf7', msg, s, pos-1, pos+1)
+                reslen = rutf8.check_utf8(r, True)
+                outsize += reslen
+                result.append(r)
             else: # begin base64-encoded section
                 inShift = 1
                 surrogate = 0
@@ -1511,83 +1536,6 @@ def utf8_encode_utf_32_le(s, errors,
                                         allow_surrogates, "little",
                                         'utf-32-le')
 # ____________________________________________________________
-# unicode-internal
-
-def str_decode_unicode_internal(s, errors, final=False,
-                                errorhandler=None):
-    if len(s) == 0:
-        return '', 0
-
-    if rutf8.MAXUNICODE < 65536:
-        unicode_bytes = 2
-    else:
-        unicode_bytes = 4
-    if BYTEORDER == "little":
-        start = 0
-        stop = unicode_bytes
-        step = 1
-    else:
-        start = unicode_bytes - 1
-        stop = -1
-        step = -1
-
-    result = StringBuilder(len(s))
-    pos = 0
-    while pos < len(s):
-        if pos > len(s) - unicode_bytes:
-            r, pos, rettype, s = errorhandler(errors, "unicode_internal",
-                                    "truncated input",
-                                    s, pos, len(s))
-            result.append(r)
-            continue
-        t = r_uint(0)
-        h = 0
-        for j in range(start, stop, step):
-            t += r_uint(ord(s[pos + j])) << (h*8)
-            h += 1
-        if t > rutf8.MAXUNICODE:
-            r, pos, rettype, s = errorhandler(errors, "unicode_internal",
-                                    "unichr(%d) not in range" % (t,),
-                                    s, pos, pos + unicode_bytes)
-            result.append(r)
-            continue
-        rutf8.unichr_as_utf8_append(result, intmask(t), allow_surrogates=True)
-        pos += unicode_bytes
-    r = result.build()
-    lgt = rutf8.check_utf8(r, True)
-    return r, lgt
-
-def utf8_encode_unicode_internal(s, errors, errorhandler, allow_surrogates=False):
-    # uses only s, ignores the other arguments
-    size = len(s)
-    if size == 0:
-        return ''
-
-    if rutf8.MAXUNICODE < 65536:
-        unicode_bytes = 2
-    else:
-        unicode_bytes = 4
-    result = StringBuilder(size * unicode_bytes)
-    pos = 0
-    while pos < size:
-        oc = rutf8.codepoint_at_pos(s, pos)
-        if BYTEORDER == "little":
-            result.append(chr(oc       & 0xFF))
-            result.append(chr(oc >>  8 & 0xFF))
-            if unicode_bytes > 2:
-                result.append(chr(oc >> 16 & 0xFF))
-                result.append(chr(oc >> 24 & 0xFF))
-        else:
-            if unicode_bytes > 2:
-                result.append(chr(oc >> 24 & 0xFF))
-                result.append(chr(oc >> 16 & 0xFF))
-            result.append(chr(oc >>  8 & 0xFF))
-            result.append(chr(oc       & 0xFF))
-        pos = rutf8.next_codepoint_pos(s, pos)
-
-    return result.build()
-
-# ____________________________________________________________
 # Charmap
 
 ERROR_CHAR = u'\ufffe'.encode('utf8')
