@@ -14,12 +14,29 @@ class AppTestBuiltinApp:
         assert d.f("abc", "def") == "abcdef"
         assert D.f("abc", "def") == "abcdef"
 
+    def test_staticmethod_dict(self):
+        sm = staticmethod(None)
+        assert sm.__dict__ == {}
+        sm.x = 42
+        assert sm.x == 42
+        assert sm.__dict__ == {"x" : 42}
+        del sm.x
+        assert not hasattr(sm, "x")
+        raises(TypeError, setattr, sm, '__dict__', [])
+        raises((AttributeError, TypeError), delattr, sm, '__dict__')
+
     def test_staticmethod_subclass(self):
         class Static(staticmethod):
             pass
         x = Static(1)
         assert isinstance(x, Static)
 
+        class C(Static):
+            def __init__(self, callable):
+                super(C, self).__init__(callable)
+        y = C(1)
+        assert isinstance(y, C)
+
     def test_classmethod(self):
         class C(object):
             def f(cls, stuff):
@@ -41,8 +58,14 @@ class AppTestBuiltinApp:
         x = Classm(1)
         assert isinstance(x, Classm)
 
+        class C(Classm):
+            def __init__(self, callable):
+                super(C, self).__init__(callable)
+        y = C(1)
+        assert isinstance(y, C)
+
     def test_property_simple(self):
-        
+
         class a(object):
             def _get(self): return 42
             def _set(self, value): raise AttributeError
@@ -70,6 +93,23 @@ class AppTestBuiltinApp:
         assert d.f() == "DBCA"
         assert D.__mro__ == (D, B, C, A, object)
 
+    def test_super_magic(self):
+        class A(object):
+            def f(self):
+                return 'A'
+        class B(A):
+            def f(self):
+                return 'B' + super().f()
+        class C(A):
+            def f(self):
+                return 'C' + super().f()
+        class D(B, C):
+            def f(self):
+                return 'D' + super().f()
+        d = D()
+        assert d.f() == "DBCA"
+        assert D.__mro__ == (D, B, C, A, object)
+
     def test_super_metaclass(self):
         class xtype(type):
             def __init__(self, name, bases, dict):
@@ -90,15 +130,26 @@ class AppTestBuiltinApp:
             f = classmethod(f)
         assert B().f() == [B, B]
 
+    def test_super_magic_classmethod(self):
+        class A(object):
+            def f(cls):
+                return cls
+            f = classmethod(f)
+        class B(A):
+            def f(cls):
+                return [cls, super().f()]
+            f = classmethod(f)
+        assert B().f() == [B, B]
+
     def test_super_fail(self):
         try:
             super(list, 2)
-        except TypeError, e:
+        except TypeError as e:
             message = e.args[0]
             assert message.startswith('super(type, obj): obj must be an instance or subtype of type')
 
     def test_super_various(self):
-        
+
         class A(object):
             def meth(self, a):
                 return "A(%r)" % a
@@ -207,6 +258,17 @@ class AppTestBuiltinApp:
         raises(TypeError, "super(D).__get__(12)")
         raises(TypeError, "super(D).__get__(C())")
 
+    def test_super_incomplete(self):
+        """
+        class M(type):
+            def mro(cls):
+                if cls.__mro__ is None:
+                    raises(AttributeError, lambda: super(cls, cls).xxx)
+                return type.mro(cls)
+        class A(metaclass=M):
+            pass
+        """
+
     def test_classmethods_various(self):
         class C(object):
             def foo(*a): return a
@@ -214,7 +276,7 @@ class AppTestBuiltinApp:
         c = C()
         assert C.goo(1) == (C, 1)
         assert c.goo(1) == (C, 1)
-        
+
         assert c.foo(1) == (c, 1)
         class D(C):
             pass
@@ -228,16 +290,75 @@ class AppTestBuiltinApp:
         assert ff.__get__(0, int)(42) == (int, 42)
         assert ff.__get__(0)(42) == (int, 42)
 
-        assert C.goo.im_self is C
-        assert D.goo.im_self is D
-        assert super(D,D).goo.im_self is D
-        assert super(D,d).goo.im_self is D
+        assert C.goo.__self__ is C
+        assert D.goo.__self__ is D
+        assert super(D,D).goo.__self__ is D
+        assert super(D,d).goo.__self__ is D
         assert super(D,D).goo() == (D,)
         assert super(D,d).goo() == (D,)
 
         meth = classmethod(1).__get__(1)
         raises(TypeError, meth)
 
+    def test_classmethod_dict(self):
+        cm = classmethod(None)
+        assert cm.__dict__ == {}
+        cm.x = 42
+        assert cm.x == 42
+        assert cm.__dict__ == {"x": 42}
+        del cm.x
+        assert not hasattr(cm, "x")
+        cm.x = 42
+        cm.__dict__ = {}
+        assert not hasattr(cm, "x")
+        raises(TypeError, setattr, cm, '__dict__', [])
+        raises((AttributeError, TypeError), delattr, cm, '__dict__')
+
+    def test_super_thisclass(self):
+        class A(object):
+            pass
+
+        assert super(A, A()).__thisclass__ is A
+
+        class B(A):
+            pass
+
+        assert super(B, B()).__thisclass__ is B
+        assert super(A, B()).__thisclass__ is A
+
+    def test_super_self_selfclass(self):
+        class A(object):
+            pass
+        class B(A):
+            pass
+        b = B()
+        assert super(A, b).__self__ is b
+        assert super(A).__self__ is None
+        assert super(A, b).__self_class__ is B
+        assert super(A).__self_class__ is None
+
+    def test_super_repr(self):
+        class A(object):
+            def __repr__(self):
+                return super(A, self).__repr__() + '!'
+        assert repr(A()).endswith('>!')
+        assert repr(super(A, A())) == "<super: <class 'A'>, <A object>>"
+
+    def test_super_get_corner_case(self):
+        class A(object):
+            pass
+        s1 = super(A, A())
+        assert s1.__get__(42) is s1
+        assert s1.__get__(42, int) is s1
+        s2 = super(A)
+        assert s2.__get__(None, "anything") is s2
+        #
+        assert s1.__get__(None, "anything") is s1
+        raises(TypeError, s2.__get__, 42)
+        raises(TypeError, s2.__get__, 42, int)
+        a = A()
+        assert s2.__get__(a).__self__ is a
+        assert s1.__get__(a) is s1
 
     def test_property_docstring(self):
         assert property.__doc__.startswith('property')
@@ -285,14 +406,18 @@ class AppTestBuiltinApp:
         assert "fdel" in attrs
 
         assert raw.__doc__ == "I'm the x property."
+        raw.__doc__ = "modified"
+        assert raw.__doc__ == "modified"
         assert raw.fget is C.__dict__['getx']
         assert raw.fset is C.__dict__['setx']
         assert raw.fdel is C.__dict__['delx']
 
-        for attr in "__doc__", "fget", "fset", "fdel":
+        for attr in "fget", "fset", "fdel":
             try:
                 setattr(raw, attr, 42)
-            except TypeError, msg:
+            # it raises TypeError on pypy, AttributeError on CPython: we catch
+            # both so that it runs also with -A
+            except (TypeError, AttributeError) as msg:
                 if str(msg).find('readonly') < 0:
                     raise Exception("when setting readonly attr %r on a "
                                     "property, got unexpected TypeError "
@@ -311,7 +436,7 @@ class AppTestBuiltinApp:
         except ZeroDivisionError:
             pass
         else:
-            raise Exception, "expected ZeroDivisionError from bad property"
+            raise Exception("expected ZeroDivisionError from bad property")
 
     def test_property_subclass(self):
         class P(property):
@@ -323,10 +448,10 @@ class AppTestBuiltinApp:
 
     def test_property_subclass_with_init(self):
         l = []
-        
+
         def x(self):
             l.append('x')
-        
+
         class P(property):
             def __init__(self):
                 property.__init__(self, x)
@@ -370,3 +495,92 @@ class AppTestBuiltinApp:
         assert x.y == 42
         del x.x
         assert x.z == 42
+
+    def test_abstract_property(self):
+        def foo(self): pass
+        foo.__isabstractmethod__ = True
+        foo = property(foo)
+        assert foo.__isabstractmethod__ is True
+
+    def test___class___variable(self):
+        class X:
+            def f(self):
+                return __class__
+        assert X().f() is X
+
+        class X:
+            @classmethod
+            def f(cls):
+                return __class__
+        assert X.f() is X
+
+        class X:
+            @staticmethod
+            def f():
+                return __class__
+        assert X.f() is X
+
+    def test_obscure_super_errors(self):
+        """
+        def f():
+            super()
+        raises(RuntimeError, f)
+        def f(x):
+            del x
+            super()
+        raises(RuntimeError, f, None)
+        class X:
+            def f(x):
+                nonlocal __class__
+                del __class__
+                super()
+        raises(RuntimeError, X().f)
+        class X:
+            def f(self):
+                def g():
+                    print(self)    # make 'self' a closure inside 'f'
+                del self
+                super()
+        raises(RuntimeError, X().f)
+        class X:
+            def f(*args):
+                super()
+        raises(RuntimeError, X().f)
+        """
+
+    def test_uninitialized_property(self):
+        p = property.__new__(property)
+        raises(AttributeError, p.__get__, 42)
+        raises(AttributeError, p.__set__, 42, None)
+        raises(AttributeError, p.__delete__, 42)
+        assert repr(p).startswith("<property object at ")
+        assert p.fget is p.fset is p.fdel is p.__doc__ is None
+        #
+        lst = []
+        p.deleter(lst.append).__delete__(42)
+        assert lst == [42]
+        #
+        lst = []
+        p.getter(lst.append).__get__(43)
+        assert lst == [43]
+        #
+        lst = []
+        p.setter(lambda x, y: lst.append((x, y))).__set__(44, 45)
+        assert lst == [(44, 45)]
+
+    def test_uninitialized_super(self):
+        s = super.__new__(super)
+        assert repr(s) == "<super: <class 'NULL'>, NULL>"
+        assert s.__thisclass__ is s.__self__ is s.__self_class__ is None
+        assert s.__get__(None, "anything") is s
+        raises(TypeError, s.__get__, 42)
+        raises(TypeError, s.__get__, int)
+        raises(TypeError, s.__get__, type(None))
+        raises(AttributeError, "s.abcde")
+        raises(AttributeError, "s.abcde = 42")
+        raises(AttributeError, "del s.abcde")
+
+    def test_cant_pickle_property(self):
+        import pickle
+        p = property(1, 2, 3)
+        raises(TypeError, pickle.dumps, p)
