@@ -43,7 +43,7 @@ def reversed_dict(space, w_obj):
     from pypy.objspace.std.dictmultiobject import W_DictMultiObject
     if not isinstance(w_obj, W_DictMultiObject):
         raise OperationError(space.w_TypeError, space.w_None)
-    return w_obj.nondescr_reversed_dict(space)
+    return w_obj.descr_reversed(space)
 
 def dict_popitem_first(space, w_obj):
     """Interp-level implementation of OrderedDict.popitem(last=False).
