@@ -40,6 +40,7 @@ class HPyDevel:
         """
         return list(map(str, [
             self.src_dir.joinpath('argparse.c'),
+            self.src_dir.joinpath('buildvalue.c'),
             self.src_dir.joinpath('helpers.c'),
         ]))
 
@@ -218,6 +219,7 @@ class build_hpy_ext_mixin:
         ext.hpy_abi = self.distribution.hpy_abi
         ext.include_dirs += self.hpydevel.get_extra_include_dirs()
         ext.sources += self.hpydevel.get_extra_sources()
+        ext.define_macros.append(('HPY', None))
         if ext.hpy_abi == 'cpython':
             ext.sources += self.hpydevel.get_ctx_sources()
             ext._hpy_needs_stub = False
@@ -292,16 +294,17 @@ class build_hpy_ext_mixin:
 
     def get_export_symbols(self, ext):
         """ Override .get_export_symbols to replace "PyInit_<module_name>"
-            with "HPyInit_<module_name>. For PyPy tests, also replace
-            "init<module_name>" with HPyInit_<module_name>
+            with "HPyInit_<module_name>.
 
             Only relevant on Windows, where the .pyd file (DLL) must export the
             module "HPyInit_" function.
         """
         exports = self._base_build_ext.get_export_symbols(self, ext)
-        if sys.version_info[0] == 2:
-            # PyPy tests
-            exports = [re.sub(r"^init", "PyInit_", name) for name in exports]
         if hasattr(ext, "hpy_abi") and ext.hpy_abi == 'universal':
-            exports = [re.sub(r"^PyInit_", "HPyInit_", name) for name in exports]
+            if sys.version_info < (3,):
+                #relevant for untranslated tests
+                init_prefix = r"^init"
+            else:
+                init_prefix = r"^PyInit_"
+            exports = [re.sub(init_prefix, "HPyInit_", name) for name in exports]
         return exports
