@@ -8,6 +8,11 @@
 _HPy_HIDDEN PyMethodDef *create_method_defs(HPyDef *hpydefs[],
                                             PyMethodDef *legacy_methods);
 
+_HPy_HIDDEN int call_traverseproc_from_trampoline(HPyFunc_traverseproc tp_traverse,
+                                                  PyObject *self,
+                                                  cpy_visitproc cpy_visit,
+                                                  void *cpy_arg);
+
 /* The C structs of pure HPy (i.e. non-legacy) custom types do NOT include
  * PyObject_HEAD. So, the CPython implementation of HPy_New must allocate a
  * memory region which is big enough to contain PyObject_HEAD + any eventual
@@ -15,7 +20,7 @@ _HPy_HIDDEN PyMethodDef *create_method_defs(HPyDef *hpydefs[],
  * that the payload is correctly aligned for every possible struct.
  *
  * Legacy custom types already include PyObject_HEAD and so do not need to
- * allocate extra memory region or use HPyPure_PyObject_HEAD_SIZE.
+ * allocate extra memory region or use _HPy_PyObject_HEAD_SIZE.
  */
 typedef struct {
     PyObject_HEAD
@@ -32,8 +37,15 @@ typedef struct {
         long double _m_longdouble;
         void *_m_pointer;
     };
-} _HPyPure_FullyAlignedSpaceForPyObject_HEAD;
+} _HPy_FullyAlignedSpaceForPyObject_HEAD;
+
+#define _HPy_PyObject_HEAD_SIZE (offsetof(_HPy_FullyAlignedSpaceForPyObject_HEAD, payload))
+
+// Return a pointer to the area of memory AFTER the PyObject_HEAD
+static inline void *_HPy_PyObject_Payload(PyObject *obj)
+{
+    return (void *) ((char *) obj + _HPy_PyObject_HEAD_SIZE);
+}
 
-#define HPyPure_PyObject_HEAD_SIZE (offsetof(_HPyPure_FullyAlignedSpaceForPyObject_HEAD, payload))
 
 #endif /* HPY_COMMON_RUNTIME_CTX_TYPE_H */
