@@ -5,6 +5,13 @@ import textwrap
 
 PY2 = sys.version_info[0] == 2
 
+# True if `sys.executable` is set to a value that allows a Python equivalent to
+# the current Python to be launched via, e.g., `python_subprocess.run(...)`.
+# By default is `True` if sys.executable is set to a true value.
+SUPPORTS_SYS_EXECUTABLE = False
+# True if we are running on the CPython debug build
+IS_PYTHON_DEBUG_BUILD = False
+
 def reindent(s, indent):
     s = textwrap.dedent(s)
     return ''.join(' '*indent + line if line.strip() else line
@@ -17,13 +24,14 @@ class DefaultExtensionTemplate(object):
         %(defines)s
         NULL
     };
+    %(globals_defs)s
     static HPyModuleDef moduledef = {
-        HPyModuleDef_HEAD_INIT,
-        .m_name = "%(name)s",
-        .m_doc = "some test for hpy",
-        .m_size = -1,
+        .name = "%(name)s",
+        .doc = "some test for hpy",
+        .size = -1,
         .legacy_methods = %(legacy_methods)s,
-        .defines = moduledefs
+        .defines = moduledefs,
+        %(globals_field)s
     };
 
     HPy_MODINIT(%(name)s)
@@ -52,9 +60,11 @@ class DefaultExtensionTemplate(object):
         self.defines_table = None
         self.legacy_methods = 'NULL'
         self.type_table = None
+        self.globals_table = None
 
     def expand(self):
         self.defines_table = []
+        self.globals_table = []
         self.type_table = []
         self.output = ['#include <hpy.h>']
         for line in self.src.split('\n'):
@@ -83,16 +93,29 @@ class DefaultExtensionTemplate(object):
         return name, args
 
     def INIT(self):
+        NL_INDENT = '\n    '
         if self.type_table:
             init_types = '\n'.join(self.type_table)
         else:
             init_types = ''
 
+        globals_defs = ''
+        globals_field = ''
+        if self.globals_table:
+            globals_defs = \
+                textwrap.dedent('''
+                static HPyGlobal *module_globals[] = {
+                    %s
+                };''') % NL_INDENT.join(self.globals_table)
+            globals_field = '.globals = module_globals'
+
         exp = self.INIT_TEMPLATE % {
             'legacy_methods': self.legacy_methods,
-            'defines': '\n        '.join(self.defines_table),
+            'defines': NL_INDENT.join(self.defines_table),
             'init_types': init_types,
-            'name': self.name}
+            'name': self.name,
+            'globals_defs': globals_defs,
+            'globals_field': globals_field}
         self.output.append(exp)
         # make sure that we don't fill the tables any more
         self.defines_table = None
@@ -101,6 +124,9 @@ class DefaultExtensionTemplate(object):
     def EXPORT(self, meth):
         self.defines_table.append('&%s,' % meth)
 
+    def EXPORT_GLOBAL(self, var):
+        self.globals_table.append('&%s,' % var)
+
     def EXPORT_LEGACY(self, pymethoddef):
         self.legacy_methods = pymethoddef
 
@@ -131,6 +157,12 @@ class Spec(object):
         self.origin = origin
 
 
+class HPyModule(object):
+    def __init__(self, name, so_file):
+        self.name = name
+        self.so_filename = so_file
+
+
 class ExtensionCompiler:
     def __init__(self, tmpdir, hpy_devel, hpy_abi, compiler_verbose=False,
                  ExtensionTemplate=DefaultExtensionTemplate,
@@ -169,10 +201,14 @@ class ExtensionCompiler:
             filename.write(source)
         return name + '.c'
 
-    def compile_module(self, ExtensionTemplate, main_src, name, extra_sources):
+    def _fixup_template(self, ExtensionTemplate):
+        return self.ExtensionTemplate if ExtensionTemplate is None else ExtensionTemplate
+
+    def compile_module(self, main_src, ExtensionTemplate=None, name='mytest', extra_sources=()):
         """
         Create and compile a HPy module from the template
         """
+        ExtensionTemplate = self._fixup_template(ExtensionTemplate)
         from distutils.core import Extension
         filename = self._expand(ExtensionTemplate, name, main_src)
         sources = [str(filename)]
@@ -220,7 +256,7 @@ class ExtensionCompiler:
                                 hpy_devel=self.hpy_devel,
                                 hpy_abi=hpy_abi,
                                 compiler_verbose=self.compiler_verbose)
-        return so_filename
+        return HPyModule(name, so_filename)
 
     def make_module(self, main_src, ExtensionTemplate=None, name='mytest',
                     extra_sources=()):
@@ -233,10 +269,10 @@ class ExtensionCompiler:
         use make_module but explicitly use compile_module and import it
         manually as required by your test.
         """
-        if ExtensionTemplate is None:
-            ExtensionTemplate = self.ExtensionTemplate
-        so_filename = self.compile_module(
-            ExtensionTemplate, main_src, name, extra_sources)
+        ExtensionTemplate = self._fixup_template(ExtensionTemplate)
+        module = self.compile_module(
+            main_src, ExtensionTemplate, name, extra_sources)
+        so_filename = module.so_filename
         if self.hpy_abi == 'universal':
             return self.load_universal_module(name, so_filename, debug=False)
         elif self.hpy_abi == 'debug':
@@ -288,6 +324,11 @@ class HPyTest:
         return self.compiler.make_module(main_src, ExtensionTemplate, name,
                                          extra_sources)
 
+    def compile_module(self, main_src, name='mytest', extra_sources=()):
+        ExtensionTemplate = self.ExtensionTemplate
+        return self.compiler.compile_module(main_src, ExtensionTemplate, name,
+                                     extra_sources)
+
     def supports_refcounts(self):
         """ Returns True if the underlying Python implementation supports
             reference counts.
@@ -391,3 +432,9 @@ def _build(tmpdir, ext, hpy_devel, hpy_abi, compiler_verbose=0, debug=None):
         distutils.log.set_threshold(old_level)
 
     return soname
+
+
+# For situations when one wants to have "support.py" on the call stack.
+# For example, for asserting that it is in a stack trace.
+def trampoline(fun, *args, **kwargs):
+    fun(*args, **kwargs)
