@@ -39,6 +39,25 @@ class TestCPythonCompatibility(HPyTest):
         if self.supports_refcounts():
             assert x == [1234, +1]
 
+    def test_frompyobject_null(self):
+        mod = self.make_module("""
+            #include <Python.h>
+            HPyDef_METH(f, "f", f_impl, HPyFunc_NOARGS)
+            static HPy f_impl(HPyContext *ctx, HPy self)
+            {
+                HPy h = HPy_FromPyObject(ctx, NULL);
+                if (HPy_IsNull(h)) {
+                    return HPy_Dup(ctx, ctx->h_True);
+                }
+                else {
+                    return HPy_Dup(ctx, ctx->h_False);
+                }
+            }
+            @EXPORT(f)
+            @INIT
+        """)
+        assert mod.f()
+
     def test_aspyobject(self):
         mod = self.make_module("""
             #include <Python.h>
@@ -55,6 +74,25 @@ class TestCPythonCompatibility(HPyTest):
         """)
         assert mod.f(21) == 42
 
+    def test_aspyobject_null(self):
+        mod = self.make_module("""
+            #include <Python.h>
+            HPyDef_METH(f, "f", f_impl, HPyFunc_NOARGS)
+            static HPy f_impl(HPyContext *ctx, HPy self)
+            {
+                PyObject *o = HPy_AsPyObject(ctx, HPy_NULL);
+                if (o == NULL) {
+                    return HPy_Dup(ctx, ctx->h_True);
+                }
+                else {
+                    return HPy_Dup(ctx, ctx->h_False);
+                }
+            }
+            @EXPORT(f)
+            @INIT
+        """)
+        assert mod.f()
+
     def test_aspyobject_custom_class(self):
         mod = self.make_module("""
             #include <Python.h>
