@@ -57,8 +57,11 @@ class HPyAppTest(object):
         # it would be nice to use the 'compiler' fixture to provide
         # make_module as the std HPyTest do. However, we don't have the space
         # yet, so it is much easier to prove make_module() here
+        prefix = request.function.__name__ + '-'
+        if sys.platform == 'win32':
+            prefix = prefix.lower()
         tmpdir = py.path.local.make_numbered_dir(rootdir=udir,
-                                                 prefix=request.function.__name__ + '-',
+                                                 prefix=prefix,
                                                  keep=0)  # keep everything
 
         hpy_devel = HPyDevel(str(BASE_DIR))
@@ -76,9 +79,9 @@ class HPyAppTest(object):
             else:
                 items_w = space.unpackiterable(w_extra_sources)
                 extra_sources = [space.text_w(item) for item in items_w]
-            py_filename = compiler.compile_module(ExtensionTemplate,
-                                                  main_src, name, extra_sources)
-            so_filename = py_filename.replace(".py", ".hpy.so")
+            module = compiler.compile_module(main_src, ExtensionTemplate,
+                                                  name, extra_sources)
+            so_filename = module.so_filename.replace(".py", ".hpy.so")
             debug = hpy_abi == 'debug'
             w_mod = space.appexec([space.newtext(so_filename),
                                    space.newtext(name),
