@@ -20,39 +20,40 @@ class W_MD5(W_Root):
         self.update(string)
 
     def digest_w(self):
-        return self.space.wrap(self.digest())
+        return self.space.newbytes(self.digest())
 
     def hexdigest_w(self):
-        return self.space.wrap(self.hexdigest())
+        return self.space.newtext(self.hexdigest())
 
     def copy_w(self):
         clone = W_MD5(self.space)
         clone._copyfrom(self)
-        return self.space.wrap(clone)
+        return clone
 
 
-@unwrap_spec(initialdata='bufferstr')
-def W_MD5___new__(space, w_subtype, initialdata=''):
+@unwrap_spec(initialdata='bufferstr', usedforsecurity=bool)
+def W_MD5___new__(space, w_subtype, initialdata='', usedforsecurity=True):
     """
     Create a new md5 object and call its initializer.
     """
     w_md5 = space.allocate_instance(W_MD5, w_subtype)
     md5 = space.interp_w(W_MD5, w_md5)
     W_MD5.__init__(md5, space)
+    # Ignore usedforsecurity
     md5.update(initialdata)
     return w_md5
 
 
 W_MD5.typedef = TypeDef(
-    'MD5Type',
+    '_md5_md5',
     __new__   = interp2app(W_MD5___new__),
     update    = interp2app(W_MD5.update_w),
     digest    = interp2app(W_MD5.digest_w),
     hexdigest = interp2app(W_MD5.hexdigest_w),
     copy      = interp2app(W_MD5.copy_w),
     digest_size = 16,
-    digestsize = 16,
     block_size = 64,
+    name      = 'md5',
     __doc__   = """md5(arg) -> return new md5 object.
 
 If arg is present, the method call update(arg) is made.""")
