@@ -13,7 +13,7 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
-#if defined(HAVE_SYS_STAT_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_STAT_H)
 #include <sys/stat.h>
 #endif
 #ifdef HAVE_SYS_SYSCALL_H
@@ -26,6 +26,9 @@
 #include <dirent.h>
 #endif
 
+#include <grp.h>
+
+
 #if defined(__ANDROID__) && __ANDROID_API__ < 21 && !defined(SYS_getdents64)
 # include <sys/linux-syscalls.h>
 # define SYS_getdents64  __NR_getdents64
@@ -391,6 +394,9 @@ pypy_subprocess_child_exec(
            int errpipe_read, int errpipe_write,
            int close_fds, int restore_signals,
            int call_setsid,
+           int call_setgid, gid_t gid,
+           int call_setgroups, size_t groups_size, const gid_t *groups,
+           int call_setuid, uid_t uid, int child_umask,
            long *py_fds_to_keep,
 	   ssize_t num_fds_to_keep,
            int (*preexec_fn)(void*),
@@ -460,7 +466,10 @@ pypy_subprocess_child_exec(
 
     if (cwd)
         POSIX_CALL(chdir(cwd));
-    
+
+    if (child_umask >= 0)
+        umask(child_umask);  /* umask() always succeeds. */
+
     if (restore_signals) {
         /* inline _Py_RestoreSignals(); */
 #ifdef SIGPIPE
@@ -480,6 +489,21 @@ pypy_subprocess_child_exec(
         POSIX_CALL(setsid());
 #endif
 
+#ifdef HAVE_SETGROUPS
+    if (call_setgroups)
+        POSIX_CALL(setgroups(groups_size, groups));
+#endif /* HAVE_SETGROUPS */
+
+#ifdef HAVE_SETREGID
+    if (call_setgid)
+        POSIX_CALL(setregid(gid, gid));
+#endif /* HAVE_SETREGID */
+
+#ifdef HAVE_SETREUID
+    if (call_setuid)
+        POSIX_CALL(setreuid(uid, uid));
+#endif /* HAVE_SETREUID */
+
     reached_preexec = 1;
     if (preexec_fn != NULL) {
         /* This is where the user has asked us to deadlock their program. */
