@@ -5,6 +5,7 @@ from pypy.interpreter.typedef import interp_attrproperty, interp_attrproperty_w
 from pypy.interpreter.typedef import make_weakref_descr
 from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from pypy.interpreter.error import OperationError, oefmt
+from pypy.objspace.std.util import generic_alias_class_getitem
 from rpython.rlib.objectmodel import compute_hash
 from rpython.rlib.rarithmetic import intmask, int_between
 from rpython.rlib import jit, rutf8
@@ -302,22 +303,28 @@ class W_SRE_Pattern(W_Root):
 
     @unwrap_spec(pos=int, endpos=int)
     def match_w(self, w_string, pos=0, endpos=sys.maxint):
+        """Matches zero or more characters at the beginning of the string."""
         ctx = self.make_ctx(w_string, pos, endpos)
         return self.getmatch(ctx, matchcontext(self.space, ctx, self.code))
 
     @unwrap_spec(pos=int, endpos=int)
     def fullmatch_w(self, w_string, pos=0, endpos=sys.maxint):
+        """Matches against all of the string."""
         ctx = self.make_ctx(w_string, pos, endpos)
         ctx.match_mode = rsre_core.MODE_FULL
         return self.getmatch(ctx, matchcontext(self.space, ctx, self.code))
 
     @unwrap_spec(pos=int, endpos=int)
     def search_w(self, w_string, pos=0, endpos=sys.maxint):
+        """Scan through string looking for a match, and return a corresponding match object instance.
+
+Return None if no position in the string matches."""
         ctx = self.make_ctx(w_string, pos, endpos)
         return self.getmatch(ctx, searchcontext(self.space, ctx, self.code))
 
     @unwrap_spec(pos=int, endpos=int)
     def findall_w(self, w_string, pos=0, endpos=sys.maxint):
+        """Return a list of all non-overlapping matches of pattern in string."""
         space = self.space
         matchlist_w = []
         ctx = self.make_ctx(w_string, pos, endpos)
@@ -343,6 +350,9 @@ class W_SRE_Pattern(W_Root):
 
     @unwrap_spec(pos=int, endpos=int)
     def finditer_w(self, w_string, pos=0, endpos=sys.maxint):
+        """Return an iterator over all non-overlapping matches for the RE pattern in string.
+
+For each match, the iterator returns a match object."""
         # this also works as the implementation of the undocumented
         # scanner() method.
         ctx = self.make_ctx(w_string, pos, endpos)
@@ -351,6 +361,7 @@ class W_SRE_Pattern(W_Root):
 
     @unwrap_spec(maxsplit=int)
     def split_w(self, w_string, maxsplit=0):
+        """Split string by the occurrences of pattern."""
         space = self.space
         #
         splitlist = []
@@ -380,14 +391,16 @@ class W_SRE_Pattern(W_Root):
 
     @unwrap_spec(count=int)
     def sub_w(self, w_repl, w_string, count=0):
+        """Return the string obtained by replacing the leftmost non-overlapping occurrences of pattern in string by the replacement repl."""
         w_item, n = self.subx(w_repl, w_string, count)
         return w_item
 
     @unwrap_spec(count=int)
     def subn_w(self, w_repl, w_string, count=0):
+        """Return the tuple (new_string, number_of_subs_made) found by replacing the leftmost non-overlapping occurrences of pattern with the replacement repl."""
         w_item, n = self.subx(w_repl, w_string, count)
         space = self.space
-        return space.newtuple([w_item, space.newint(n)])
+        return space.newtuple2(w_item, space.newint(n))
 
     def subx(self, w_ptemplate, w_string, count):
         space = self.space
@@ -611,6 +624,7 @@ def SRE_Pattern__new__(space, w_subtype, w_pattern, flags, w_code,
 
 W_SRE_Pattern.typedef = TypeDef(
     're.Pattern',
+    __doc__      = "Compiled regular expression object.",
     __new__      = interp2app(SRE_Pattern__new__),
     __copy__     = interp2app(W_SRE_Pattern.copy_identity_w),
     __deepcopy__ = interp2app(W_SRE_Pattern.copy_identity_w),
@@ -633,6 +647,8 @@ W_SRE_Pattern.typedef = TypeDef(
     groups       = interp_attrproperty('num_groups', W_SRE_Pattern,
         wrapfn="newint"),
     pattern      = interp_attrproperty_w('w_pattern', W_SRE_Pattern),
+    __class_getitem__ = interp2app(
+        generic_alias_class_getitem, as_classmethod=True),
 )
 W_SRE_Pattern.typedef.acceptable_as_base_class = False
 
@@ -674,6 +690,9 @@ class W_SRE_Match(W_Root):
 
     @jit.look_inside_iff(lambda self, args_w: jit.isconstant(len(args_w)))
     def group_w(self, args_w):
+        """group([group1, ...]) -> str or tuple.
+Return subgroup(s) of the match by indices or names.
+For 0 returns the entire match."""
         space = self.space
         ctx = self.ctx
         if len(args_w) <= 1:
@@ -740,8 +759,8 @@ class W_SRE_Match(W_Root):
     def new_charindex_tuple(self, start, end):
         start = self.bytepos_to_charindex(start)
         end = self.bytepos_to_charindex(end)
-        return self.space.newtuple([self.space.newint(start),
-                                    self.space.newint(end)])
+        return self.space.newtuple2(self.space.newint(start),
+                                    self.space.newint(end))
 
     def bytepos_to_charindex(self, bytepos):
         # Transform a 'byte position', as returned by all methods from
@@ -844,6 +863,8 @@ class W_SRE_Match(W_Root):
 
 W_SRE_Match.typedef = TypeDef(
     're.Match',
+    __doc__      = """The result of re.match() and re.search().
+Match objects always have a boolean value of True.""",
     __copy__     = interp2app(W_SRE_Match.copy_identity_w),
     __deepcopy__ = interp2app(W_SRE_Match.copy_identity_w),
     __repr__     = interp2app(W_SRE_Match.repr_w),
@@ -864,6 +885,8 @@ W_SRE_Match.typedef = TypeDef(
     lastgroup    = GetSetProperty(W_SRE_Match.fget_lastgroup),
     lastindex    = GetSetProperty(W_SRE_Match.fget_lastindex),
     regs         = GetSetProperty(W_SRE_Match.fget_regs),
+    __class_getitem__ = interp2app(
+        generic_alias_class_getitem, as_classmethod=True),
 )
 W_SRE_Match.typedef.acceptable_as_base_class = False
 
