@@ -31,13 +31,16 @@ class AppTestFfi:
 
     def setup_class(cls):
         import _winreg as winreg
+        from platform import machine
         space = cls.space
         cls.root_key = winreg.HKEY_CURRENT_USER
-        cls.test_key_name = "SOFTWARE\\Pypy Registry Test Key - Delete Me"
+        cls.test_key_name = "SOFTWARE\\Pypy Test Key - Delete Me [%d]" % os.getpid()
         cls.w_root_key = space.wrap(cls.root_key)
         cls.w_test_key_name = space.wrap(cls.test_key_name)
         cls.w_canSaveKey = space.wrap(canSaveKey)
         cls.w_tmpfilename = space.wrap(str(udir.join('winreg-temp')))
+        cls.w_runappdirect = space.wrap(cls.runappdirect)
+        cls.w_win64_machine = space.wrap(machine() == "AMD64")
 
         test_data = [
             ("Int Value", 0xFEDCBA98, winreg.REG_DWORD),
@@ -45,6 +48,11 @@ class AppTestFfi:
             ("Unicode Value", "A unicode Value", winreg.REG_SZ),
             ("Str Expand", "The path is %path%", winreg.REG_EXPAND_SZ),
             ("Multi Str", [b"Several", u"string", u"values"], winreg.REG_MULTI_SZ),
+            ("Multi Str Empty", ["", "", ""], winreg.REG_MULTI_SZ),
+            ("Raw None", None, winreg.REG_BINARY),
+            # issue 3342, round-trip invalid UTF-16 strings
+            (u"Invalid \uDC00", u"\uDC00", winreg.REG_SZ),
+            (u"Invalid \uD800", u"\uD800", winreg.REG_SZ),
             ]
         cls.w_test_data = w_test_data = space.wrap(test_data)
         w_btest = space.newtuple([space.wrap("Raw data"),
@@ -56,7 +64,7 @@ class AppTestFfi:
         import _winreg
         try:
             _winreg.DeleteKey(cls.root_key, cls.test_key_name)
-        except WindowsError:
+        except WindowsError as e:
             pass
 
     def test_constants(self):
@@ -153,9 +161,13 @@ class AppTestFfi:
                             REG_BINARY, memoryview(b'abc'))
 
     def test_readValues(self):
+        # needs the keys from test_SetValueEx
         from winreg import OpenKey, EnumValue, QueryValueEx, EnumKey
         from winreg import REG_SZ, REG_EXPAND_SZ
-        key = OpenKey(self.root_key, self.test_key_name)
+        try:
+            key = OpenKey(self.root_key, self.test_key_name)
+        except FileNotFoundError:
+            raise RuntimeError("run test_SetValueEx first")
         sub_key = OpenKey(key, "sub_key")
         index = 0
         while 1:
@@ -178,6 +190,16 @@ class AppTestFfi:
         assert EnumKey(key, 0) == "sub_key"
         raises(EnvironmentError, EnumKey, key, 1)
 
+    def test_qword(self):
+        # REG_QWORD is added in Python 3.6, can't set in setup_class
+        from winreg import CreateKey, SetValueEx, QueryValueEx, REG_QWORD
+        key = CreateKey(self.root_key, self.test_key_name)
+        sub_key = CreateKey(key, u"sub_key")
+        name = 'Long Value'
+        value = 0xFEDCBA9876543210
+        SetValueEx(sub_key, name, 0, REG_QWORD, value)
+        assert QueryValueEx(sub_key, name) == (value, REG_QWORD)
+
     def test_delete(self):
         # must be run after test_SetValueEx
         from winreg import OpenKey, KEY_ALL_ACCESS, DeleteValue, DeleteKey
@@ -188,9 +210,24 @@ class AppTestFfi:
             DeleteValue(sub_key, name)
         # cannot wrap a memoryview in setup_class for test_data
         DeleteValue(sub_key, 'test_name')
+        # REG_QWORD is added in Python 3.6, can't set in setup_class
+        DeleteValue(sub_key, 'Long Value')
 
         DeleteKey(key, "sub_key")
 
+    def test_truncate_null(self):
+        # bpo-25778: REG_SZ should not contain null characters
+        from winreg import CreateKey, SetValueEx, QueryValueEx, DeleteValue
+        from winreg import REG_SZ, HKEY_CURRENT_USER
+        key = CreateKey(self.root_key, self.test_key_name)
+
+        test_name = "test_truncate"
+        test_val = "A string\x00 with a null"
+        expected_val = "A string"
+        SetValueEx(key, test_name, 0, REG_SZ, test_val)
+        assert QueryValueEx(key, test_name) == (expected_val, REG_SZ)
+        DeleteValue(key, test_name)
+
     def test_connect(self):
         from winreg import ConnectRegistry, HKEY_LOCAL_MACHINE
         h = ConnectRegistry(None, HKEY_LOCAL_MACHINE)
@@ -240,6 +277,8 @@ class AppTestFfi:
 
     def test_dynamic_key(self):
         from winreg import EnumValue, QueryValueEx, HKEY_PERFORMANCE_DATA
+        if not self.runappdirect:
+            skip('very slow untranslated')
         try:
             EnumValue(HKEY_PERFORMANCE_DATA, 0)
         except WindowsError as e:
@@ -264,12 +303,43 @@ class AppTestFfi:
             raises(NotImplementedError, DeleteKeyEx, self.root_key,
                    self.test_key_name)
 
+    def test_reflection(self):
+        import sys
+        from winreg import DisableReflectionKey, EnableReflectionKey, \
+                           QueryReflectionKey, OpenKey, HKEY_LOCAL_MACHINE
+        # Adapted from lib-python test
+        if not self.win64_machine:
+            skip("Requires 64-bit host")
+        # Test that we can call the query, enable, and disable functions
+        # on a key which isn't on the reflection list with no consequences.
+        with OpenKey(HKEY_LOCAL_MACHINE, "Software") as key:
+            # HKLM\Software is redirected but not reflected in all OSes
+            assert QueryReflectionKey(key)
+            assert EnableReflectionKey(key) is None
+            assert DisableReflectionKey(key) is None
+            assert QueryReflectionKey(key)
+
     def test_named_arguments(self):
-        from winreg import KEY_ALL_ACCESS, CreateKeyEx, DeleteKey, OpenKeyEx
+        from winreg import KEY_ALL_ACCESS, CreateKeyEx, DeleteKey, DeleteKeyEx, OpenKeyEx
         with CreateKeyEx(key=self.root_key, sub_key=self.test_key_name,
                          reserved=0, access=KEY_ALL_ACCESS) as ckey:
             assert ckey.handle != 0
         with OpenKeyEx(key=self.root_key, sub_key=self.test_key_name,
                        reserved=0, access=KEY_ALL_ACCESS) as okey:
             assert okey.handle != 0
-        DeleteKey(self.root_key, self.test_key_name)
+        if self.win64_machine:
+            DeleteKeyEx(key=self.root_key, sub_key=self.test_key_name,
+                        access=KEY_ALL_ACCESS, reserved=0)
+        else:
+            DeleteKey(self.root_key, self.test_key_name)
+
+    def test_open_with_surrogate(self):
+        import winreg
+        s = '\uac55\ud8ab\u2d00\u8800PVL'
+        try:
+            # issue 3345: fails to interpret subkey, raises UnicodeEncodeError
+            with winreg.OpenKey(winreg.HKEY_CLASSES_ROOT, s) as key:
+                pass
+        except OSError:
+            pass
+        
