@@ -1,5 +1,6 @@
 from pypy.interpreter.gateway import unwrap_spec
 from rpython.rlib.rstring import StringBuilder
+from pypy.module.binascii.interp_binascii import AsciiBufferUnwrapper
 
 MAXLINESIZE = 76
 
@@ -14,7 +15,7 @@ def hexval(c):
         return ord(c) - (ord('a') - 10)
 hexval._always_inline_ = True
 
-@unwrap_spec(data='bufferstr', header=int)
+@unwrap_spec(data=AsciiBufferUnwrapper, header=int)
 def a2b_qp(space, data, header=0):
     "Decode a string of qp-encoded data."
 
@@ -56,7 +57,7 @@ def a2b_qp(space, data, header=0):
             if header and c == '_':
                 c = ' '
             odata.append(c)
-    return space.wrap(odata.build())
+    return space.newbytes(odata.build())
 
 # ____________________________________________________________
 
@@ -159,4 +160,4 @@ both encoded.  When quotetabs is set, space and tabs are encoded."""
                 odata.append(c)
                 inp += 1
 
-    return space.wrap(odata.build())
+    return space.newbytes(odata.build())
