@@ -26,11 +26,14 @@ from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.gateway import unwrap_spec
 from pypy.interpreter.nestedscope import Cell
 from pypy.interpreter.module import Module
-from pypy.interpreter.function import StaticMethod
+from pypy.interpreter.function import StaticMethod, ClassMethod
 from pypy.interpreter.pyparser import pygram
+from pypy.interpreter.typedef import Function, Method, PyTraceback
+from pypy.objspace.std.dictmultiobject import W_DictViewKeysObject, W_DictViewValuesObject
 from pypy.objspace.std.sliceobject import W_SliceObject
 from pypy.objspace.std.unicodeobject import encode_object
 from pypy.module.__builtin__.descriptor import W_Property
+from pypy.module.__builtin__.functional import W_ReversedIterator, W_Range
 #from pypy.module.micronumpy.base import W_NDimArray
 from pypy.module.__pypy__.interp_buffer import W_Bufferable
 from rpython.rlib.entrypoint import entrypoint_lowlevel
@@ -62,10 +65,10 @@ parse_dir = pypydir / 'module' / 'cpyext' / 'parse'
 source_dir = pypydir / 'module' / 'cpyext' / 'src'
 translator_c_dir = py.path.local(cdir)
 include_dirs = [
+    udir,
     include_dir,
     parse_dir,
     translator_c_dir,
-    udir,
     ]
 if WIN32:
     include_dirs.insert(0, pc_dir)
@@ -132,12 +135,13 @@ udir.join(pypy_decl).write("/* Will be filled later */\n")
 udir.join('pypy_structmember_decl.h').write("/* Will be filled later */\n")
 udir.join('pypy_marshal_decl.h').write("/* Will be filled later */\n")
 udir.join('pypy_macros.h').write("/* Will be filled later */\n")
+udir.join('genericaliasobject.h').write("/* Will be filled later */\n")
 
 constant_names = """
 Py_TPFLAGS_READY Py_TPFLAGS_READYING
 METH_COEXIST METH_STATIC METH_CLASS Py_TPFLAGS_BASETYPE
 METH_NOARGS METH_VARARGS METH_KEYWORDS METH_FASTCALL METH_O
-Py_TPFLAGS_HEAPTYPE
+Py_TPFLAGS_HEAPTYPE METH_METHOD
 Py_LT Py_LE Py_EQ Py_NE Py_GT Py_GE Py_MAX_NDIMS
 Py_CLEANUP_SUPPORTED PyBUF_READ
 PyBUF_FORMAT PyBUF_ND PyBUF_STRIDES PyBUF_WRITABLE PyBUF_SIMPLE PyBUF_WRITE
@@ -152,6 +156,11 @@ for name in ('LONG', 'LIST', 'TUPLE', 'UNICODE', 'DICT', 'BASE_EXC',
 for name in ('SIGN', 'ADD_DOT_0', 'ALT'):
     constant_names.append('Py_DTSF_%s' % name)
 
+# PyPy-specific flags
+for name in ('FLOAT',):
+    constant_names.append('Py_TPPYPYFLAGS_%s_SUBCLASS' % name)
+
+
 for name in constant_names:
     setattr(CConfig_constants, name, rffi_platform.ConstantInteger(name))
 globals().update(rffi_platform.configure(CConfig_constants))
@@ -581,7 +590,7 @@ FUNCTIONS_BY_HEADER = defaultdict(dict)
 
 # These are C symbols which cpyext will export, but which are defined in .c
 # files somewhere in the implementation of cpyext (rather than being defined in
-# RPython).
+# RPython). Their name will be mangled by a #define
 SYMBOLS_C = [
     'Py_FatalError', 'PyOS_snprintf', 'PyOS_vsnprintf', 'PyArg_Parse',
     'PyArg_ParseTuple', 'PyArg_UnpackTuple', 'PyArg_ParseTupleAndKeywords',
@@ -594,7 +603,7 @@ SYMBOLS_C = [
     '_PyArg_Parse_SizeT', '_PyArg_ParseTuple_SizeT',
     '_PyArg_ParseTupleAndKeywords_SizeT', '_PyArg_VaParse_SizeT',
     '_PyArg_VaParseTupleAndKeywords_SizeT',
-    '_Py_BuildValue_SizeT', '_Py_VaBuildValue_SizeT',
+    '_Py_BuildValue_SizeT', '_Py_VaBuildValue_SizeT', 'PyUnicode_AppendAndDel',
 
     'PyErr_Format', 'PyErr_NewException', 'PyErr_NewExceptionWithDoc',
     'PyErr_WarnFormat', '_PyErr_FormatFromCause',
@@ -630,7 +639,8 @@ SYMBOLS_C = [
 
     'PyStructSequence_InitType', 'PyStructSequence_InitType2',
     'PyStructSequence_New', 'PyStructSequence_UnnamedField',
-    'PyStructSequence_NewType',
+    'PyStructSequence_NewType', 'PyStructSequence_GetItem',
+    'PyStructSequence_SetItem', 
 
     'PyFunction_Type', 'PyMethod_Type', 'PyRange_Type', 'PyTraceBack_Type',
 
@@ -645,12 +655,12 @@ SYMBOLS_C = [
     'PyTraceMalloc_Track', 'PyTraceMalloc_Untrack',
     'PyBytes_FromFormat', 'PyBytes_FromFormatV',
 
-    'PyType_FromSpec',
+    'PyType_FromSpec', 'PyType_GetModule', 'PyType_GetModuleState',
     'Py_IncRef', 'Py_DecRef', 'PyObject_Free', 'PyObject_GC_Del', 'PyType_GenericAlloc',
     '_PyObject_New', '_PyObject_NewVar',
     '_PyObject_GC_Malloc', '_PyObject_GC_New', '_PyObject_GC_NewVar',
     'PyObject_Init', 'PyObject_InitVar',
-    'PyTuple_New', '_Py_Dealloc',
+    'PyTuple_New', '_Py_Dealloc', '_Py_object_dealloc',
     'PyVectorcall_Call',
 ]
 if sys.platform == "win32":
@@ -719,6 +729,8 @@ def build_exported_objects():
         "PyUnicode_Type": "space.w_unicode",
         "PyDict_Type": "space.w_dict",
         "PyDictProxy_Type": 'space.gettypeobject(cpyext.dictproxyobject.W_DictProxyObject.typedef)',
+        "PyDictValues_Type": "space.gettypeobject(W_DictViewValuesObject.typedef)",
+        "PyDictKeys_Type": "space.gettypeobject(W_DictViewKeysObject.typedef)",
         "PyTuple_Type": "space.w_tuple",
         "PyList_Type": "space.w_list",
         "PySet_Type": "space.w_set",
@@ -737,6 +749,7 @@ def build_exported_objects():
         'PyProperty_Type': 'space.gettypeobject(W_Property.typedef)',
         'PySlice_Type': 'space.gettypeobject(W_SliceObject.typedef)',
         'PyStaticMethod_Type': 'space.gettypeobject(StaticMethod.typedef)',
+        'PyClassMethod_Type': 'space.gettypeobject(ClassMethod.typedef)',
         'PyCFunction_Type': 'space.gettypeobject(cpyext.methodobject.W_PyCFunctionObject.typedef)',
         'PyClassMethodDescr_Type': 'space.gettypeobject(cpyext.methodobject.W_PyCClassMethodObject.typedef)',
         'PyGetSetDescr_Type': 'space.gettypeobject(cpyext.typeobject.W_GetSetPropertyEx.typedef)',
@@ -745,6 +758,11 @@ def build_exported_objects():
         'PyWrapperDescr_Type': 'space.gettypeobject(cpyext.methodobject.W_PyCWrapperObject.typedef)',
         'PyInstanceMethod_Type': 'space.gettypeobject(cpyext.classobject.InstanceMethod.typedef)',
         'PyBufferable_Type': 'space.gettypeobject(W_Bufferable.typedef)',
+        'PyReversed_Type': 'space.gettypeobject(W_ReversedIterator.typedef)',
+        'PyRange_Type': 'space.gettypeobject(W_Range.typedef)',
+        'PyFunction_Type': 'space.gettypeobject(Function.typedef)',
+        'PyMethod_Type': 'space.gettypeobject(Method.typedef)',
+        'PyTraceBack_Type': 'space.gettypeobject(PyTraceback.typedef)',
         }.items():
         register_global(cpyname, 'PyTypeObject*', pypyexpr, header=pypy_decl)
 
@@ -801,6 +819,7 @@ PyTypeObject = cts.gettype('PyTypeObject')
 PyTypeObjectPtr = cts.gettype('PyTypeObject *')
 PyObjectStruct = cts.gettype('PyObject')
 PyObject = cts.gettype('PyObject *')
+PyObjectC = cts.gettype('PyObject const *')
 PyObjectFields = (("ob_base", PyObjectStruct),)
 PyVarObjectStruct = cts.gettype('PyVarObject')
 PyVarObjectFields = (("ob_base", PyVarObjectStruct),)
@@ -1200,7 +1219,7 @@ def attach_c_functions(space, eci, prefix):
         compilation_info=eci,
         _nowrapper=True)
     state.C._PyPy_int_dealloc = rffi.llexternal(
-        '_PyPy_int_dealloc', [PyObject], lltype.Void,
+        mangle_name(prefix, '_Py_int_dealloc'), [PyObject], lltype.Void,
         compilation_info=eci, _nowrapper=True)
     state.C.PyTuple_New = rffi.llexternal(
         mangle_name(prefix, 'PyTuple_New'),
@@ -1208,7 +1227,7 @@ def attach_c_functions(space, eci, prefix):
         compilation_info=eci,
         _nowrapper=True)
     state.C._PyPy_tuple_dealloc = rffi.llexternal(
-        '_PyPy_tuple_dealloc', [PyObject], lltype.Void,
+        mangle_name(prefix, '_Py_tuple_dealloc'), [PyObject], lltype.Void,
         compilation_info=eci, _nowrapper=True)
     _, state.C.set_marker = rffi.CExternVariable(
                    rffi.VOIDP, '_pypy_rawrefcount_w_marker_deallocating',
@@ -1218,11 +1237,12 @@ def attach_c_functions(space, eci, prefix):
         [PyObject], lltype.Void,
         compilation_info=eci, _nowrapper=True)
     state.C._PyPy_object_dealloc = rffi.llexternal(
-        '_PyPy_object_dealloc', [PyObject], lltype.Void,
+        mangle_name(prefix, '_Py_object_dealloc'),
+        [PyObject], lltype.Void,
         compilation_info=eci, _nowrapper=True)
     FUNCPTR = lltype.Ptr(lltype.FuncType([], rffi.INT))
     state.C.get_pyos_inputhook = rffi.llexternal(
-        '_PyPy_get_PyOS_InputHook', [], FUNCPTR,
+        mangle_name(prefix, '_Py_get_PyOS_InputHook'), [], FUNCPTR,
         compilation_info=eci, _nowrapper=True)
     state.C.tuple_new = rffi.llexternal(
         mangle_name(prefix, '_Py_tuple_new'),
@@ -1253,11 +1273,16 @@ def attach_c_functions(space, eci, prefix):
             link_files = link_files,
             library_dirs = library_dirs,
            )
-    state.C.flag_setters = {}
+    flag_setters = {}
     for c_name, attr in _flags:
         _, setter = rffi.CExternVariable(rffi.INT_real, c_name, eci_flags,
                                          _nowrapper=True, c_type='int')
-        state.C.flag_setters[attr] = setter
+        flag_setters[attr] = setter
+    unroll_flag_setters = unrolling_iterable(flag_setters.items())
+    def init_flags(space):
+        for attr, setter in unroll_flag_setters:
+            setter(rffi.cast(rffi.INT_real, space.sys.get_flag(attr)))
+    state.C.init_flags = init_flags
 
 
 def init_function(func):
@@ -1273,11 +1298,9 @@ def run_bootstrap_functions(space):
         func(space)
 
 @init_function
-def init_flags(space):
+def call_init_flags(space):
     state = space.fromcache(State)
-    for _, attr in _flags:
-        f = state.C.flag_setters[attr]
-        f(rffi.cast(rffi.INT_real, space.sys.get_flag(attr)))
+    state.C.init_flags(space)
 
 #_____________________________________________________
 # Build the bridge DLL, Allow extension DLLs to call
@@ -1470,23 +1493,42 @@ def mangle_name(prefix, name):
     else:
         raise ValueError("Error converting '%s'" % name)
 
-def write_header(header_name, decls):
-    lines = [
-        '#include "cpyext_object.h"',
-        '''
-#ifdef _WIN64
-#define Signed   Py_ssize_t          /* xxx temporary fix */
-#define Unsigned unsigned long long  /* xxx temporary fix */
-#else
-#define Signed   Py_ssize_t     /* xxx temporary fix */
-#define Unsigned unsigned long  /* xxx temporary fix */
-#endif
-        '''] + decls + [
-        '',
-        '#undef Signed    /* xxx temporary fix */',
-        '#undef Unsigned  /* xxx temporary fix */',
-        '']
+def write_header(header_name, decls, needs_signed=True, add_guards=False):
     decl_h = udir.join(header_name)
+    lines = []
+    if add_guards:
+        guard = 'Py_' + header_name.replace('.', '_').upper()
+        lines += ['#ifndef ' + guard,
+                  '#define ' + guard,
+                  '#ifdef __cplusplus',
+                  'extern "C" {',
+                  '#endif', ''
+                 ]
+    if needs_signed:
+        lines += [
+            '',
+            '#include "cpyext_object.h"',
+            '',
+            '#ifdef _WIN64',
+            '#define Signed   Py_ssize_t          /* xxx temporary fix */',
+            '#define Unsigned unsigned long long  /* xxx temporary fix */',
+            '#else',
+            '#define Signed   Py_ssize_t     /* xxx temporary fix */',
+            '#define Unsigned unsigned long  /* xxx temporary fix */',
+            '#endif',
+            ] + decls + [
+            '',
+            '#undef Signed    /* xxx temporary fix */',
+            '#undef Unsigned  /* xxx temporary fix */',
+            '']
+    else:
+        lines += decls
+    if add_guards:
+        lines += ['#ifdef __cplusplus',
+                  '}',
+                  '#endif',
+                  '#endif /* !' + guard + ' */',
+                 ]
     decl_h.write('\n'.join(lines))
 
 def generate_decls_and_callbacks(db, prefix=''):
@@ -1552,16 +1594,23 @@ static int PySlice_GetIndicesEx(PyObject *arg0, Py_ssize_t arg1,
         decls[header].append('PyAPI_DATA(%s) %s;' % (typ, name))
 
     for header_name, header_decls in decls.iteritems():
-        write_header(header_name, header_decls)
+        # Hardcoded :(
+        if header_name in ('genericaliasobject.h',):
+            write_header(header_name, header_decls,
+                         needs_signed=False, add_guards=True)
+        else:
+            write_header(header_name, header_decls)
 
     # generate graminit.h
     graminit_h = udir.join('graminit.h')
-    graminit_h.write('/* Generated from pypy.interpreter.pyparser.pygram.syms */')
-    for attr in dir(pygram.syms):
-        val = getattr(pygram.syms, attr)
-        graminit_h.write('#define {} {}'.format(attr, val))
-
+    with graminit_h.open('w', ensure=True) as fid:
+        fid.write('/* Generated from pypy.interpreter.pyparser.pygram.syms */\n\n')
+        for attr in dir(pygram.syms):
+            val = getattr(pygram.syms, attr)
+            if isinstance(val, int):
+                fid.write('#define {} {}\n'.format(attr, val))
 
+ 
 separate_module_files = [source_dir / "varargwrapper.c",
                          source_dir / "pyerrors.c",
                          source_dir / "modsupport.c",
