@@ -127,6 +127,9 @@ typedef unsigned long Py_uhash_t;
 /* CPython needs this for the c-extension datetime, which is pure python on PyPy
    downstream packages assume it is here (Pandas for instance) */
 #include <time.h>
+#ifndef _MSC_VER
+#include <sys/time.h>
+#endif
 
 /*******************************
  * stat() and fstat() fiddling *
