@@ -17,7 +17,7 @@ typedef struct {
 struct _typeobject;
 typedef void (*freefunc)(void *);
 typedef void (*destructor)(PyObject *);
-typedef int (*printfunc)(PyObject *, FILE *, int);
+typedef Py_ssize_t printfunc;
 typedef PyObject *(*getattrfunc)(PyObject *, char *);
 typedef PyObject *(*getattrofunc)(PyObject *, PyObject *);
 typedef int (*setattrfunc)(PyObject *, char *, PyObject *);
@@ -282,6 +282,11 @@ typedef struct _typeobject {
     vectorcallfunc tp_vectorcall;
 
     printfunc tp_print; // deprecated, but stays around for compatibility
+
+    /* PyPy specific extra fields: make sure that they are ALWAYS at the end,
+       for compatibility with CPython */
+    long tp_pypy_flags;
+
 } PyTypeObject;
 
 typedef struct{
@@ -305,4 +310,10 @@ typedef struct _heaptypeobject {
     PySequenceMethods as_sequence;
     PyBufferProcs as_buffer;
     PyObject *ht_name, *ht_slots, *ht_qualname;
+    PyObject *ht_module;
 } PyHeapTypeObject;
+
+typedef struct {
+    PyCFunctionObject func;
+    PyTypeObject *mm_class; /* Class that defines this method */
+} PyCMethodObject;
