@@ -1,12 +1,14 @@
 from rpython.rtyper.lltypesystem import rffi, lltype
 from pypy.module.cpyext.api import cpython_api, CANNOT_FAIL
 from pypy.module.cpyext.state import State
+from pypy.module.cpyext.pyobject import PyObject
+from pypy.module.cpyext.pyerrors import PyErr_SetNone
 
 @cpython_api([], rffi.INT_real, error=CANNOT_FAIL)
 def Py_IsInitialized(space):
     return 1
 
-@cpython_api([], rffi.CCHARP, error=CANNOT_FAIL)
+@cpython_api([], rffi.CWCHARP, error=CANNOT_FAIL)
 def Py_GetProgramName(space):
     """
     Return the program name set with Py_SetProgramName(), or the default.
@@ -38,11 +40,15 @@ def Py_AtExit(space, func_ptr):
     called at most once.  Since Python's internal finalization will have
     completed before the cleanup function, no Python APIs should be called by
     func."""
-    from pypy.module import cpyext
+    from pypy.module.cpyext.moduledef import Module
     w_module = space.getbuiltinmodule('cpyext')
-    module = space.interp_w(cpyext.Module, w_module)
+    module = space.interp_w(Module, w_module)
     try:
         module.register_atexit(func_ptr)
     except ValueError:
         return -1
     return 0
+
+@cpython_api([], PyObject, error=CANNOT_FAIL)
+def PyThread_exit_thread(space):
+    PyErr_SetNone(space, space.w_SystemExit)
