@@ -2,12 +2,6 @@
 
 #include "Python.h"
 
-PyTypeObject PyFunction_Type;
-
-PyTypeObject PyMethod_Type;
-PyTypeObject PyRange_Type;
-PyTypeObject PyTraceBack_Type;
-
 int Py_DebugFlag = 1;
 int Py_VerboseFlag = 0;
 int Py_QuietFlag = 0;
@@ -34,7 +28,18 @@ const char *Py_FileSystemDefaultEncoding;  /* filled when cpyext is imported */
 void _Py_setfilesystemdefaultencoding(const char *enc) {
     Py_FileSystemDefaultEncoding = enc;
 }
+
+#ifdef CPYEXT_TESTS
+#define _Py_get_PyOS_InputHook _cpyexttest_get_PyOS_InputHook
+#ifdef __GNUC__
+__attribute__((visibility("default")))
+#else
+__declspec(dllexport)
+#endif
+#else  /* CPYEXT_TESTS */
+#define _Py_get_PyOS_InputHook _PyPy_get_PyOS_InputHook
+#endif  /* CPYEXT_TESTS */
 int (*PyOS_InputHook)(void) = 0;  /* only ever filled in by C extensions */
-PyAPI_FUNC(_pypy_pyos_inputhook) _PyPy_get_PyOS_InputHook(void) {
+PyAPI_FUNC(_pypy_pyos_inputhook) _Py_get_PyOS_InputHook(void) {
     return PyOS_InputHook;
 }
