@@ -136,7 +136,8 @@ class CpyextLeak(leakfinder.MallocMismatch):
 class LeakCheckingTest(object):
     """Base class for all cpyext tests."""
     spaceconfig = {"usemodules" : ['cpyext', 'thread', 'struct', 'array',
-                                   'itertools', 'time', 'binascii', 'mmap',
+                                   'itertools', 'time', 'binascii',
+                                   'mmap', 'signal',
                                    '_cffi_backend',
                                    ],
                    "objspace.disable_entrypoints_in_cffi": True}
@@ -773,15 +774,13 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
         module = self.import_module(name='foo', body=body)
 
         # uncaught interplevel exceptions are turned into SystemError
-        expected1 = "ZeroDivisionError('integer division or modulo by zero',)"
-        # win64 uses long internally not int, which gives a different error
-        expected2 = "ZeroDivisionError('integer division by zero',)"
+        expected = "ZeroDivisionError('integer division"
         exc = raises(SystemError, module.crash1)
-        v = exc.value.args[0]
-        assert v == expected1 or v == expected2
+        # Work around difference in err msg btween CPython2 and PyPy2
+        assert exc.value.args[0].startswith(expected)
 
         exc = raises(SystemError, module.crash2)
-        assert v == expected1 or v == expected2
+        assert exc.value.args[0].startswith(expected)
 
         # caught exception, api.cpython_api return value works
         assert module.crash3() == -1
@@ -789,7 +788,7 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
         expected = 'An exception was set, but function returned a value'
         # PyPy only incompatibility/extension
         exc = raises(SystemError, module.crash4)
-        assert v == expected1 or v == expected2
+        assert exc.value.args[0] == expected
 
         # An exception was set by the previous call, it can pass
         # cleanly through a call that doesn't check error state
@@ -848,18 +847,19 @@ class AppTestCpythonExtension(AppTestCpythonExtensionBase):
              '''
              /* XXX in tests, the C type is not correct */
              #define NAME(type) ((PyTypeObject*)&type)->tp_name
-             return Py_BuildValue("sssss",
+             return Py_BuildValue("ssssss",
                                   NAME(PyCell_Type),
                                   NAME(PyModule_Type),
                                   NAME(PyProperty_Type),
                                   NAME(PyStaticMethod_Type),
+                                  NAME(PyClassMethod_Type),
                                   NAME(PyCFunction_Type)
                                   );
              '''
              ),
             ])
         assert mod.get_names() == ('cell', 'module', 'property',
-                                   'staticmethod',
+                                   'staticmethod', 'classmethod',
                                    'builtin_function_or_method')
 
     def test_get_programname(self):
