@@ -313,10 +313,22 @@ class AppTestFetch(AppTestCpythonExtensionBase):
                  Py_DECREF(filenameObject);
                  return NULL;
                  '''),
+                 ("set_from_errno2", "METH_NOARGS",
+                 '''
+                 PyObject *filenameObject = PyUnicode_FromString("/path/to/file");
+                 errno = EBADF;
+                 PyErr_SetFromErrnoWithFilenameObjects(PyExc_OSError,
+                                                      filenameObject, filenameObject);
+                 Py_DECREF(filenameObject);
+                 return NULL;
+                 '''),
                 ],
                 prologue="#include <errno.h>")
         exc_info = raises(OSError, module.set_from_errno)
         assert exc_info.value.filename == "/path/to/file"
+        exc_info = raises(OSError, module.set_from_errno2)
+        assert exc_info.value.filename == "/path/to/file"
+        assert exc_info.value.filename2 == "/path/to/file"
         if self.runappdirect:
             # untranslated the errno can get reset by the calls to ll2ctypes
             assert exc_info.value.errno == errno.EBADF
@@ -665,4 +677,39 @@ class AppTestFetch(AppTestCpythonExtensionBase):
         msg = output.strip().replace('\r', '').splitlines()
         assert msg[0] == "sys.unraisablehook RuntimeError('nonfatal-error') Exception ignored sometext"
  
-
+    def test_fetch_normalized(self):
+        module = self.import_extension('foo', [
+            ("clevel_error", "METH_O",
+             '''
+                const char *fname = PyUnicode_AsUTF8(args);
+                char buffer[1024];
+                sprintf(buffer, "open('%s', 'r')", fname);
+                // printf("calling %s\\n", buffer);
+                PyObject *ret = PyRun_String(buffer, Py_eval_input, pyx_d, pyx_d);
+                if (ret) {
+                    Py_DECREF(ret);
+                    PyErr_SetString(PyExc_AssertionError, "should raise");
+                    return NULL;
+                }
+                PyObject *type, *value, *tb;
+                PyErr_Fetch(&type, &value, &tb);
+                if (type != PyExc_FileNotFoundError)
+                {
+                    // printf("type is %s\\n", ((PyTypeObject*)type)->tp_name);
+                    return value;
+                }
+                PyErr_Clear();
+                // decrefs?
+                Py_RETURN_TRUE;
+             '''),
+            ], prologue="""
+            static PyObject * pyx_d;
+            """, more_init="""
+            pyx_d = PyModule_GetDict(mod);
+            """)
+        import os
+        fname = 'this file should not exist'
+        assert not os.path.exists(fname)
+        ret = module.clevel_error(fname)
+        print(ret)
+        assert ret is True
