@@ -129,22 +129,32 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         assert obj.foo == 42
         obj2 = obj.create()
         assert obj2.foo == 42
+        assert obj2.create.__qualname__ == "foo.create"
 
     def test_classmethod(self):
         module = self.import_module(name="foo")
         classmeth = module.fooType.classmeth
+        assert classmeth.__qualname__ == "foo.classmeth"
         obj = classmeth()
         assert obj is module.fooType
         class _C:
             def _m(self): pass
         MethodType = type(_C()._m)
-        print(type(classmeth).mro())
-        print(MethodType.mro())
         assert not isinstance(classmeth, MethodType)
 
+    def test_class_getitem(self):
+        module = self.import_module(name='foo')
+        f = module.fooType.__class_getitem__
+        print(dir(f))
+        assert f.__qualname__ == "foo.__class_getitem__"
+        out = f(42)
+        assert str(out) == 'foo.foo[42]'
+
     def test_methoddescr(self):
         module = self.import_module(name='foo')
         descr = module.fooType.copy
+        print(descr.__qualname__)
+        assert descr.__qualname__ == "foo.copy"
         assert type(descr).__name__ == 'method_descriptor'
         assert str(descr) in ("<method 'copy' of 'foo.foo' objects>",
             "<method 'copy' of 'foo' objects>")
@@ -856,7 +866,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                  return Py_None;
              '''
              )
-            ])
+        ])
         x = [42]
         assert module.tp_init(list, x, ("hi",)) is None
         assert x == ["h", "i"]
@@ -867,6 +877,15 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         assert module.tp_init(list, x, ("hi",)) is None
         assert x == ["h", "i"]
 
+        init_called = []
+        class CALL(object):
+            def __init__(self):
+                init_called.append(42)
+        x = object.__new__(CALL)
+        x.__init__()
+        module.tp_init(CALL, x, ())
+        assert len(init_called) == 2, '%s' % len(init_called)
+
     def test_mp_subscript(self):
         module = self.import_extension('foo', [
            ("new_obj", "METH_NOARGS",
@@ -1290,7 +1309,6 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         except TypeError as e:
             import sys
             if '__pypy__' in sys.builtin_module_names:
-                print(str(e))
                 assert 'instance layout conflicts in multiple inheritance' in str(e)
 
             else:
@@ -1721,7 +1739,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                     PyErr_SetString(PyExc_TypeError, "kwnames must be None or a tuple");
                     return NULL;
                 }
-                return _PyObject_Vectorcall(func, stack, nargs, kwnames);
+                return PyObject_Vectorcall(func, stack, nargs, kwnames);
             '''),
             ("pyvectorcall_call", "METH_VARARGS",
              # taken from _testcapimodule.c
@@ -1789,7 +1807,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                     .tp_call = PyVectorcall_Call,
                     .tp_vectorcall_offset = offsetof(MethodDescriptorObject, vectorcall),
                     .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE |
-                                Py_TPFLAGS_METHOD_DESCRIPTOR | _Py_TPFLAGS_HAVE_VECTORCALL,
+                                Py_TPFLAGS_METHOD_DESCRIPTOR | Py_TPFLAGS_HAVE_VECTORCALL,
                     .tp_descr_get = func_descr_get,
                 };
 
@@ -1820,7 +1838,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                     .tp_new = MethodDescriptor2_new,
                     .tp_call = PyVectorcall_Call,
                     .tp_vectorcall_offset = offsetof(MethodDescriptor2Object, vectorcall),
-                    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | _Py_TPFLAGS_HAVE_VECTORCALL,
+                    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_VECTORCALL,
                 };
 
 
@@ -1980,7 +1998,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                     PyErr_SetString(PyExc_TypeError, "kwnames must be None or a dict");
                     return NULL;
                 }
-                return _PyObject_FastCallDict(func, stack, nargs, kwargs);
+                return PyObject_VectorcallDict(func, stack, nargs, kwargs);
             ''')])
         def pyfunc(arg1, arg2):
             return [arg1, arg2]
@@ -1991,18 +2009,6 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         res = module.test_fastcalldict(pyfunc, (1, ), {"arg2": 2})
         assert res == [1, 2]
 
-    def test_call_no_args(self):
-        module = self.import_extension('foo', [
-            ("test_callnoarg", "METH_VARARGS",
-             '''
-                PyObject *func = NULL;
-                if (!PyArg_ParseTuple(args, "O", &func)) {
-                    return NULL;
-                }
-                return _PyObject_CallNoArg(func);
-            ''')])
-        assert module.test_callnoarg(lambda : 4) == 4
-
 
 class AppTestHashable(AppTestCpythonExtensionBase):
     def test_unhashable(self):
@@ -2080,6 +2086,32 @@ class AppTestFlags(AppTestCpythonExtensionBase):
             pass
         assert module.test_flags(MyList, Py_TPFLAGS_LIST_SUBCLASS) == 0
 
+    def test_has_pypy_subclass_flag(self):
+        module = self.import_extension('foo', [
+           ("test_pypy_flags", "METH_VARARGS",
+            '''
+                long long in_flag, my_flag;
+                PyObject * obj;
+                if (!PyArg_ParseTuple(args, "OL", &obj, &in_flag))
+                    return NULL;
+                if (!PyType_Check(obj))
+                {
+                    PyErr_SetString(PyExc_ValueError, "input must be type");
+                    return NULL;
+                }
+                my_flag = ((PyTypeObject*)obj)->tp_pypy_flags;
+                if ((my_flag & in_flag) != in_flag)
+                    return PyLong_FromLong(-1);
+                return PyLong_FromLong(0);
+            '''),])
+        # copied from object.h
+        Py_TPPYPYFLAGS_FLOAT_SUBCLASS = (1<<0)
+
+        class MyFloat(float):
+            pass
+        assert module.test_pypy_flags(float, Py_TPPYPYFLAGS_FLOAT_SUBCLASS) == 0
+        assert module.test_pypy_flags(MyFloat, Py_TPPYPYFLAGS_FLOAT_SUBCLASS) == 0
+
     def test_newgetset(self):
         # Taken from the yara-python project
         module = self.import_extension('foo', [
@@ -2119,3 +2151,124 @@ class AppTestFlags(AppTestCpythonExtensionBase):
         errtype = module.newexc()
         err = errtype("abc")
         assert err.warnings == "abc"
+
+    def test_heaptype_dealloc(self):
+        # Taken from https://github.com/wjakob/pypy_issues at commit 03890103
+        import gc
+        module = self.import_module(name='nanobind1', filename="nanobind1")
+        for i in range(100):
+            module.heap_type()
+            gc.collect()
+
+    def test_heaptype_metaclass(self):
+        # Taken from https://github.com/wjakob/pypy_issues at commit 03890103
+        import gc
+        module = self.import_module(name='nanobind1', filename="nanobind1")
+        # 2 vs 3 shenanigans to declare
+        # class X(object, metaclass=module.metaclass): pass
+        X = module.metaclass_good('X', (object,), {})
+        x = X()
+
+        import sys
+        if '__pypy__' in sys.builtin_module_names:
+            exc = raises(SystemError, module.metaclass_bad, 'X', (object,), {})
+            assert 'tp_itemsize' in str(exc.value)
+
+    def test_vectorcall2(self):
+        # Taken from https://github.com/wjakob/pypy_issues at commit 03890103
+        # py3.9+ only
+        module = self.import_module(name='nanobind1', filename="nanobind1")
+        c = module.callable()
+        assert c() == 1234
+        
+        def f(value):
+            assert value == 1234
+
+        module.call(f)
+
+    def test_nanobind2_tp_traverse(self):
+        # Taken from https://github.com/wjakob/pypy_issues at commit 89a8585
+        import gc
+        import sys
+        if sys.implementation.name == 'pypy':
+            skip("tp_traverse not yet implemented in PyPy")
+        module = self.import_module(name='nanobind2', filename="nanobind2")
+        # Create an unreferenced cycle
+        a = module.wrapper()
+        a.nested = a
+        del a
+        for i in range(5):
+            gc.collect()
+        gl = module.global_list
+        assert gl == ['wrapper tp_init called.',
+                      'wrapper tp_traverse called.',
+                      'wrapper tp_traverse called.',
+                      'wrapper tp_clear called.',
+                      'wrapper tp_dealloc called.',
+                     ]
+
+    def test_nanobind2_module_attributes(self):
+        # Taken from https://github.com/wjakob/pypy_issues at commit 89a8585
+        import sys
+        module = self.import_module(name='nanobind2', filename="nanobind2")
+
+        f = module.func()
+        if sys.version_info >= (3, 9) or sys.implementation.name == 'pypy':
+            assert f.__module__   == "my_module"
+        else:
+            assert f.__module__   == "nanobind2"
+        assert f.__name__ == "my_name"
+        assert f.__qualname__ == "my_qualname"
+
+    def test_nanobind2_vectorcall_method(self):
+        # Taken from https://github.com/wjakob/pypy_issues at commit 89a8585
+        module = self.import_module(name='nanobind2', filename="nanobind2")
+
+        class A:
+            def __init__(self):
+                self.value = 0
+
+            def my_method(self, *args, **kwargs):
+                self.args = args
+                self.kwargs = kwargs
+                return "success"
+
+        a = A()
+        assert module.method_call(a) == "success"
+        assert a.args == (1234,) and a.kwargs == {}
+        assert module.method_call_kw(a) == "success"
+        assert a.args == (1234,) and a.kwargs == {"foo" : "bar"}
+
+
+        def unbound_method(*args, **kwargs):
+            args_value.update(args)
+            kwargs_value.update(kwargs)
+            return "unbound"
+
+        a.my_method = unbound_method
+        kwargs_value = {}
+        args_value = set()
+
+
+        assert module.method_call(a) == "unbound"
+        assert args_value == set([1234]) and kwargs_value == {}
+        assert module.method_call_kw(a) == "unbound"
+        assert args_value == set([1234]) and kwargs_value == {"foo" : "bar"}
+
+    def test_nanobind3(self):
+        module = self.import_module(name='nanobind3', filename="nanobind3")
+        old_list = module.global_list[:]
+
+        o = module.my_object()
+        c = module.my_callable()
+
+        with raises(ValueError):
+            c(o)
+
+        old_list = module.global_list[:]
+        del o
+        self.debug_collect()  # will call gc.collect unless run untranslated
+
+        # Make sure o.tp_dealloc was called
+        new_list = module.global_list[:]
+        assert len(new_list) == len(old_list) + 1, "%s %s" %(old_list, new_list)
