@@ -21,11 +21,12 @@ from pypy.module.cpyext.api import (
     Py_TPFLAGS_DICT_SUBCLASS, Py_TPFLAGS_BASE_EXC_SUBCLASS,
     Py_TPFLAGS_TYPE_SUBCLASS,
     Py_TPFLAGS_BYTES_SUBCLASS,
+    Py_TPPYPYFLAGS_FLOAT_SUBCLASS,
     )
 
 from rpython.tool.cparser import CTypeSpace
 from pypy.module.cpyext.methodobject import (W_PyCClassMethodObject,
-    PyCFunction_NewEx, PyCFunction, PyMethodDef,
+    PyCFunction, PyMethodDef,
     W_PyCMethodObject, W_PyCFunctionObject, extract_doc, extract_txtsig,
     W_PyCWrapperObject)
 from pypy.module.cpyext.modsupport import convert_method_defs
@@ -38,7 +39,7 @@ from pypy.module.cpyext.slotdefs import (
 from pypy.module.cpyext.state import State
 from pypy.module.cpyext.structmember import PyMember_GetOne, PyMember_SetOne
 from pypy.module.cpyext.typeobjectdefs import (
-    PyGetSetDef, PyMemberDef, PyMappingMethods, printfunc,
+    PyGetSetDef, PyMemberDef, PyMappingMethods,
     PyNumberMethods, PySequenceMethods, PyBufferProcs)
 from pypy.objspace.std.typeobject import W_TypeObject, find_best_base
 
@@ -229,7 +230,8 @@ def classmethoddescr_realize(space, obj):
     method = rffi.cast(lltype.Ptr(PyMethodDef), obj)
     w_type = from_ref(space, rffi.cast(PyObject, obj.c_ob_type))
     w_obj = space.allocate_instance(W_PyCClassMethodObject, w_type)
-    w_obj.__init__(space, method, w_type)
+    assert isinstance(w_type, W_TypeObject)
+    w_obj.__init__(space, method, w_type, w_type.qualname)
     track_reference(space, obj, w_obj)
     return w_obj
 
@@ -238,7 +240,8 @@ def methoddescr_realize(space, obj):
     method = rffi.cast(lltype.Ptr(PyMethodDef), obj)
     w_type = from_ref(space, rffi.cast(PyObject, obj.c_ob_type))
     w_obj = space.allocate_instance(W_PyCMethodObject, w_type)
-    w_obj.__init__(space, method, w_type)
+    assert isinstance(w_type, W_TypeObject)
+    w_obj.__init__(space, method, None, None, w_type, w_type.qualname)
     track_reference(space, obj, w_obj)
     return w_obj
 
@@ -384,7 +387,7 @@ def fill_slot(space, pto, w_type, slot_names, slot_func_helper):
 
         setattr(struct, slot_names[1], slot_func_helper)
 
-def add_operators(space, w_type, dict_w, pto, name):
+def _add_operators(space, w_type, dict_w, pto, type_name):
     from pypy.module.cpyext.object import PyObject_HashNotImplemented
     hash_not_impl = llslot(space, PyObject_HashNotImplemented)
     for method_name, slot_names, wrapper_class, doc in slotdefs_for_wrappers:
@@ -415,13 +418,8 @@ def add_operators(space, w_type, dict_w, pto, name):
 
         assert issubclass(wrapper_class, W_PyCWrapperObject)
 
-        w_obj = wrapper_class(space, w_type, method_name, doc, func_voidp)
+        w_obj = wrapper_class(space, w_type, method_name, doc, func_voidp, type_name)
         dict_w[method_name] = w_obj
-    if pto.c_tp_doc:
-        raw_doc = rffi.constcharp2str(pto.c_tp_doc)
-        dict_w['__doc__'] = space.newtext_or_none(extract_doc(raw_doc, name))
-    if pto.c_tp_new:
-        add_tp_new_wrapper(space, dict_w, pto)
 
 @slot_function([PyObject, PyObject, PyObject], PyObject)
 def tp_new_wrapper(space, self, w_args, w_kwds):
@@ -472,7 +470,7 @@ def add_tp_new_wrapper(space, dict_w, pto):
     if "__new__" in dict_w:
         return
     pyo = rffi.cast(PyObject, pto)
-    dict_w["__new__"] = PyCFunction_NewEx(space, get_new_method_def(space),
+    dict_w["__new__"] = W_PyCFunctionObject(space, get_new_method_def(space),
                                           from_ref(space, pyo), None)
 
 def inherit_special(space, pto, w_obj, base_pto):
@@ -501,10 +499,9 @@ def inherit_special(space, pto, w_obj, base_pto):
         flags |= Py_TPFLAGS_LIST_SUBCLASS
     elif space.issubtype_w(w_obj, space.w_dict):
         flags |= Py_TPFLAGS_DICT_SUBCLASS
+    # the following types are a pypy-specific extensions, using tp_pypy_flags
     elif space.issubtype_w(w_obj, space.w_float):
-        # Py3.8 v7.3.7 compatibility. tp_pypy_flags
-        # in v7.3.7 was repurposed to tp_print in v7.3.8
-        pto.c_tp_print = rffi.cast(printfunc, 1)
+        pto.c_tp_pypy_flags = rffi.cast(rffi.LONG, widen(pto.c_tp_pypy_flags) | Py_TPPYPYFLAGS_FLOAT_SUBCLASS)
     pto.c_tp_flags = rffi.cast(rffi.ULONG, flags)
 
 def check_descr(space, w_self, w_type):
@@ -569,6 +566,21 @@ class GettersAndSetters:
         finally:
             decref(space, pyref)
 
+def get_type_name(name, is_heaptype):
+    # This is a refactored copy of W_TypeObject.getname()
+    # we cannot use self.getname since the self is not fully initialized
+    if is_heaptype:
+        result = name
+    else:
+        dot = name.rfind('.')
+        if dot >= 0:
+            result = name[dot+1:]
+        else:
+            result = name
+    return result
+
+
+
 class W_PyCTypeObject(W_TypeObject):
     @jit.dont_look_inside
     def __init__(self, space, pto):
@@ -576,16 +588,19 @@ class W_PyCTypeObject(W_TypeObject):
         dict_w = {}
 
         flag_heaptype = widen(pto.c_tp_flags) & Py_TPFLAGS_HEAPTYPE
-        if flag_heaptype:
-            hto = cts.cast('PyHeapTypeObject*', pto)
-            name = space.text_w(from_ref(space, hto.c_ht_name))
-        else:
-            name = rffi.constcharp2str(pto.c_tp_name)
-        add_operators(space, self, dict_w, pto, name)
-        convert_method_defs(space, dict_w, pto.c_tp_methods, self)
+        name = rffi.constcharp2str(pto.c_tp_name)
+        type_name = get_type_name(name, flag_heaptype)
+        _add_operators(space, self, dict_w, pto, type_name)
+        convert_method_defs(space, dict_w, pto.c_tp_methods, self, type_name=type_name)
         convert_getset_defs(space, dict_w, pto.c_tp_getset, self)
         convert_member_defs(space, dict_w, pto.c_tp_members, self)
 
+        if pto.c_tp_doc:
+            raw_doc = rffi.constcharp2str(pto.c_tp_doc)
+            dict_w['__doc__'] = space.newtext_or_none(extract_doc(raw_doc, name))
+        if pto.c_tp_new:
+            add_tp_new_wrapper(space, dict_w, pto)
+
         w_dict = from_ref(space, pto.c_tp_dict)
         if w_dict is not None:
             dictkeys_w = space.listview(w_dict)
@@ -652,9 +667,12 @@ def type_alloc(typedescr, space, w_metatype, itemsize=0):
         if not flags & Py_TPFLAGS_HEAPTYPE:
             decref(space, metatype)
 
-    heaptype = lltype.malloc(PyHeapTypeObject.TO,
+    basicsize = max(rffi.sizeof(PyHeapTypeObject.TO), metatype.c_tp_basicsize)
+    heaptype = lltype.malloc(rffi.VOIDP.TO,
+                             basicsize,
                              flavor='raw', zero=True,
                              add_memory_pressure=True)
+    heaptype = rffi.cast(PyHeapTypeObject, heaptype)
     pto = heaptype.c_ht_type
     rffi.cast(PyObject, pto).c_ob_refcnt = 1
     rffi.cast(PyObject, pto).c_ob_pypy_link = 0
@@ -680,7 +698,7 @@ def type_attach(space, py_obj, w_type, w_userdata=None):
 
     typedescr = get_typedescr(w_type.layout.typedef)
 
-    if space.is_w(w_type, space.w_bytes) or space.is_w(w_type, space.w_text):
+    if space.is_w(w_type, space.w_bytes):
         pto.c_tp_itemsize = 1
     elif space.is_w(w_type, space.w_tuple):
         pto.c_tp_itemsize = rffi.sizeof(PyObject)
@@ -845,7 +863,7 @@ def _type_realize(space, py_obj):
     if w_obj is None:
         w_obj = space.allocate_instance(W_PyCTypeObject, w_metatype)
         track_reference(space, py_obj, w_obj)
-    # __init__ wraps all slotdefs functions from py_type via add_operators
+    # __init__ wraps all slotdefs functions from py_type via _add_operators
     w_obj.__init__(space, py_type)
     w_obj.ready()
 
@@ -965,6 +983,12 @@ def get_ht_slot(ht, slotnum):
     PyType_FromSpecWithBases(PyType_Spec *spec, PyObject *bases)""",
     result_is_ll=True)
 def PyType_FromSpecWithBases(space, spec, bases):
+    return PyType_FromSpecWithBases(space, None, spec, bases)
+
+@cts.decl("""PyObject *
+    PyType_FromModuleAndSpec(PyObject *, PyType_Spec *spec, PyObject *bases)""",
+    result_is_ll=True)
+def PyType_FromSpecWithBases(space, module, spec, bases):
     from pypy.module.cpyext.unicodeobject import PyUnicode_FromString
     state = space.fromcache(State)
     p_type = cts.cast('PyTypeObject*', make_ref(space, space.w_type))
@@ -983,7 +1007,10 @@ def PyType_FromSpecWithBases(space, spec, bases):
     res.c_ht_name = make_ref(space, space.newtext(name))
     res.c_ht_qualname = res.c_ht_name
     incref(space, res.c_ht_qualname)
-    typ.c_tp_name = spec.c_name
+    typ.c_tp_name = cts.cast('const char*', rffi.str2charp(name))
+    if module:
+        incref(space, module)
+        res.c_ht_module = module
     slotdefs = rffi.cast(rffi.CArrayPtr(cts.gettype('PyType_Slot')), spec.c_slots)
     if not bases:
         w_base = space.w_object
@@ -1033,6 +1060,29 @@ def PyType_FromSpecWithBases(space, spec, bases):
             # Processed above
             i += 1
             continue
+        if slot == cts.macros['Py_tp_members']:
+            members = rffi.cast(rffi.CArrayPtr(PyMemberDef), slotdef.c_pfunc)
+            if members:
+                m = 0
+                while True:
+                    member = members[m]
+                    name = member.c_name
+                    if not name:
+                        break
+                    name = rffi.constcharp2str(name)
+                    if name == "__weaklistoffset__":
+                        assert widen(member.c_type) == structmemberdefs.T_PYSSIZET
+                        assert widen(member.c_flags) == structmemberdefs.READONLY
+                        typ.c_tp_weaklistoffset = member.c_offset
+                    elif name == "__dictoffset__":
+                        assert widen(member.c_type) == structmemberdefs.T_PYSSIZET
+                        assert widen(member.c_flags) == structmemberdefs.READONLY
+                        typ.c_tp_dictoffset = member.c_offset
+                    elif name == "__vectorcalloffset__":
+                        assert widen(member.c_type) == structmemberdefs.T_PYSSIZET
+                        assert widen(member.c_flags) == structmemberdefs.READONLY
+                        typ.c_tp_vectorcall_offset = member.c_offset
+                    m += 1
         fill_ht_slot(res, slot, slotdef.c_pfunc)
         # XXX: need to make a copy of the docstring slot, which usually
         # points to a static string literal
@@ -1040,14 +1090,28 @@ def PyType_FromSpecWithBases(space, spec, bases):
 
     if not typ.c_tp_dealloc:
         typ.c_tp_dealloc = state.C._PyPy_subtype_dealloc
+
     py_type_ready(space, typ)
     res = cts.cast('PyObject*', res)
     if modname is not None:
         w_type = from_ref(space, res)
         w_type.setdictvalue(space, '__module__', space.newtext(modname))
+    # Convert getsets
+    if typ.c_tp_getset:
+        w_type = from_ref(space, res)
+        getsets = rffi.cast(rffi.CArrayPtr(PyGetSetDef), typ.c_tp_getset)
+        i = -1
+        while True:
+            i = i + 1
+            getset = getsets[i]
+            name = getset.c_name
+            if not name:
+                break
+            name = rffi.constcharp2str(name)
+            w_descr = W_GetSetPropertyEx(getset, w_type)
+            w_type.setdictvalue(space, name, w_descr)
     return res
 
-
 @cpython_api([PyTypeObjectPtr, rffi.INT], rffi.VOIDP)
 def PyType_GetSlot(space, typ, slot):
     """ Use the Py_tp* macros in typeslots.h to return a slot function
@@ -1085,3 +1149,8 @@ def PyType_Modified(space, w_obj):
         return
     if w_obj.is_cpytype():
         w_obj.mutated(None)
+
+@cpython_api([PyObject, PyObject], PyObject, header='genericaliasobject.h')
+def Py_GenericAlias(space, w_cls, w_item):
+    from pypy.objspace.std.util import generic_alias_class_getitem
+    return generic_alias_class_getitem(space, w_cls, w_item)
