@@ -14,14 +14,16 @@ from pypy.module.cpyext.api import (
     CANNOT_FAIL, Py_ssize_t, cpython_api,
     bootstrap_function, CONST_STRING, INTP_real, Py_TPFLAGS_UNICODE_SUBCLASS,
     CONST_WSTRING, Py_CLEANUP_SUPPORTED, slot_function, cts, parse_dir,
-    PyTypeObjectPtr, PyVarObject)
-from pypy.module.cpyext.pyerrors import PyErr_BadArgument
+    PyTypeObjectPtr, PyVarObject, PY_SSIZE_T_MAX)
+from pypy.module.cpyext.pyerrors import PyErr_BadArgument, PyErr_BadInternalCall
 from pypy.module.cpyext.pyobject import (
     PyObject, PyObjectP, decref, make_ref, from_ref, track_reference,
-    make_typedescr, get_typedescr, as_pyobj, pyobj_has_w_obj, BaseCpyTypedescr)
+    make_typedescr, get_typedescr, as_pyobj, pyobj_has_w_obj, BaseCpyTypedescr,
+    incref, decref)
 from pypy.module.cpyext.bytesobject import PyBytes_Check, PyBytes_FromObject
 from pypy.module._codecs.interp_codecs import (
     CodecState, latin_1_decode, utf_16_decode, utf_32_decode)
+from pypy.module.cpyext.state import State
 from pypy.objspace.std import unicodeobject
 from rpython.rlib.debug import fatalerror
 import sys
@@ -85,7 +87,6 @@ def PyUnicode_CheckExact(space, ref):
     w_obj = from_ref(space, rffi.cast(PyObject, ref))
     w_obj_type = space.type(w_obj)
     return space.is_w(w_obj_type, space.w_unicode)
- 
 
 def new_empty_unicode(space, length):
     """
@@ -151,8 +152,10 @@ def unicode_alloc(typedescr, space, w_type, itemcount):
         # subclass, will not use compact format, so no need for extra space
         return BaseCpyTypedescr.allocate(typedescr, space, w_type, 1)
     # This could be optimized: PyUnicodeObject is 80 bytes, PyASCIIObject
-    # is 48. In any case, leave room for a NULL char
-    return BaseCpyTypedescr.allocate(typedescr, space, w_type, itemcount + 1)
+    # is 48. In any case, leave room for a NULL char. Also override the
+    # tp_itemsize since we use the compact form
+    return BaseCpyTypedescr.allocate(typedescr, space, w_type, itemcount + 1,
+                                     itemsize=1)
 
 @slot_function([PyObject], lltype.Void)
 def unicode_dealloc(space, py_obj):
@@ -285,74 +288,116 @@ def set_compact(py_obj, value):
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISSPACE(space, ch):
     """Return 1 or 0 depending on whether ch is a whitespace character."""
-    return unicodedb.isspace(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.isspace(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISALPHA(space, ch):
     """Return 1 or 0 depending on whether ch is an alphabetic character."""
-    return unicodedb.isalpha(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.isalpha(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISALNUM(space, ch):
     """Return 1 or 0 depending on whether ch is an alphanumeric character."""
-    return unicodedb.isalnum(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.isalnum(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISLINEBREAK(space, ch):
     """Return 1 or 0 depending on whether ch is a linebreak character."""
-    return unicodedb.islinebreak(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.islinebreak(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISDECIMAL(space, ch):
     """Return 1 or 0 depending on whether ch is a decimal character."""
-    return unicodedb.isdecimal(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.isdecimal(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISDIGIT(space, ch):
     """Return 1 or 0 depending on whether ch is a digit character."""
-    return unicodedb.isdigit(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.isdigit(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISNUMERIC(space, ch):
     """Return 1 or 0 depending on whether ch is a numeric character."""
-    return unicodedb.isnumeric(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.isnumeric(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISLOWER(space, ch):
     """Return 1 or 0 depending on whether ch is a lowercase character."""
-    return unicodedb.islower(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.islower(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISUPPER(space, ch):
     """Return 1 or 0 depending on whether ch is an uppercase character."""
-    return unicodedb.isupper(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.isupper(ch)
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_ISTITLE(space, ch):
     """Return 1 or 0 depending on whether ch is a titlecase character."""
-    return unicodedb.istitle(ord(ch))
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return 0
+    return unicodedb.istitle(ch)
 
 @cpython_api([Py_UNICODE], Py_UNICODE, error=CANNOT_FAIL)
 def Py_UNICODE_TOLOWER(space, ch):
     """Return the character ch converted to lower case."""
-    return unichr(unicodedb.tolower(ord(ch)))
+    val = ord(ch)
+    if val >= rutf8.MAXUNICODE:
+        return ch
+    return unichr(unicodedb.tolower(val))
 
 @cpython_api([Py_UNICODE], Py_UNICODE, error=CANNOT_FAIL)
 def Py_UNICODE_TOUPPER(space, ch):
     """Return the character ch converted to upper case."""
-    return unichr(unicodedb.toupper(ord(ch)))
+    val = ord(ch)
+    if val >= rutf8.MAXUNICODE:
+        return ch
+    return unichr(unicodedb.toupper(val))
 
 @cpython_api([Py_UNICODE], Py_UNICODE, error=CANNOT_FAIL)
 def Py_UNICODE_TOTITLE(space, ch):
     """Return the character ch converted to title case."""
-    return unichr(unicodedb.totitle(ord(ch)))
+    val = ord(ch)
+    if val >= rutf8.MAXUNICODE:
+        return ch
+    return unichr(unicodedb.totitle(val))
 
 @cpython_api([Py_UNICODE], rffi.INT_real, error=CANNOT_FAIL)
 def Py_UNICODE_TODECIMAL(space, ch):
     """Return the character ch converted to a decimal positive integer.  Return
     -1 if this is not possible.  This macro does not raise exceptions."""
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return -1
     try:
-        return unicodedb.decimal(ord(ch))
+        return unicodedb.decimal(ch)
     except KeyError:
         return -1
 
@@ -360,8 +405,11 @@ def Py_UNICODE_TODECIMAL(space, ch):
 def Py_UNICODE_TODIGIT(space, ch):
     """Return the character ch converted to a single digit integer. Return -1 if
     this is not possible.  This macro does not raise exceptions."""
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return -1
     try:
-        return unicodedb.digit(ord(ch))
+        return unicodedb.digit(ch)
     except KeyError:
         return -1
 
@@ -369,8 +417,11 @@ def Py_UNICODE_TODIGIT(space, ch):
 def Py_UNICODE_TONUMERIC(space, ch):
     """Return the character ch converted to a double. Return -1.0 if this is not
     possible.  This macro does not raise exceptions."""
+    ch = ord(ch)
+    if ch >= rutf8.MAXUNICODE:
+        return -1.0
     try:
-        return unicodedb.numeric(ord(ch))
+        return unicodedb.numeric(ch)
     except KeyError:
         return -1.0
 
@@ -755,7 +806,7 @@ def PyUnicode_FromEncodedObject(space, w_obj, encoding, errors):
 
 @cpython_api([PyObject, CONST_STRING], PyObject)
 def PyUnicode_EncodeLocale(space, w_obj, errors):
-    from pypy.module._codecs.locale import utf8_encode_locale 
+    from pypy.module._codecs.locale import utf8_encode_locale
     if errors:
         s = rffi.charp2str(errors)
     else:
@@ -769,7 +820,7 @@ def PyUnicode_EncodeLocale(space, w_obj, errors):
 
 @cpython_api([CONST_STRING, CONST_STRING], PyObject)
 def PyUnicode_DecodeLocale(space, obj, errors):
-    from pypy.module._codecs.locale import str_decode_locale 
+    from pypy.module._codecs.locale import str_decode_locale
     if errors:
         s = rffi.charp2str(errors)
     else:
@@ -782,7 +833,7 @@ def PyUnicode_DecodeLocale(space, obj, errors):
 
 @cpython_api([CONST_STRING, Py_ssize_t, CONST_STRING], PyObject)
 def PyUnicode_DecodeLocaleAndSize(space, obj, length, errors):
-    from pypy.module._codecs.locale import str_decode_locale 
+    from pypy.module._codecs.locale import str_decode_locale
     if errors:
         s = rffi.charp2str(errors)
     else:
@@ -902,7 +953,6 @@ def PyUnicode_InternInPlace(space, string):
     decref(space, string[0])
     string[0] = make_ref(space, w_str)
 
-
 @cpython_api([CONST_STRING], PyObject)
 def PyUnicode_InternFromString(space, s):
     """A combination of PyUnicode_FromString() and
@@ -1014,7 +1064,13 @@ make_conversion_functions('UTF32', 'utf-32', only_for_asstring=True)
 make_conversion_functions('ASCII', 'ascii')
 make_conversion_functions('Latin1', 'latin-1')
 if sys.platform == 'win32':
+    from pypy.module._codecs.interp_codecs import code_page_encode
     make_conversion_functions('MBCS', 'mbcs')
+    @cpython_api([rffi.INT_real, PyObject, CONST_STRING], PyObject)
+    def PyUnicode_EncodeCodePage(space, code_page, w_obj, errors):
+        res = code_page_encode(space, widen(code_page), w_obj,
+                               rffi.charp2str(errors))
+        return space.listview(res)[0]
 
 @cpython_api([CONST_STRING, Py_ssize_t, CONST_STRING, INTP_real], PyObject)
 def PyUnicode_DecodeUTF16(space, s, size, llerrors, pbyteorder):
@@ -1188,6 +1244,44 @@ def PyUnicode_Concat(space, w_left, w_right):
     return space.add(w_left, w_right)
 
 @cpython_api([PyObject, CONST_STRING], rffi.INT_real, error=CANNOT_FAIL)
+def _PyUnicode_EqualToASCIIString(space, w_uni, string):
+    """Test whether a unicode is equal to ASCII string.  Return 1 if true,
+   0 otherwise.  The right argument must be ASCII-encoded string.
+   Any error occurs inside will be cleared before return."""
+    utf8 = space.utf8_w(w_uni)
+    lgt = space.len_w(w_uni)
+    i = 0
+    # Compare Unicode string and source character set string
+    for ch in rutf8.Utf8StringIterator(utf8):
+        if string[i] == '\0':
+            break
+        s = ord(string[i])
+        if ch != s:
+            if ch != s:
+                return 0
+        i += 1
+    if i < lgt:
+        return 0  # uni is longer
+    if string[i] != '\0':
+        return 0  # str is longer
+    return 1
+
+@cpython_api([PyObject, PyObject], rffi.INT_real, error=CANNOT_FAIL)
+def _PyUnicode_EQ(space, w_aa, w_bb):
+    if not space.isinstance_w(w_aa, space.w_unicode) or not space.isinstance_w(w_bb, space.w_unicode):
+        raise oefmt(space.w_TypeError, "_PyUnicode_EQ(aa, bb) must be called with two str instances")
+    aa = space.utf8_w(w_aa)
+    la = space.len_w(w_aa)
+    bb = space.utf8_w(w_bb)
+    lb = space.len_w(w_bb)
+    if la != lb:
+        return 0
+    if la == 0:
+        return 1
+    if aa == bb:
+        return 1
+    return 0 
+@cpython_api([PyObject, CONST_STRING], rffi.INT_real, error=CANNOT_FAIL)
 def PyUnicode_CompareWithASCIIString(space, w_uni, string):
     """Compare a unicode object, uni, with string and return -1, 0, 1 for less
     than, equal, and greater than, respectively. It is best to pass only
@@ -1421,7 +1515,7 @@ def PyUnicode_New(space, size, maxchar):
     else:
         unicode_size = rffi.sizeof(PyUnicodeObject.TO)
         data = rffi.ptradd(rffi.cast(rffi.CCHARP, pyobj), unicode_size)
-        set_data(pyobj, cts.cast('void *', data)) 
+        set_data(pyobj, cts.cast('void *', data))
     if is_sharing:
         set_wbuffer(pyobj, rffi.cast(rffi.CWCHARP, get_data(pyobj)))
     if not is_ascii:
@@ -1429,7 +1523,7 @@ def PyUnicode_New(space, size, maxchar):
     set_ready(pyobj, True)
     return pyobj
 
-@cts.decl("""Py_ssize_t PyUnicode_FindChar(PyObject *str, Py_UCS4 ch, 
+@cts.decl("""Py_ssize_t PyUnicode_FindChar(PyObject *str, Py_UCS4 ch,
           Py_ssize_t start, Py_ssize_t end, int direction)""", error=-1)
 def PyUnicode_FindChar(space, ref, ch, start, end, direction):
     if not pyunicode_check(ref):
@@ -1458,7 +1552,7 @@ def PyUnicode_ReadChar(space, ref, index):
     w_obj = from_ref(space, ref)
     w_ch = space.getitem(w_obj, space.newint(index))
     return space.int_w(space.ord(w_ch))
-        
+
 @cts.decl("int PyUnicode_WriteChar(PyObject *unicode, Py_ssize_t index, Py_UCS4 ch)", error=-1)
 def PyUnicode_WriteChar(space, ref, index, ch):
     """ Write a single ch at index before ref is ready. In order for this to
@@ -1489,11 +1583,29 @@ def PyUnicode_WriteChar(space, ref, index, ch):
     end = rutf8.next_codepoint_pos(as_str, index)
     ch_as_utf8 = rutf8.unichr_as_utf8(ch)
     if len(ch_as_utf8) != end - start:
-        raise oefmt(space.w_ValueError, 
+        raise oefmt(space.w_ValueError,
                     'cannot write ch to string, would need to reallocate')
     j = 0
     for i in range(start, end):
         utf8[i] = ch_as_utf8[j]
         j += 1
     return 0
- 
+
+@cpython_api([PyObjectP, PyObject], lltype.Void)
+def PyUnicode_Append(space, p_left, right):
+    state = space.fromcache(State)
+    operror = state.get_exception()
+    if not p_left:
+        if operror is not None:
+            PyErr_BadInternalCall(space)
+        return
+    left = p_left[0]
+    if not left or not right or not pyunicode_check(left) or not pyunicode_check(right):
+        if operror is not None:
+            PyErr_BadInternalCall(space)
+        p_left[0] = rffi.cast(PyObject, 0)
+        return
+    w_left = from_ref(space, left)
+    w_right = from_ref(space, right)
+    w_append = space.add(w_left, w_right)
+    p_left[0] = make_ref(space, w_append)
