@@ -2,9 +2,10 @@ from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.typedef import TypeDef, make_weakref_descr
 from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
+from pypy.objspace.std.util import generic_alias_class_getitem
 from rpython.rlib import jit
 
-from pypy.module.__builtin__.functional import W_Filter, W_Map
+from pypy.module.__builtin__.functional import W_Filter, build_iterators_from_args
 
 class W_Count(W_Root):
     def __init__(self, space, w_firstval, w_step):
@@ -42,8 +43,8 @@ class W_Count(W_Root):
             args_w = [self.w_c]
         else:
             args_w = [self.w_c, self.w_step]
-        return space.newtuple([space.gettypefor(W_Count),
-                               space.newtuple(args_w)])
+        return space.newtuple2(space.gettypefor(W_Count),
+                               space.newtuple(args_w))
 
 def check_number(space, w_obj):
     if (space.lookup(w_obj, '__int__') is None and
@@ -281,8 +282,8 @@ W_DropWhile.typedef = TypeDef(
 class W_FilterFalse(W_Filter):
     reverse = True
     def descr_reduce(self, space):
-        args_w = [space.w_None if self.no_predicate else self.w_predicate,
-                  self.iterable]
+        args_w = [space.w_None if self.w_predicate is None else self.w_predicate,
+                  self.w_iterable]
         return space.newtuple([space.type(self), space.newtuple(args_w)])
 
 def W_FilterFalse___new__(space, w_subtype, w_predicate, w_iterable):
@@ -548,14 +549,16 @@ def chain_from_iterable(space, w_cls, w_arg):
     return r
 
 W_Chain.typedef = TypeDef(
-        'itertools.chain',
-        __new__  = interp2app(W_Chain___new__),
-        __iter__ = interp2app(W_Chain.iter_w),
-        __next__ = interp2app(W_Chain.next_w),
-        __reduce__ = interp2app(W_Chain.descr_reduce),
-        __setstate__ = interp2app(W_Chain.descr_setstate),
-        from_iterable = interp2app(chain_from_iterable, as_classmethod=True),
-        __doc__  = """Make an iterator that returns elements from the first iterable
+    'itertools.chain',
+    __new__  = interp2app(W_Chain___new__),
+    __iter__ = interp2app(W_Chain.iter_w),
+    __next__ = interp2app(W_Chain.next_w),
+    __reduce__ = interp2app(W_Chain.descr_reduce),
+    __setstate__ = interp2app(W_Chain.descr_setstate),
+    from_iterable = interp2app(chain_from_iterable, as_classmethod=True),
+    __class_getitem__ = interp2app(
+        generic_alias_class_getitem, as_classmethod=True),
+    __doc__  = """Make an iterator that returns elements from the first iterable
     until it is exhausted, then proceeds to the next iterable, until
     all of the iterables are exhausted. Used for treating consecutive
     sequences as a single sequence.
@@ -566,12 +569,19 @@ W_Chain.typedef = TypeDef(
         for it in iterables:
             for element in it:
                 yield element
-    """)
+""")
 
 
-class W_ZipLongest(W_Map):
-    _error_name = "zip_longest"
-    _immutable_fields_ = ["w_fillvalue"]
+class W_ZipLongest(W_Root):
+    _immutable_fields_ = ["w_fillvalue", "iterators"]
+
+    def __init__(self, space, w_fun, args_w):
+        self.space = space
+        self.w_fun = w_fun
+        self.iterators_w = build_iterators_from_args(space, args_w, "zip_longest")
+
+    def iter_w(self):
+        return self
 
     def _fetch(self, index):
         w_iter = self.iterators_w[index]
@@ -592,10 +602,10 @@ class W_ZipLongest(W_Map):
     def next_w(self):
         # common case: 2 arguments
         if len(self.iterators_w) == 2:
-            objects = [self._fetch(0), self._fetch(1)]
+            return self.space.newtuple2(self._fetch(0), self._fetch(1))
         else:
             objects = self._get_objects()
-        return self.space.newtuple(objects)
+            return self.space.newtuple(objects)
 
     def _get_objects(self):
         # the loop is out of the way of the JIT
@@ -621,6 +631,14 @@ class W_ZipLongest(W_Map):
     def descr_setstate(self, space, w_state):
         self.w_fillvalue = w_state
 
+    def iterator_greenkey(self, space):
+        # XXX in theory we should tupleize the greenkeys of all the
+        # sub-iterators, but much more work
+        if len(self.iterators_w) > 0:
+            return space.iterator_greenkey(self.iterators_w[0])
+        return None
+
+
 def W_ZipLongest___new__(space, w_subtype, __args__):
     arguments_w, kwds_w = __args__.unpack()
     w_fillvalue = space.w_None
@@ -821,17 +839,16 @@ def tee(space, w_iterable, n=2):
     if n < 0:
         raise oefmt(space.w_ValueError, "n must be >= 0")
 
-    if space.findattr(w_iterable, space.newtext("__copy__")) is not None:
+    w_iterator = space.iter(w_iterable)
+    if space.findattr(w_iterator, space.newtext("__copy__")) is not None:
         # In this case, we don't instantiate any W_TeeIterable.
         # We just rely on doing repeated __copy__().  This case
         # includes the situation where w_iterable is already
         # a W_TeeIterable itself.
-        iterators_w = [w_iterable] * n
+        iterators_w = [w_iterator] * n
         for i in range(1, n):
-            w_iterable = space.call_method(w_iterable, "__copy__")
-            iterators_w[i] = w_iterable
+            iterators_w[i] = space.call_method(w_iterator, "__copy__")
     else:
-        w_iterator = space.iter(w_iterable)
         w_chained_list = W_TeeChainedListNode(space)
         iterators_w = [W_TeeIterable(space, w_iterator, w_chained_list)
                        for x in range(n)]
@@ -984,7 +1001,7 @@ class W_GroupBy(W_Root):
         self._skip_to_next_iteration_group()
         w_key = self.w_tgtkey = self.w_currkey
         w_grouper = W_GroupByIterator(self, w_key)
-        return self.space.newtuple([w_key, w_grouper])
+        return self.space.newtuple2(w_key, w_grouper)
 
     def _skip_to_next_iteration_group(self):
         space = self.space
