@@ -273,6 +273,20 @@ class AppTestMarshal:
         res = marshal.loads(b'I\xf7\xe6\xd5\xc4\xb3\xa2\x91\x80')
         assert res == -0x7f6e5d4c3b2a1909
 
+    def test_co_filename_bug(self):
+        import marshal
+        code = compile('pass', 'tmp-\udcff.py', "exec")
+        res = marshal.dumps(code) # must not crash
+        code2 = marshal.loads(res)
+        assert code.co_filename == code2.co_filename
+
+    def test_ascii_bug(self):
+        import marshal
+        with raises(ValueError):
+            # TYPE_ASCII but not actually ascii
+            marshal.loads(b'a\x01\0\0\0\xff')
+
+
 @pytest.mark.skipif('config.option.runappdirect or sys.maxint > 2 ** 32')
 class AppTestSmallLong(AppTestMarshal):
     spaceconfig = AppTestMarshal.spaceconfig.copy()
