@@ -22,6 +22,7 @@ corresponding Unix manual entries for more information on calls."""
         'uname_result': 'app_posix.uname_result',
         'urandom': 'app_posix.urandom',
         'terminal_size': 'app_posix.terminal_size',
+        'waitstatus_to_exitcode': 'app_posix.waitstatus_to_exitcode',
     }
     if os.name == 'nt':
         del appleveldefs['urandom'] # at interp on win32
@@ -226,7 +227,7 @@ corresponding Unix manual entries for more information on calls."""
             assert getattr(rposix, _name) is not None, "missing %r" % (_name,)
             interpleveldefs[_name] = 'space.wrap(%d)' % getattr(rposix, _name)
 
-    if sys.platform.startswith('linux'): #hasattr(rposix, 'sendfile'):
+    if sys.platform != 'win32':
         interpleveldefs['sendfile'] = 'interp_posix.sendfile'
 
     if hasattr(rposix, 'pread'):
