@@ -7,15 +7,16 @@ class TestAlloc(BaseTestPyPyC):
                           [2 ** n - 1 for n in range(26)])
 
     def test_newstr_constant_size(self):
-        for size in TestAlloc.SIZES:
+        for size in sorted(TestAlloc.SIZES):
             yield self.newstr_constant_size, size
 
     def newstr_constant_size(self, size):
+        print 'size =', size
         src = """if 1:
                     N = %(size)d
                     part_a = 'a' * N
                     part_b = 'b' * N
-                    for i in xrange(20):
+                    for i in range(20):
                         ao = '%%s%%s' %% (part_a, part_b)
                     def main():
                         return 42
