@@ -120,7 +120,7 @@ class TestString(BaseTestPyPyC):
             p25 = newstr(1)
             i23 = strgetitem(ConstPtr(ptr92), i19)
             strsetitem(p25, 0, i23)
-            i107 = call_i(ConstClass(string_to_int), p25, 16, 1, 1, descr=<Calli . riii EF=4>)
+            i107 = call_i(ConstClass(string_to_int), p25, 16, 1, 1, 0, descr=<Calli . riiii EF=4>)
             guard_no_exception(descr=...)
             i95 = int_add_ovf(i6, i107)
             guard_no_overflow(descr=...)
@@ -362,3 +362,46 @@ class TestString(BaseTestPyPyC):
             i56 = call_i(ConstClass(next_codepoint_pos_dont_look_inside), p47, 0, descr=...)
             i57 = int_sub(i52, i56)
         ''')
+
+    def test_decode_encode(self):
+        log = self.run(r"""
+        def main(n):
+            global s
+            u = b'ab\xc3\xa4\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa7\xe2\x80\x8d\xf0\x9f\x91\xa6'.decode("utf-8")
+            count = 0
+            for i in range(n):
+                b = (u + str(i)).encode("utf-8")
+                u = b.decode("utf-8") # ID: decode
+                count += 1
+            return count
+        """, [10000])
+        loop, = log.loops_by_filename(self.filepath)
+        # No call to _check_utf8 is necessary, because the bytes come from
+        # W_UnicodeObject.utf8_w.
+        assert loop.match_by_id('decode', '''
+        ''')
+
+    def test_find(self):
+        log = self.run("""
+        def main(n):
+            global s
+            b = b'abaaac' * 10000
+            count = 0
+            start = 0
+            while 1:
+                start = b.find(b'a', start)
+                if start < 0:
+                    break
+                b[start] # ID: index
+                count += 1
+                start += 1
+            return count
+        """, [10000])
+        loop, = log.loops_by_filename(self.filepath)
+        # no bounds check, the < 0 is done by the check for end of string.
+        # the >= len is not necessary, because find has a record_exact_value
+        # that records the fact that this is always true.
+        assert loop.match_by_id('index', '''
+            i2 = strgetitem(p1, i1)
+        ''')
+
