@@ -7,7 +7,7 @@ import platform
 
 class AppTestEpoll(object):
     spaceconfig = {
-        "usemodules": ["select", "_socket", "posix", "rctime"],
+        "usemodules": ["select", "_socket", "posix", "time"],
     }
 
     def setup_class(cls):
@@ -20,6 +20,10 @@ class AppTestEpoll(object):
         self.w_sockets = self.space.wrap([])
         if platform.machine().startswith('arm'):
             self.w_timeout = self.space.wrap(0.06)
+        if platform.machine().startswith('s390x'):
+            # s390x is not slow, but it seems there is one case when epoll
+            # modify method is called that takes longer on s390x
+            self.w_timeout = self.space.wrap(0.06)
         else:
             self.w_timeout = self.space.wrap(0.02)
 
@@ -28,7 +32,7 @@ class AppTestEpoll(object):
             self.space.call_method(socket, "close")
 
     def w_socket_pair(self):
-        import socket
+        import _socket as socket
 
         server_socket = socket.socket()
         server_socket.bind(('127.0.0.1', 0))
@@ -38,7 +42,9 @@ class AppTestEpoll(object):
         raises(socket.error,
             client.connect, ('127.0.0.1', server_socket.getsockname()[1])
         )
-        server, addr = server_socket.accept()
+        fd, addr = server_socket._accept()
+        server = socket.socket(server_socket.family, server_socket.type,
+                      server_socket.proto, fileno=fd)
 
         self.sockets.extend([server_socket, client, server])
         return client, server
@@ -53,6 +59,16 @@ class AppTestEpoll(object):
         assert ep.closed
         raises(ValueError, ep.fileno)
 
+    def test_with(self):
+        import select
+
+        ep = select.epoll(16)
+        assert ep.fileno() > 0
+        with ep:
+            assert not ep.closed
+        assert ep.closed
+        raises(ValueError, ep.__enter__)
+
     def test_badcreate(self):
         import select
 
@@ -147,8 +163,8 @@ class AppTestEpoll(object):
         then = time.time()
         assert not events
 
-        client.send("Hello!")
-        server.send("world!!!")
+        client.send(b"Hello!")
+        server.send(b"world!!!")
 
         now = time.time()
         events = ep.poll(1, 4)
@@ -184,6 +200,7 @@ class AppTestEpoll(object):
     def test_unregister_closed(self):
         import select
         import time
+        import errno
 
         client, server = self.socket_pair()
 
@@ -197,7 +214,9 @@ class AppTestEpoll(object):
         assert then - now < self.timeout
 
         server.close()
-        ep.unregister(fd)
+        with raises(OSError) as cm:
+            ep.unregister(fd)
+        assert cm.value.errno == errno.EBADF
 
     def test_close_twice(self):
         import select
@@ -205,3 +224,10 @@ class AppTestEpoll(object):
         ep = select.epoll()
         ep.close()
         ep.close()
+
+    def test_non_inheritable(self):
+        import select, posix
+
+        ep = select.epoll()
+        assert posix.get_inheritable(ep.fileno()) == False
+        ep.close()
