@@ -3,9 +3,9 @@ from pypy.interpreter.error import OperationError, oefmt, oefmt_attribute_error
 from pypy.interpreter.baseobjspace import ObjSpace
 from pypy.interpreter.function import Function, Method, FunctionWithFixedCode
 from pypy.interpreter.argument import Arguments
-from pypy.interpreter.typedef import default_identity_hash
+from pypy.interpreter.typedef import default_identity_hash, use_special_method_shortcut
 from rpython.tool.sourcetools import compile2, func_with_new_name
-from rpython.rlib.objectmodel import specialize
+from rpython.rlib.objectmodel import specialize, always_inline
 from rpython.rlib import jit
 
 @specialize.memo()
@@ -71,8 +71,14 @@ def get_attribute_name(space, w_obj, w_name):
             raiseattrerror(space, w_obj, w_name)
         raise
 
+@always_inline
 def _same_class_w(space, w_obj1, w_obj2, w_typ1, w_typ2):
-    return space.is_w(w_typ1, w_typ2)
+    from pypy.objspace.std.typeobject import W_TypeObject
+    # micro-optimization, call on the instances instead of using space.is_w so
+    # we can inline it
+    assert isinstance(w_typ1, W_TypeObject)
+    assert isinstance(w_typ2, W_TypeObject)
+    return w_typ1.is_w(space, w_typ2)
 
 
 class Object(object):
@@ -106,9 +112,15 @@ class Object(object):
         name = get_attribute_name(space, w_obj, w_name)
         w_descr = space.lookup(w_obj, name)
         if w_descr is not None:
-            if space.is_data_descr(w_descr):
-                space.set(w_descr, w_obj, w_value)
-                return
+            # shortcut for:
+            # if space.is_data_descr(w_descr):
+            #     return space.set(w_descr, w_obj, w_value)
+            w_set = space.lookup(w_descr, '__set__')
+            if w_set is not None:
+                return space.get_and_call_function(w_set, w_descr, w_obj, w_value)
+            if space.lookup(w_descr, '__delete__') is not None:
+                raise oefmt(space.w_AttributeError,
+                            "'%T' object is not a descriptor with set", w_descr)
         if w_obj.setdictvalue(space, name, w_value):
             return
         raiseattrerror(space, w_obj, w_name, w_descr)
@@ -240,6 +252,8 @@ class DescrOperation(object):
                         w_obj)
         return space.get_and_call_function(w_descr, w_obj, w_name)
 
+    # would be cool to shortcut this too, but old-style classes return int from
+    # __nonzero__
     def is_true(space, w_obj):
         w_descr = space.lookup(w_obj, "__bool__")
         if w_descr is None:
@@ -268,6 +282,7 @@ class DescrOperation(object):
         else:
             return space.w_False
 
+    @use_special_method_shortcut('__len__')
     def _len(space, w_obj):
         w_descr = space.lookup(w_obj, '__len__')
         if w_descr is None:
@@ -302,6 +317,8 @@ class DescrOperation(object):
                 return False
         return True
 
+    @use_special_method_shortcut('__iter__',
+            lambda space, w_iter: space.lookup(w_iter, '__next__') is not None)
     def iter(space, w_obj):
         w_descr = space.lookup(w_obj, '__iter__')
         if w_descr is None:
@@ -311,12 +328,16 @@ class DescrOperation(object):
                 raise oefmt(space.w_TypeError,
                             "'%T' object is not iterable", w_obj)
             return space.newseqiter(w_obj)
+        if space.is_w(w_descr, space.w_None):
+            raise oefmt(space.w_TypeError,
+                        "'%T' object is not iterable", w_obj)
         w_iter = space.get_and_call_function(w_descr, w_obj)
         w_next = space.lookup(w_iter, '__next__')
         if w_next is None:
             raise oefmt(space.w_TypeError, "iter() returned non-iterator")
         return w_iter
 
+    @use_special_method_shortcut('__next__')
     def next(space, w_obj):
         w_descr = space.lookup(w_obj, '__next__')
         if w_descr is None:
@@ -324,9 +345,14 @@ class DescrOperation(object):
                         "'%T' object is not an iterator", w_obj)
         return space.get_and_call_function(w_descr, w_obj)
 
+    @use_special_method_shortcut('__getitem__')
     def getitem(space, w_obj, w_key):
         w_descr = space.lookup(w_obj, '__getitem__')
         if w_descr is None and space.isinstance_w(w_obj, space.w_type):
+            # you've got to be kidding me :-( - cpython does the same
+            if space.is_w(w_obj, space.w_type):
+                from pypy.objspace.std.util import generic_alias_class_getitem
+                return generic_alias_class_getitem(space, w_obj, w_key)
             try:
                 w_descr = space.getattr(w_obj, space.newtext('__class_getitem__'))
             except OperationError as e:
@@ -340,6 +366,7 @@ class DescrOperation(object):
                         w_obj, w_key)
         return space.get_and_call_function(w_descr, w_obj, w_key)
 
+    @use_special_method_shortcut('__setitem__')
     def setitem(space, w_obj, w_key, w_val):
         w_descr = space.lookup(w_obj, '__setitem__')
         if w_descr is None:
@@ -347,6 +374,7 @@ class DescrOperation(object):
                         "'%T' object does not support item assignment", w_obj)
         return space.get_and_call_function(w_descr, w_obj, w_key, w_val)
 
+    @use_special_method_shortcut('__delitem__')
     def delitem(space, w_obj, w_key):
         w_descr = space.lookup(w_obj, '__delitem__')
         if w_descr is None:
@@ -408,6 +436,8 @@ class DescrOperation(object):
                 return w_res
         return space.pow(w_lhs, w_rhs, space.w_None)
 
+    @use_special_method_shortcut('__contains__',
+            lambda space, w_res: space.is_w(space.type(w_res), space.w_bool))
     def contains(space, w_container, w_item):
         w_descr = space.lookup(w_container, '__contains__')
         if w_descr is not None:
@@ -427,7 +457,7 @@ class DescrOperation(object):
                 if not e.match(space, space.w_StopIteration):
                     raise
                 return space.w_False
-            if space.eq_w(w_item, w_next):
+            if space.eq_w(w_next, w_item):
                 return space.w_True
 
     def sequence_count(space, w_container, w_item):
@@ -543,6 +573,10 @@ def _invoke_binop(space, w_impl, w_obj1, w_obj2):
 class PrintCache(object):
     def __init__(self, space):
         self.w_print = space.getattr(space.builtin, space.newtext("print"))
+
+# specialize on the special method names to use the lookup_in_type_where fast
+# paths
+@specialize.arg(3, 4)
 def _call_binop_impl(space, w_obj1, w_obj2, left, right, seq_bug_compat):
     w_typ1 = space.type(w_obj1)
     w_typ2 = space.type(w_obj2)
@@ -580,7 +614,7 @@ def _call_binop_impl(space, w_obj1, w_obj2, left, right, seq_bug_compat):
 
 def _make_binop_impl(symbol, specialnames):
     left, right = specialnames
-    errormsg = "unsupported operand type(s) for %s: '%%N' and '%%N'" % (
+    errormsg = "unsupported operand type(s) for %s: '%%T' and '%%T'" % (
         symbol.replace('%', '%%'),)
     seq_bug_compat = (symbol == '+' or symbol == '*')
 
@@ -590,15 +624,29 @@ def _make_binop_impl(symbol, specialnames):
     if symbol == "-":
         printerrormsg = errormsg + '. Did you mean "print(<-number>)"?'
 
+    @use_special_method_shortcut(left)
+    def shortcut_binop(space, w_obj1, w_obj2):
+        # makes a few assumptions: if the two rpython types are the same, then
+        # the left and the right implementations are the same too
+        w_impl = space.lookup(w_obj1, left)
+        if w_impl is not None:
+            w_res = space.get_and_call_function(w_impl, w_obj1, w_obj2)
+            if _check_notimplemented(space, w_res):
+                return w_res
+        raise oefmt(space.w_TypeError, errormsg, w_obj1, w_obj2)
+
     def binop_impl(space, w_obj1, w_obj2):
+        # shortcut: rpython classes are the same
+        if type(w_obj1) is type(w_obj2) and not w_obj1.user_overridden_class:
+            w_res = shortcut_binop(space, w_obj1, w_obj2)
+            if _check_notimplemented(space, w_res):
+                return w_res
         w_res = _call_binop_impl(space, w_obj1, w_obj2, left, right, seq_bug_compat)
         if w_res is not None:
             return w_res
-        w_typ1 = space.type(w_obj1)
-        w_typ2 = space.type(w_obj2)
         if printerrormsg is not None and w_obj1 is space.fromcache(PrintCache).w_print:
-            raise oefmt(space.w_TypeError, printerrormsg, w_typ1, w_typ2)
-        raise oefmt(space.w_TypeError, errormsg, w_typ1, w_typ2)
+            raise oefmt(space.w_TypeError, printerrormsg, w_obj1, w_obj2)
+        raise oefmt(space.w_TypeError, errormsg, w_obj1, w_obj2)
 
     return func_with_new_name(binop_impl, "binop_%s_impl"%left.strip('_'))
 
@@ -629,9 +677,26 @@ def _invoke_comparison(space, w_descr, w_obj1, w_obj2):
 def _make_comparison_impl(symbol, specialnames):
     left, right = specialnames
     op = getattr(operator, left)
+
+    @use_special_method_shortcut(left)
+    def shortcut_comparison(space, w_obj1, w_obj2):
+        # see shortcut_binop
+        w_impl = space.lookup(w_obj1, left)
+        if w_impl is not None:
+            w_res = space.get_and_call_function(w_impl, w_obj1, w_obj2)
+            if _check_notimplemented(space, w_res):
+                return w_res
+        return _no_special_method_found_cmp(space, w_obj1, w_obj2, w_obj1, w_obj2)
+
+
     def comparison_impl(space, w_obj1, w_obj2):
         w_orig_obj1 = w_obj1
         w_orig_obj2 = w_obj2
+        # shortcut, rpython classes are the same
+        if left == right and type(w_obj1) is type(w_obj2) and not w_obj1.user_overridden_class:
+            w_res = shortcut_comparison(space, w_obj1, w_obj2)
+            if _check_notimplemented(space, w_res):
+                return w_res
         w_typ1 = space.type(w_obj1)
         w_typ2 = space.type(w_obj2)
         w_left_src, w_left_impl = space.lookup_in_type_where(w_typ1, left)
@@ -654,7 +719,9 @@ def _make_comparison_impl(symbol, specialnames):
         w_res = _invoke_comparison(space, w_right_impl, w_obj2, w_obj1)
         if w_res is not None:
             return w_res
-        #
+        return _no_special_method_found_cmp(space, w_obj1, w_obj2, w_orig_obj1, w_orig_obj2)
+
+    def _no_special_method_found_cmp(space, w_obj1, w_obj2, w_orig_obj1, w_orig_obj2):
         # we did not find any special method, let's do the default logic for
         # == and !=
         if left == '__eq__':
@@ -684,14 +751,14 @@ def _make_inplace_impl(symbol, specialnames):
     seq_bug_compat = (symbol == '+=' or symbol == '*=')
     rhs_method = '__r' + specialname[3:]
     lhs_method = '__' + specialname[3:]
-    errormsg = "unsupported operand type(s) for %s: '%%N' and '%%N'" % (
+    errormsg = "unsupported operand type(s) for %s: '%%T' and '%%T'" % (
         symbol.replace('%', '%%'),)
 
     def inplace_impl(space, w_lhs, w_rhs):
         w_impl = space.lookup(w_lhs, specialname)
         if w_impl is not None:
             # 'seq_bug_compat' is for cpython bug-to-bug compatibility:
-            # see objspace/test/test_descrobject.*rmul_overrides.
+            # see objspace/test/apptest_descroperation.*rmul_overrides.
             # For cases like "list += object-overriding-__radd__".
             if (seq_bug_compat and space.type(w_lhs).flag_sequence_bug_compat
                            and not space.type(w_rhs).flag_sequence_bug_compat):
@@ -713,15 +780,14 @@ def _make_inplace_impl(symbol, specialnames):
         if w_res is not None:
             return w_res
 
-        w_typ1 = space.type(w_lhs)
-        w_typ2 = space.type(w_rhs)
-        raise oefmt(space.w_TypeError, errormsg, w_typ1, w_typ2)
+        raise oefmt(space.w_TypeError, errormsg, w_lhs, w_rhs)
 
     return func_with_new_name(inplace_impl, 'inplace_%s_impl'%specialname.strip('_'))
 
 def _make_unaryop_impl(symbol, specialnames):
     specialname, = specialnames
     errormsg = "unsupported operand type for unary %s: '%%T'" % symbol
+    @use_special_method_shortcut(specialname)
     def unaryop_impl(space, w_obj):
         w_impl = space.lookup(w_obj, specialname)
         if w_impl is None:
@@ -739,9 +805,11 @@ for targetname, specialname, checkerspec in [
     l = ["space.isinstance_w(w_result, %s)" % x
                 for x in checkerspec]
     checker = " or ".join(l)
-    msg = "unsupported operand type for %(targetname)s(): '%%T'"
+    msg = "must be real number, not %%T"
     msg = msg % locals()
     source = """if 1:
+        @use_special_method_shortcut(%(specialname)r,
+            lambda space, w_result: %(checker)s)
         def %(targetname)s(space, w_obj):
             w_impl = space.lookup(w_obj, %(specialname)r)
             if w_impl is None:
@@ -766,6 +834,9 @@ for targetname, specialname in [
     ('repr', '__repr__')]:
 
     source = """if 1:
+        @use_special_method_shortcut(%(specialname)r,
+            lambda space, w_res: space.isinstance_w(w_res, space.w_text)
+        )
         def %(targetname)s(space, w_obj):
             w_impl = space.lookup(w_obj, %(specialname)r)
             if w_impl is None:
