@@ -2,7 +2,7 @@ import math
 import operator
 import sys
 
-from rpython.rlib import rarithmetic, rfloat
+from rpython.rlib import rarithmetic, rfloat, jit
 from rpython.rlib.rarithmetic import LONG_BIT, intmask, ovfcheck_float_to_int
 from rpython.rlib.rarithmetic import int_between
 from rpython.rlib.rbigint import rbigint
@@ -46,6 +46,9 @@ def float2string(x, code, precision):
         s = "nan"
     return s
 
+def float_repr(x):
+    return float2string(x, 'r', 0)
+
 
 def detect_floatformat():
     from rpython.rtyper.lltypesystem import rffi, lltype
@@ -212,20 +215,6 @@ class W_FloatObject(W_Root):
     @staticmethod
     @unwrap_spec(w_x=WrappedDefault(0.0))
     def descr__new__(space, w_floattype, w_x, __posonly__):
-        def _string_to_float(space, w_source, string):
-            try:
-                if "_" in string:
-                    string = _remove_underscores(string)
-            except ValueError:
-                pass
-            else:
-                try:
-                    return rfloat.string_to_float(string)
-                except ParseStringError as e:
-                    pass
-            raise oefmt(space.w_ValueError,
-                        "could not convert string to float: %R", w_source)
-
         w_value = w_x     # 'x' is the keyword argument name in CPython
         if space.lookup(w_value, "__float__") is not None:
             w_obj = space.float(w_value)
@@ -433,11 +422,12 @@ class W_FloatObject(W_Root):
         return _round_float(space, self, w_ndigits)
 
     def descr_repr(self, space):
-        return space.newtext(float2string(self.floatval, 'r', 0))
+        res = float_repr(self.floatval)
+        return space.newutf8(res, len(res)) # always ascii
     descr_str = func_with_new_name(descr_repr, 'descr_str')
 
     def descr_hash(self, space):
-        h = _hash_float(space, self.floatval)
+        h = _hash_float(self.floatval)
         return space.newint(h)
 
     def descr_format(self, space, w_spec):
@@ -455,6 +445,12 @@ class W_FloatObject(W_Root):
     def descr_trunc(self, space):
         return newint_from_float(space, self.floatval)
 
+    def descr_floor(self, space):
+        return newint_from_float(space, math.floor(self.floatval))
+
+    def descr_ceil(self, space):
+        return newint_from_float(space, math.ceil(self.floatval))
+
     def descr_neg(self, space):
         return W_FloatObject(-self.floatval)
 
@@ -650,7 +646,7 @@ class W_FloatObject(W_Root):
         w_num = space.newlong_from_rbigint(num)
         w_den = space.newlong_from_rbigint(den)
         # Try to return int
-        return space.newtuple([space.int(w_num), space.int(w_den)])
+        return space.newtuple2(space.int(w_num), space.int(w_den))
 
     def descr_hex(self, space):
         """float.hex() -> string
@@ -714,6 +710,8 @@ Convert a string or number to a floating point number, if possible.''',
     __int__ = interp2app(W_FloatObject.descr_trunc),
     __float__ = interp2app(W_FloatObject.descr_float),
     __trunc__ = interp2app(W_FloatObject.descr_trunc),
+    __floor__ = interp2app(W_FloatObject.descr_floor),
+    __ceil__ = interp2app(W_FloatObject.descr_ceil),
     __neg__ = interp2app(W_FloatObject.descr_neg),
     __pos__ = interp2app(W_FloatObject.descr_pos),
     __abs__ = interp2app(W_FloatObject.descr_abs),
@@ -771,8 +769,22 @@ def _remove_underscores(string):
         raise ValueError
     return "".join(res)
 
+def _string_to_float(space, w_source, string):
+    try:
+        if "_" in string:
+            string = _remove_underscores(string)
+    except ValueError:
+        pass
+    else:
+        try:
+            return rfloat.string_to_float(string)
+        except ParseStringError as e:
+            pass
+    raise oefmt(space.w_ValueError,
+                "could not convert string to float: %R", w_source)
 
-def _hash_float(space, v):
+@jit.elidable
+def _hash_float(v):
     if not isfinite(v):
         if math.isinf(v):
             return HASH_INF if v > 0 else -HASH_INF
