@@ -18,6 +18,7 @@ from rpython.rlib.objectmodel import (
 from rpython.rlib.rarithmetic import ovfcheck
 from rpython.rlib import longlong2float
 from rpython.tool.sourcetools import func_with_new_name
+from rpython.rlib.rstring import StringBuilder
 
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError, oefmt
@@ -35,7 +36,7 @@ from pypy.objspace.std.iterobject import (
 from pypy.objspace.std.sliceobject import W_SliceObject, unwrap_start_stop
 from pypy.objspace.std.tupleobject import W_AbstractTupleObject
 from pypy.objspace.std.unicodeobject import W_UnicodeObject
-from pypy.objspace.std.util import get_positive_index, negate
+from pypy.objspace.std.util import get_positive_index, negate, generic_alias_class_getitem
 
 __all__ = ['W_ListObject', 'make_range_list', 'make_empty_list_with_size']
 
@@ -160,12 +161,51 @@ def _do_extend_from_iterable(space, w_list, w_iterable):
         i += 1
     return i
 
+def _get_printable_location(strategy_type, typ):
+    return 'list.repr [%s, %s]' % (
+        strategy_type,
+        typ)
+
+listrepr_jitdriver = jit.JitDriver(
+    name='list.repr',
+    greens=['strategy_type', 'typ'],
+    reds='auto',
+    get_printable_location=_get_printable_location)
+
+def listrepr(space, w_currently_in_repr, w_list):
+    length = w_list.length()
+    if space.contains_w(w_currently_in_repr, w_list):
+        return space.newtext('[...]')
+    space.setitem(w_currently_in_repr, w_list, space.newint(1))
+    try:
+        return _listrepr_inner(space, w_list, length)
+    finally:
+        try:
+            space.delitem(w_currently_in_repr, w_list)
+        except OperationError as e:
+            if not e.match(space, space.w_KeyError):
+                raise
+
+def _listrepr_inner(space, w_list, length):
+    assert length > 0
+    builder = rutf8.Utf8StringBuilder()
+    builder.append_char('[')
+    w_first = w_list.getitem(0)
+    builder.append_utf8(*space.utf8_len_w(space.repr(w_first)))
+    typ = type(w_first)
+    for i in range(1, length):
+        listrepr_jitdriver.jit_merge_point(
+            typ=typ,
+            strategy_type=type(w_list.strategy))
+        builder.append_utf8(', ', 2)
+        w_item = w_list.getitem(i)
+        builder.append_utf8(*space.utf8_len_w(space.repr(w_item)))
+    builder.append_char(']')
+    return space.newutf8(builder.build(), builder.getlength())
+
 
 def list_unroll_condition(w_list1, space, w_list2):
-    return (jit.loop_unrolling_heuristic(w_list1, w_list1.length(),
-                                         UNROLL_CUTOFF) or
-            jit.loop_unrolling_heuristic(w_list2, w_list2.length(),
-                                         UNROLL_CUTOFF))
+    return (w_list1._unrolling_heuristic() or w_list2._unrolling_heuristic())
 
 
 class W_ListObject(W_Root):
@@ -181,6 +221,10 @@ class W_ListObject(W_Root):
             self.strategy = space.fromcache(ObjectListStrategy)
         self.init_from_list_w(wrappeditems)
 
+    def _unrolling_heuristic(self):
+        strategy = self.strategy
+        return strategy._unrolling_heuristic(self)
+
     @staticmethod
     def from_storage_and_strategy(space, storage, strategy):
         self = instantiate(W_ListObject)
@@ -286,12 +330,13 @@ class W_ListObject(W_Root):
         strategy and storage according to the other W_List"""
         self.strategy.copy_into(self, other)
 
-    def find(self, w_item, start=0, end=sys.maxint):
-        """Find w_item in list[start:end]. If not found, raise ValueError"""
-        return self.strategy.find(self, w_item, start, end)
+    def find_or_count(self, w_item, start=0, end=sys.maxint, count=False):
+        """Find w_item in list[start:end]. If not found, raise ValueError.
+        if count=True, count number of occurences instead"""
+        return self.strategy.find_or_count(self, w_item, start, end, count)
 
     def append(self, w_item):
-        """L.append(object) -> None -- append object to end"""
+        """Append object to the end of the list."""
         self.strategy.append(self, w_item)
 
     def length(self):
@@ -430,6 +475,7 @@ class W_ListObject(W_Root):
             self.extend(w_iterable)
 
     def descr_repr(self, space):
+        return self.strategy.repr(self)
         if self.length() == 0:
             return space.newtext('[]')
         return listrepr(space, space.get_objects_in_repr(), self)
@@ -497,7 +543,7 @@ class W_ListObject(W_Root):
 
     def descr_contains(self, space, w_obj):
         try:
-            self.find(w_obj)
+            self.find_or_count(w_obj)
             return space.w_True
         except ValueError:
             return space.w_False
@@ -615,14 +661,8 @@ class W_ListObject(W_Root):
 
     def descr_count(self, space, w_value):
         '''L.count(value) -> integer -- return number of occurrences of value'''
-        # needs to be safe against eq_w() mutating the w_list behind our back
-        count = 0
-        i = 0
-        while i < self.length():
-            if space.eq_w(self.getitem(i), w_value):
-                count += 1
-            i += 1
-        return space.newint(count)
+        i = self.find_or_count(w_value, count=True)
+        return space.newint(i)
 
     @unwrap_spec(index=int)
     def descr_insert(self, space, index, w_value):
@@ -661,7 +701,7 @@ Raises IndexError if list is empty or index is out of range."""
 Raises ValueError if the value is not present."""
         # needs to be safe against eq_w() mutating the w_list behind our back
         try:
-            i = self.find(w_value, 0, sys.maxint)
+            i = self.find_or_count(w_value, 0, sys.maxint)
         except ValueError:
             raise oefmt(space.w_ValueError,
                         "list.remove(): %R is not in list", w_value)
@@ -677,7 +717,7 @@ Raises ValueError if the value is not present."""
         i, stop = unwrap_start_stop(space, size, w_start, w_stop)
         # note that 'i' and 'stop' can be bigger than the length of the list
         try:
-            i = self.find(w_value, i, stop)
+            i = self.find_or_count(w_value, i, stop)
         except ValueError:
             raise oefmt(space.w_ValueError, "%R is not in list", w_value)
         return space.newint(i)
@@ -707,6 +747,7 @@ Raises ValueError if the value is not present."""
         sorter.space = space
 
         try:
+            strategy = self.strategy
             # The list is temporarily made empty, so that mutations performed
             # by comparison functions can't affect the slice of memory we're
             # sorting (allowing mutations during sorting is an IndexError or
@@ -715,10 +756,11 @@ Raises ValueError if the value is not present."""
 
             # wrap each item in a KeyContainer if needed
             if has_key:
-                for i in range(sorter.listlength):
-                    w_item = sorter.list[i]
-                    w_keyitem = space.call_function(w_key, w_item)
-                    sorter.list[i] = KeyContainer(w_keyitem, w_item)
+                # XXX inefficient for unwrapped strategies:
+                # we wrap the elements twice, once for the key, and once to get
+                # KeyContainers. Then unwrap carefully in the __init__ call below.
+                # Could type-specialize this.
+                _compute_keys_for_sorting(strategy, sorter.list, w_key)
 
             # Reverse sort stability achieved by initially reversing the list,
             # applying a stable forward sort, then reversing the final result.
@@ -749,9 +791,42 @@ Raises ValueError if the value is not present."""
         if mucked:
             raise oefmt(space.w_ValueError, "list modified during sort")
 
-def get_printable_location(strategy_type, tp):
-    return "list.find [%s, %s]" % (strategy_type, tp.getname(tp.space), )
-find_jmp = jit.JitDriver(greens=['strategy_type', 'tp'], reds='auto', name='list.find', get_printable_location=get_printable_location)
+def get_printable_location_sortkey(strategy_type, tp):
+    return "_compute_keys_for_sorting [%s, %s]" % (strategy_type, tp.getname(tp.space), )
+
+sortkey_jmp = jit.JitDriver(
+    greens=['strategy_type', 'tp'],
+    reds='auto',
+    name='_compute_keys_for_sorting',
+    get_printable_location=get_printable_location_sortkey)
+
+def _compute_keys_for_sorting(strategy, list_w, w_callable):
+    space = strategy.space
+    i = 0
+    # XXX would like a new API space.greenkey_for_callable here
+    # (also in min/max and map/filter)
+    tp = space.type(w_callable)
+    while i < len(list_w):
+        # bit weird: we have a list_w at this point, but we still specialize on
+        # the strategy to distinguish the cases better
+        sortkey_jmp.jit_merge_point(tp=tp, strategy_type=type(strategy))
+        w_item = list_w[i]
+        w_keyitem = space.call_function(w_callable, w_item)
+        list_w[i] = KeyContainer(w_keyitem, w_item)
+        i += 1
+
+def get_printable_location_find(count, strategy_type, tp):
+    if count:
+        start = "list.count"
+    else:
+        start = "list.find"
+    return "%s [%s, %s]" % (start, strategy_type, tp.getname(tp.space), )
+
+find_or_count_jmp = jit.JitDriver(
+    greens=['count', 'strategy_type', 'tp'],
+    reds='auto',
+    name='list.find_or_count',
+    get_printable_location=get_printable_location_find)
 
 class ListStrategy(object):
 
@@ -773,16 +848,23 @@ class ListStrategy(object):
     def _resize_hint(self, w_list, hint):
         raise NotImplementedError
 
-    def find(self, w_list, w_item, start, stop):
+    def find_or_count(self, w_list, w_item, start, stop, count):
         space = self.space
         i = start
+        result = 0
         # needs to be safe against eq_w mutating stuff
         tp = space.type(w_item)
         while i < stop and i < w_list.length():
-            find_jmp.jit_merge_point(tp=tp, strategy_type=type(self))
-            if space.eq_w(w_item, w_list.getitem(i)):
-                return i
+            find_or_count_jmp.jit_merge_point(tp=tp, strategy_type=type(self), count=count)
+            if space.eq_w(w_list.getitem(i), w_item):
+                if count:
+                    result += 1
+                else:
+                    return i
             i += 1
+        if count:
+            return result
+        # not find case, not found
         raise ValueError
 
     def length(self, w_list):
@@ -904,6 +986,18 @@ class ListStrategy(object):
     def physical_size(self, w_list):
         raise oefmt(self.space.w_ValueError, "can't get physical size of list")
 
+    def repr(self, w_list):
+        space = self.space
+        if self.length(w_list) == 0:
+            return space.newtext('[]')
+        return listrepr(space, space.get_objects_in_repr(), w_list)
+
+    def _unrolling_heuristic(self, w_list):
+        # default implementation: we will only go by size, not whether the list
+        # is virtual
+        size = self.length(w_list)
+        return size == 0 or (jit.isconstant(size) and size <= UNROLL_CUTOFF)
+
 
 class EmptyListStrategy(ListStrategy):
     """EmptyListStrategy is used when a W_List withouth elements is created.
@@ -939,7 +1033,9 @@ class EmptyListStrategy(ListStrategy):
         if hint:
             w_list.strategy = SizeListStrategy(self.space, hint)
 
-    def find(self, w_list, w_item, start, stop):
+    def find_or_count(self, w_list, w_item, start, stop, count):
+        if count:
+            return 0
         raise ValueError
 
     def length(self, w_list):
@@ -1067,6 +1163,13 @@ class EmptyListStrategy(ListStrategy):
     def physical_size(self, w_list):
         return 0
 
+    def repr(self, w_list):
+        space = self.space
+        return space.newtext('[]')
+
+    def _unrolling_heuristic(self, w_list):
+        return True
+
 
 class SizeListStrategy(EmptyListStrategy):
     """Like empty, but when modified it'll preallocate the size to sizehint."""
@@ -1176,6 +1279,8 @@ class BaseRangeListStrategy(ListStrategy):
             self.switch_to_integer_strategy(w_list)
             w_list.sort(reverse)
 
+    # default _unrolling_heuristic is fine
+
 
 class SimpleRangeListStrategy(BaseRangeListStrategy):
     """SimpleRangeListStrategy is used when a list is created using the range
@@ -1186,15 +1291,20 @@ class SimpleRangeListStrategy(BaseRangeListStrategy):
     erase = staticmethod(erase)
     unerase = staticmethod(unerase)
 
-    def find(self, w_list, w_obj, startindex, stopindex):
+    def find_or_count(self, w_list, w_obj, startindex, stopindex, count):
         if is_plain_int1(w_obj):
             obj = self.unwrap(w_obj)
             length = self.unerase(w_list.lstorage)[0]
             if 0 <= obj < length and startindex <= obj < stopindex:
+                if count:
+                    return 1
                 return obj
             else:
+                if count:
+                    return 0
                 raise ValueError
-        return ListStrategy.find(self, w_list, w_obj, startindex, stopindex)
+        return ListStrategy.find_or_count(
+            self, w_list, w_obj, startindex, stopindex, count)
 
     def length(self, w_list):
         return self.unerase(w_list.lstorage)[0]
@@ -1258,7 +1368,7 @@ class RangeListStrategy(BaseRangeListStrategy):
     erase = staticmethod(erase)
     unerase = staticmethod(unerase)
 
-    def find(self, w_list, w_obj, startindex, stopindex):
+    def find_or_count(self, w_list, w_obj, startindex, stopindex, count):
         if is_plain_int1(w_obj):
             obj = self.unwrap(w_obj)
             start, step, length = self.unerase(w_list.lstorage)
@@ -1268,11 +1378,18 @@ class RangeListStrategy(BaseRangeListStrategy):
                  (start - obj) % step == 0)):
                 index = (obj - start) // step
             else:
+                if count:
+                    return 0
                 raise ValueError
             if startindex <= index < stopindex:
+                if count:
+                    return 1
                 return index
+            if count:
+                return 0
             raise ValueError
-        return ListStrategy.find(self, w_list, w_obj, startindex, stopindex)
+        return ListStrategy.find_or_count(
+            self, w_list, w_obj, startindex, stopindex, count)
 
     def length(self, w_list):
         return self.unerase(w_list.lstorage)[2]
@@ -1394,17 +1511,25 @@ class AbstractUnwrappedStrategy(object):
         items = self.unerase(w_list.lstorage)[:]
         w_other.lstorage = self.erase(items)
 
-    def find(self, w_list, w_obj, start, stop):
+    def find_or_count(self, w_list, w_obj, start, stop, count):
         if self.is_correct_type(w_obj):
-            return self._safe_find(w_list, self.unwrap(w_obj), start, stop)
-        return ListStrategy.find(self, w_list, w_obj, start, stop)
+            return self._safe_find_or_count(
+                w_list, self.unwrap(w_obj), start, stop, count)
+        return ListStrategy.find_or_count(
+            self, w_list, w_obj, start, stop, count)
 
-    def _safe_find(self, w_list, obj, start, stop):
+    def _safe_find_or_count(self, w_list, obj, start, stop, count):
         l = self.unerase(w_list.lstorage)
+        result = 0
         for i in range(start, min(stop, len(l))):
             val = l[i]
             if val == obj:
-                return i
+                if count:
+                    result += 1
+                else:
+                    return i
+        if count:
+            return result
         raise ValueError
 
     def length(self, w_list):
@@ -1438,13 +1563,11 @@ class AbstractUnwrappedStrategy(object):
             func_with_new_name(getitems_copy, "getitems_unroll"))
 
     getitems_copy = jit.look_inside_iff(lambda self, w_list:
-            jit.loop_unrolling_heuristic(w_list, w_list.length(),
-                                         UNROLL_CUTOFF))(getitems_copy)
+            w_list._unrolling_heuristic())(getitems_copy)
 
 
     @jit.look_inside_iff(lambda self, w_list:
-            jit.loop_unrolling_heuristic(w_list, w_list.length(),
-                                         UNROLL_CUTOFF))
+            w_list._unrolling_heuristic())
     def getitems_fixedsize(self, w_list):
         return self.getitems_unroll(w_list)
 
@@ -1657,6 +1780,10 @@ class AbstractUnwrappedStrategy(object):
         l = self.unerase(w_list.lstorage)
         return list_get_physical_size(l)
 
+    def _unrolling_heuristic(self, w_list):
+        storage = self.unerase(w_list.lstorage)
+        return jit.loop_unrolling_heuristic(storage, len(storage), UNROLL_CUTOFF)
+
 
 class ObjectListStrategy(ListStrategy):
     import_from_mixin(AbstractUnwrappedStrategy)
@@ -1677,8 +1804,7 @@ class ObjectListStrategy(ListStrategy):
     unerase = staticmethod(unerase)
 
     @jit.look_inside_iff(lambda self, w_list:
-            jit.loop_unrolling_heuristic(w_list, w_list.length(),
-                                         UNROLL_CUTOFF))
+            w_list._unrolling_heuristic())
     def getitems_copy(self, w_list):
         storage = self.unerase(w_list.lstorage)
         return storage[:]
@@ -1700,8 +1826,9 @@ class ObjectListStrategy(ListStrategy):
     def clear(self, w_list):
         w_list.lstorage = self.erase([])
 
-    def find(self, w_list, w_obj, start, stop):
-        return ListStrategy.find(self, w_list, w_obj, start, stop)
+    def find_or_count(self, w_list, w_obj, start, stop, count):
+        return ListStrategy.find_or_count(
+                self, w_list, w_obj, start, stop, count)
 
     def getitems(self, w_list):
         return self.unerase(w_list.lstorage)
@@ -1808,6 +1935,11 @@ class IntegerListStrategy(ListStrategy):
         # no, fall back to ObjectListStrategy
         w_list.switch_to_object_strategy()
 
+    def repr(self, w_list):
+        space = self.space
+        res = str(self.unerase(w_list.lstorage))
+        return space.newtext(res)
+
 
 class FloatListStrategy(ListStrategy):
     import_from_mixin(AbstractUnwrappedStrategy)
@@ -1869,20 +2001,29 @@ class FloatListStrategy(ListStrategy):
         return self._base_setslice(w_list, start, step, slicelength, w_other)
 
 
-    def _safe_find(self, w_list, obj, start, stop):
+    def _safe_find_or_count(self, w_list, obj, start, stop, count):
         l = self.unerase(w_list.lstorage)
         stop = min(stop, len(l))
+        result = 0
         if not math.isnan(obj):
             for i in range(start, stop):
                 val = l[i]
                 if val == obj:
-                    return i
+                    if count:
+                        result += 1
+                    else:
+                        return i
         else:
             search = longlong2float.float2longlong(obj)
             for i in range(start, stop):
                 val = l[i]
                 if longlong2float.float2longlong(val) == search:
-                    return i
+                    if count:
+                        result += 1
+                    else:
+                        return i
+        if count:
+            return result
         raise ValueError
 
     @staticmethod
@@ -1918,6 +2059,21 @@ class FloatListStrategy(ListStrategy):
         # no, fall back to ObjectListStrategy
         w_list.switch_to_object_strategy()
 
+    def repr(self, w_list):
+        from pypy.objspace.std.floatobject import float_repr
+        l = self.unerase(w_list.lstorage)
+        if len(l) == 0:
+            return self.space.newtext('[]')
+        b = StringBuilder()
+        b.append('[')
+        for i in range(len(l)):
+            if i > 0:
+                b.append(', ')
+            b.append(float_repr(l[i]))
+        b.append(']')
+        res = b.build()
+        return self.space.newutf8(res, len(res))
+
 
 class IntOrFloatListStrategy(ListStrategy):
     import_from_mixin(AbstractUnwrappedStrategy)
@@ -2017,20 +2173,51 @@ class IntOrFloatListStrategy(ListStrategy):
                 w_other = self._temporary_longlong_list(longlong_list)
         return self._base_setslice(w_list, start, step, slicelength, w_other)
 
-    def _safe_find(self, w_list, obj, start, stop):
+    def _safe_find_or_count(self, w_list, obj, start, stop, count):
         l = self.unerase(w_list.lstorage)
         # careful: we must consider that 0.0 == -0.0 == 0, but also
         # NaN == NaN if they have the same bit pattern.
         fobj = longlong2float.maybe_decode_longlong_as_float(obj)
+        result = 0
         for i in range(start, min(stop, len(l))):
             llval = l[i]
             if llval == obj:     # equal as longlongs: includes NaN == NaN
-                return i
+                if count:
+                    result += 1
+                    continue
+                else:
+                    return i
             fval = longlong2float.maybe_decode_longlong_as_float(llval)
             if fval == fobj:     # cases like 0.0 == -0.0 or 42 == 42.0
-                return i
+                if count:
+                    result += 1
+                else:
+                    return i
+        if count:
+            return result
         raise ValueError
 
+    def repr(self, w_list):
+        from pypy.objspace.std.floatobject import float_repr
+        l = self.unerase(w_list.lstorage)
+        if len(l) == 0:
+            return self.space.newtext('[]')
+        b = StringBuilder()
+        b.append('[')
+        for i in range(len(l)):
+            if i > 0:
+                b.append(', ')
+            llval = l[i]
+            if longlong2float.is_int32_from_longlong_nan(llval):
+                intval = longlong2float.decode_int32_from_longlong_nan(llval)
+                b.append(str(intval))
+            else:
+                floatval = longlong2float.longlong2float(llval)
+                b.append(float_repr(floatval))
+        b.append(']')
+        res = b.build()
+        return self.space.newtext(res, len(res))
+
 
 class BytesListStrategy(ListStrategy):
     import_from_mixin(AbstractUnwrappedStrategy)
@@ -2118,22 +2305,6 @@ def plain_int_w(space, w_obj):
 init_signature = Signature(['sequence'], posonlyargcount=1)
 init_defaults = [None]
 
-app = applevel("""
-    def listrepr(currently_in_repr, l):
-        'The app-level part of repr().'
-        if l in currently_in_repr:
-            return '[...]'
-        currently_in_repr[l] = 1
-        try:
-            return "[" + ", ".join([repr(x) for x in l]) + ']'
-        finally:
-            try:
-                del currently_in_repr[l]
-            except:
-                pass
-""", filename=__file__)
-
-listrepr = app.interphook("listrepr")
 
 # ____________________________________________________________
 # Sorting
@@ -2180,21 +2351,6 @@ class IntOrFloatSort(IntOrFloatBaseTimSort):
         return fa < fb
 
 
-class CustomCompareSort(SimpleSort):
-    def lt(self, a, b):
-        space = self.space
-        w_cmp = self.w_cmp
-        w_result = space.call_function(w_cmp, a, b)
-        try:
-            result = space.int_w(w_result)
-        except OperationError as e:
-            if e.match(space, space.w_TypeError):
-                raise oefmt(space.w_TypeError,
-                            "comparison function must return int")
-            raise
-        return result < 0
-
-
 class CustomKeySort(SimpleSort):
     def lt(self, a, b):
         assert isinstance(a, KeyContainer)
@@ -2204,8 +2360,11 @@ class CustomKeySort(SimpleSort):
 
 
 W_ListObject.typedef = TypeDef("list",
-    __doc__ = """list() -> new empty list
-list(iterable) -> new list initialized from iterable's items""",
+    __doc__ = """Built-in mutable sequence.
+
+If no argument is given, the constructor creates a new empty list.
+The argument must be an iterable if specified.""",
+    _text_signature_ = "(iterable=(), /)",
     __new__ = interp2app(W_ListObject.descr_new),
     __init__ = interp2app(W_ListObject.descr_init),
     __repr__ = interp2app(W_ListObject.descr_repr),
@@ -2232,6 +2391,9 @@ list(iterable) -> new list initialized from iterable's items""",
     __setitem__ = interp2app(W_ListObject.descr_setitem),
     __delitem__ = interp2app(W_ListObject.descr_delitem),
 
+    __class_getitem__ = interp2app(
+        generic_alias_class_getitem, as_classmethod=True),
+
     sort = interp2app(W_ListObject.descr_sort),
     index = interp2app(W_ListObject.descr_index),
     copy = interp2app(W_ListObject.descr_copy),
