@@ -2,6 +2,7 @@
 
 import functools
 
+from rpython.rlib import jit
 from rpython.rlib.objectmodel import specialize
 from rpython.rlib.rarithmetic import intmask
 from rpython.rlib.rbigint import SHIFT, _load_unsigned_digit, rbigint
@@ -51,6 +52,13 @@ class W_AbstractLongObject(W_AbstractIntObject):
         except OverflowError:
             raise oefmt(space.w_OverflowError, "too many digits in integer")
 
+    def descr_bit_count(self, space):
+        bigint = space.bigint_w(self)
+        try:
+            return space.newint(bigint.bit_count())
+        except OverflowError:
+            raise oefmt(space.w_OverflowError, "too many digits in integer")
+
     def _truediv(self, space, w_other):
         try:
             f = self.asbigint().truediv(w_other.asbigint())
@@ -75,10 +83,32 @@ class W_AbstractLongObject(W_AbstractIntObject):
                                        newformat.LONG_KIND)
 
     def descr_hash(self, space):
-        return space.newint(_hash_long(space, self.asbigint()))
+        return space.newint(_hash_long(self.asbigint()))
 
     def descr_str(self, space):
-        res = self.asbigint().str()
+        from pypy.module.sys.system import MAX_STR_DIGITS_THRESHOLD
+        from pypy.module.sys.state import get_int_max_str_digits
+        from rpython.rlib.rbigint import SHIFT, MaxIntError
+        msg_fmt_to_str = ("Exceeds the limit (%d) for integer string "
+              "conversion; use sys.set_int_max_str_digits() to increase the "
+              "limit")
+
+        bigint = space.bigint_w(self)
+        numdigits = bigint.numdigits()
+        max_str_digits = space.int_w(get_int_max_str_digits(space))
+        # quick and dirty pre-check for overflowing the decimal digit limit,
+        if bigint.numdigits() >= (10 * MAX_STR_DIGITS_THRESHOLD / 
+                                  (3 * SHIFT + 2)):
+            if (max_str_digits != 0 and
+                    max_str_digits / (3 * SHIFT) <= (numdigits - 11) / 10):
+                raise oefmt(space.w_ValueError, msg_fmt_to_str, max_str_digits)
+        # Do an additional more accurate check that
+        # strlen(res) < max_str_digits (actually they check before allocating
+        # the buffer to hold the string.
+        try:
+            res = self.asbigint().str(max_str_digits=max_str_digits)
+        except MaxIntError:
+            raise oefmt(space.w_ValueError, msg_fmt_to_str, max_str_digits)
         return space.newutf8(res, len(res))
     descr_repr = descr_str
 
@@ -389,7 +419,7 @@ class W_LongObject(W_AbstractLongObject):
         except ZeroDivisionError:
             raise oefmt(space.w_ZeroDivisionError,
                         "integer division or modulo by zero")
-        return space.newtuple([newlong(space, div), newlong(space, mod)])
+        return space.newtuple2(newlong(space, div), newlong(space, mod))
 
     def _int_divmod(self, space, other):
         try:
@@ -397,7 +427,7 @@ class W_LongObject(W_AbstractLongObject):
         except ZeroDivisionError:
             raise oefmt(space.w_ZeroDivisionError,
                         "long division or modulo by zero")
-        return space.newtuple([newlong(space, div), newlong(space, mod)])
+        return space.newtuple2(newlong(space, div), newlong(space, mod))
 
     descr_divmod, descr_rdivmod = _make_descr_binop(_divmod, _int_divmod)
 
@@ -408,7 +438,8 @@ class W_LongObject(W_AbstractLongObject):
 # us apply extra optimizations to the hash function.
 _HASH_SHIFT = SHIFT % HASH_BITS
 
-def _hash_long(space, v):
+@jit.elidable
+def _hash_long(v):
     i = v.numdigits() - 1
     if i == -1:
         return 0
