@@ -53,11 +53,11 @@ FLAG_DONE      = '\x00'
 TYPE_INT64     = 'I'     # no longer generated
 
 # the following typecodes have been added in version 4.
-TYPE_ASCII                = 'a'   # never generated so far by pypy
-TYPE_ASCII_INTERNED       = 'A'   # never generated so far by pypy
+TYPE_ASCII                = 'a'
+TYPE_ASCII_INTERNED       = 'A'
 TYPE_SMALL_TUPLE          = ')'
-TYPE_SHORT_ASCII          = 'z'   # never generated so far by pypy
-TYPE_SHORT_ASCII_INTERNED = 'Z'   # never generated so far by pypy
+TYPE_SHORT_ASCII          = 'z'
+TYPE_SHORT_ASCII_INTERNED = 'Z'
 
 
 _marshallers = []
@@ -398,7 +398,7 @@ def marshal_pycode(space, w_pycode, m):
     _marshal_tuple(space, co_varnames_w, m)  # more lists, now of w_unicodes
     _marshal_tuple(space, co_freevars_w, m)
     _marshal_tuple(space, co_cellvars_w, m)
-    _marshal_unicode(space, x.co_filename, m)
+    marshal(space, x.w_filename, m)
     _marshal_unicode(space, x.co_name, m)
     m.put_int(x.co_firstlineno)
     m.atom_str(TYPE_STRING, x.co_lnotab)
@@ -435,7 +435,9 @@ def unmarshal_pycode(space, u, tc):
     varnames    = _unmarshal_strlist(u)
     freevars    = _unmarshal_strlist(u)
     cellvars    = _unmarshal_strlist(u)
-    filename    = space.utf8_0_w(u.load_w_obj())
+    w_fn = u.load_w_obj()
+    filename    = space.bytes0_w(space.fsencode(w_fn))
+
     name        = space.utf8_w(u.load_w_obj())
     firstlineno = u.get_int()
     lnotab      = space.bytes_w(u.load_w_obj())
@@ -447,8 +449,35 @@ def unmarshal_pycode(space, u, tc):
                   hidden_applevel=u.hidden_applevel)
     return w_codeobj
 
+def _marshal_ascii_unicode(space, s, m, w_unicode, w_interned):
+    # determine typecode
+    if len(s) < 256:
+        is_short = True
+        if w_interned is not None:
+            typecode = TYPE_SHORT_ASCII_INTERNED
+        else:
+            typecode = TYPE_SHORT_ASCII
+    else:
+        is_short = False
+        if w_interned is not None:
+            typecode = TYPE_ASCII_INTERNED
+        else:
+            typecode = TYPE_ASCII
+    # use ref
+    if w_unicode is not None:
+        typecode = write_ref(typecode, w_unicode, m)
+        if typecode == FLAG_DONE:
+            return
+    # write
+    m.start(typecode)
+    if is_short:
+        m.put(chr(len(s)))
+    else:
+        m.put_int(len(s))
+    m.put(s)
 
 def _marshal_unicode(space, s, m, w_unicode=None):
+    from rpython.rlib import rutf8
     if m.version >= 3:
         w_interned = space.get_interned_str(s)
     else:
@@ -458,10 +487,21 @@ def _marshal_unicode(space, s, m, w_unicode=None):
         typecode = TYPE_INTERNED  #   as a key for u.all_refs
     else:
         typecode = TYPE_UNICODE
+    if m.version >= 4:
+        # check whether it's ascii
+        if w_unicode is not None:
+            is_ascii = w_unicode.is_ascii()
+        else:
+            is_ascii = len(s) == rutf8.check_utf8(s, True)
+        if is_ascii:
+            _marshal_ascii_unicode(space, s, m, w_unicode, w_interned)
+            return
+    # general case, write unicode
     if w_unicode is not None:
         typecode = write_ref(typecode, w_unicode, m)
-    if typecode != FLAG_DONE:
-        m.atom_str(typecode, s)
+        if typecode == FLAG_DONE:
+            return
+    m.atom_str(typecode, s)
 
 # surrogate-preserving variants
 _decode_utf8 = unicodehelper.decode_utf8sp
@@ -481,32 +521,32 @@ def unmarshal_interned(space, u, tc):
     w_ret = unmarshal_unicode(space, u, tc)
     return u.space.new_interned_w_str(w_ret)
 
-def _unmarshal_ascii(u, short_length, interned):
+def _unmarshal_ascii(space, u, short_length, interned):
     from rpython.rlib import rutf8
     if short_length:
         lng = ord(u.get1())
     else:
         lng = u.get_lng()
     s = u.get(lng)
-    # Treat each chr as a single codepoint
-    utf8 = ''.join([rutf8.unichr_as_utf8(ord(c), True) for c in s])
-    w_u = u.space.newtext(utf8)
-    if interned:
-        w_u = u.space.new_interned_w_str(w_u)
+    try:
+        rutf8.check_ascii(s)
+    except rutf8.CheckError:
+        raise oefmt(space.w_ValueError, "bad marshal data (string is not ascii)")
+    w_u = u.space.newtext(s, len(s)) # ascii is valid utf-8
     return w_u
 
-@unmarshaller(TYPE_ASCII)    # nb. never generated by pypy so far
+@unmarshaller(TYPE_ASCII)
 def unmarshal_ascii(space, u, tc):
-    return _unmarshal_ascii(u, False, False)
+    return _unmarshal_ascii(space, u, False, False)
 @unmarshaller(TYPE_ASCII_INTERNED)
 def unmarshal_ascii(space, u, tc):
-    return _unmarshal_ascii(u, False, True)
+    return _unmarshal_ascii(space, u, False, True)
 @unmarshaller(TYPE_SHORT_ASCII)
 def unmarshal_ascii(space, u, tc):
-    return _unmarshal_ascii(u, True, False)
+    return _unmarshal_ascii(space, u, True, False)
 @unmarshaller(TYPE_SHORT_ASCII_INTERNED)
 def unmarshal_ascii(space, u, tc):
-    return _unmarshal_ascii(u, True, True)
+    return _unmarshal_ascii(space, u, True, True)
 
 
 @marshaller(W_SetObject)
