@@ -550,10 +550,6 @@ class StringMethods(object):
 
         sub = self._op_val(space, w_old)
         by = self._op_val(space, w_new)
-        # the following two lines are for being bug-to-bug compatible
-        # with CPython: see issue #2448
-        if count >= 0 and len(input) == 0:
-            return self._empty()
         try:
             res = replace(input, sub, by, count)
         except OverflowError:
@@ -820,6 +816,21 @@ class StringMethods(object):
     def descr_getnewargs(self, space):
         return space.newtuple([self._new(self._val(space))])
 
+    def descr_removeprefix(self, space, w_prefix):
+        prefix = self._op_val(space, w_prefix)
+        selfval = self._val(space)
+        if startswith(selfval, prefix):
+            return self._new(selfval[len(prefix):])
+        return self._new(selfval)
+
+    def descr_removesuffix(self, space, w_suffix):
+        suffix = self._op_val(space, w_suffix)
+        selfval = self._val(space)
+        if suffix and endswith(selfval, suffix):
+            end = len(selfval) - len(suffix)
+            assert end >= 0
+            return self._new(selfval[:end])
+        return self._new(selfval)
 
 # ____________________________________________________________
 # helpers for slow paths, moved out because they contain loops
