@@ -936,14 +936,8 @@ class AppTestBytesObject:
         raises(OverflowError, s.replace, b"", s)
 
     def test_replace_issue2448(self):
-        # CPython's replace() method has a bug that makes
-        #   ''.replace('', 'x')  gives a different answer than
-        #   ''.replace('', 'x', 1000).  This is the case in all
-        # known versions, at least until 2.7.13.  Some people
-        # call that a feature on the CPython issue report and
-        # the discussion dies out, so it might never be fixed.
-        assert ''.replace('', 'x') == 'x'
-        assert ''.replace('', 'x', 1000) == ''
+        assert b''.replace(b'', b'x') == b'x'
+        assert b''.replace(b'', b'x', 1000) == b'x'
 
     def test_getslice(self):
         s = b"abc"
@@ -1101,3 +1095,14 @@ class AppTestBytesObject:
         if hasattr(bytes.upper, 'im_func'):
             e = raises(TypeError, bytes.upper.im_func, 42)
             assert "'bytes'" in str(e.value)
+
+    def test_removeprefix(self):
+        assert b'abc'.removeprefix(b'x') == b'abc'
+        assert b'abc'.removeprefix(b'ab') == b'c'
+        assert b'abc'.removeprefix(b'') == b'abc'
+
+    def test_removesuffix(self):
+        assert b'abc'.removesuffix(b'x') == b'abc'
+        assert b'abc'.removesuffix(b'bc') == b'a'
+        assert b'abc'.removesuffix(b'') == b'abc'
+        assert b'spam'.removesuffix(b'am') == b'sp'
