@@ -3,7 +3,8 @@ import sys
 import py
 
 from pypy.objspace.std.dictmultiobject import (W_DictMultiObject,
-    W_DictObject, BytesDictStrategy, ObjectDictStrategy, UnicodeDictStrategy,
+    W_DictObject, BytesDictStrategy, ObjectDictStrategy, EmptyDictStrategy,
+    update1_dict_dict, UnicodeDictStrategy,
     IntDictStrategy)
 from pypy.objspace.std.longobject import W_LongObject
 from rpython.rlib.rbigint import rbigint
@@ -189,6 +190,17 @@ class TestW_DictObject(object):
         # w_l = self.space.call_method(w_d, "keys")
         # assert sorted(self.space.listview_unicode(w_l)) == [u"a", u"b"]
 
+    def test_update_empty_does_copy(self, monkeypatch):
+        w = self.space.wrap
+        wb = self.space.newbytes
+        w_empty = self.space.newdict()
+        w_d = self.space.newdict()
+        w_d.initialize_content([(w(1), wb("a")), (w(2), wb("b"))])
+        monkeypatch.setattr(EmptyDictStrategy, "setitem", None)
+        update1_dict_dict(self.space, w_empty, w_d)
+        assert self.space.eq_w(w_empty, w_d)
+
+
     def test_integer_strategy_with_w_long(self):
         space = self.space
         w = W_LongObject(rbigint.fromlong(42))
@@ -772,6 +784,29 @@ class AppTest_DictObject:
         d2 = my_dict_2({'a': 1, 'b': 2})
         assert dict(d2) == {'b': 2}  # uses overridden keys()
 
+    def test_or(self):
+        d = {'spam': 1, 'eggs': 2, 'cheese': 3}
+        e = {'cheese': 'cheddar', 'aardvark': 'Ethel'}
+        assert d | e == {'spam': 1, 'eggs': 2, 'cheese': 'cheddar', 'aardvark': 'Ethel'}
+        assert e | d == {'cheese': 3, 'aardvark': 'Ethel', 'spam': 1, 'eggs': 2}
+        assert d.__or__(None) is NotImplemented
+
+    def test_ior(self):
+        orig = d = {'spam': 1, 'eggs': 2, 'cheese': 3}
+        e = {'cheese': 'cheddar', 'aardvark': 'Ethel'}
+        d |= e
+        assert orig == {'spam': 1, 'eggs': 2, 'cheese': 'cheddar', 'aardvark': 'Ethel'}
+
+        d = orig = {"a": 6, "b": 7}
+        d |= [("b", 43), ("c", -1j)]
+        assert orig == {"a": 6, "b": 43, "c": -1j}
+
+
+    def test_class_getitem(self):
+        assert dict[int, str].__origin__ is dict
+        assert dict[int, str].__args__ == (int, str)
+
+
 class AppTest_DictMultiObject(AppTest_DictObject):
 
     def test_emptydict_unhashable(self):
@@ -1431,6 +1466,9 @@ class FakeSpace:
     def newtuple(self, l):
         return tuple(l)
 
+    def newtuple2(self, a, b):
+        return a, b
+
     def newdict(self, module=False, instance=False):
         return W_DictObject.allocate_and_init_instance(
                 self, module=module, instance=instance)
@@ -1476,6 +1514,7 @@ class Config:
         class std:
             methodcachesizeexp = 11
             withmethodcachecounter = False
+        honor__builtins__ = False
 
 FakeSpace.config = Config()
 
