@@ -580,6 +580,10 @@ class AppTestAppFloatTest:
         assert math.trunc(-100.999) == -100
         raises(OverflowError, math.trunc, float("inf"))
 
+    def test_floor_ceil(self):
+        import math
+        assert math.floor(1.5) == float(1.5).__floor__()
+        assert math.ceil(1.5) == float(1.5).__ceil__()
 
     def test_call_special(self):
         assert 5 .__add__(3.14) is NotImplemented
@@ -1030,7 +1034,7 @@ class AppTestFloatHex:
         raises(ValueError, float.fromhex, "0P")
 
     def test_division_edgecases(self):
-        import math, os
+        import math
 
         # inf
         inf = float("inf")
@@ -1041,14 +1045,9 @@ class AppTestFloatHex:
         assert math.isnan(y)
         x, y = divmod(3, inf)
         z = 3 % inf
-        if os.name == 'nt':
-            assert math.isnan(x)
-            assert math.isnan(y)
-            assert math.isnan(z)
-        else:
-            assert x == 0
-            assert y == 3
-            assert z == 3
+        assert x == 0
+        assert y == 3
+        assert z == 3
 
         # divide by 0
         raises(ZeroDivisionError, lambda: inf % 0)
