@@ -424,14 +424,27 @@ class TestW_ListObject(object):
         monkeypatch.setattr(self.space, "eq_w", None)
         w = self.space.wrap
         intlist = W_ListObject(self.space, [w(1),w(2),w(3),w(4),w(5),w(6),w(7)])
-        res = intlist.find(w(4), 0, 7)
+        res = intlist.find_or_count(w(4), 0, 7)
         assert res == 3
-        res = intlist.find(w(4), 0, 100)
+        res = intlist.find_or_count(w(4), 0, 100)
         assert res == 3
         with py.test.raises(ValueError):
-            intlist.find(w(4), 4, 7)
+            intlist.find_or_count(w(4), 4, 7)
         with py.test.raises(ValueError):
-            intlist.find(w(4), 0, 2)
+            intlist.find_or_count(w(4), 0, 2)
+
+    def test_count_fast_on_intlist(self, monkeypatch):
+        monkeypatch.setattr(self.space, "eq_w", None)
+        w = self.space.wrap
+        intlist = W_ListObject(self.space, [w(1),w(2),w(3),w(4),w(5),w(6),w(7), w(4)])
+        res = intlist.find_or_count(w(4), 0, 7, count=True)
+        assert res == 1
+        res = intlist.find_or_count(w(4), 0, 100, count=True)
+        assert res == 2
+        res = intlist.find_or_count(w(4), 4, 7, count=True)
+        assert res == 0
+        res = intlist.find_or_count(w(4), 0, 2, count=True)
+        assert res == 0
 
     def test_intlist_to_object_too_much_wrapping(self):
         w = self.space.wrap
@@ -507,9 +520,14 @@ class AppTestListObject(object):
         cls.w_runappdirect = cls.space.wrap(cls.runappdirect)
 
     def test_doc(self):
-        assert list.__doc__ == "list() -> new empty list\nlist(iterable) -> new list initialized from iterable's items"
+        assert list.__doc__ == """Built-in mutable sequence.
+
+If no argument is given, the constructor creates a new empty list.
+The argument must be an iterable if specified."""
         assert list.__new__.__doc__ == "Create and return a new object.  See help(type) for accurate signature."
         assert list.__init__.__doc__ == "Initialize self.  See help(type(self)) for accurate signature."
+        print(list.__text_signature__)
+        assert list.__text_signature__ == "(iterable=(), /)"
 
     def test_getstrategyfromlist_w(self):
         l0 = ["a", "2", "a", True]
@@ -586,13 +604,13 @@ class AppTestListObject(object):
         foo_list = [foo1, foo2, foo3]
         foo42 in foo_list
         logger_copy = logger[:]  # prevent re-evaluation during pytest error print
-        assert logger_copy == [(foo42, foo1), (foo42, foo2), (foo42, foo3)]
+        assert logger_copy == [(foo1, foo42), (foo2, foo42), (foo3, foo42)]
 
         del logger[:]
         foo2_bis = Foo(2, '2 bis')
         foo2_bis in foo_list
         logger_copy = logger[:]  # prevent re-evaluation during pytest error print
-        assert logger_copy == [(foo2_bis, foo1), (foo2_bis, foo2)]
+        assert logger_copy == [(foo1, foo2_bis), (foo2, foo2_bis)]
 
     def test_call_list(self):
         assert list('') == []
@@ -1215,6 +1233,22 @@ class AppTestListObject(object):
         l.append(l)
         assert repr(l) == '[[...]]'
 
+    def test_repr_strategies(self):
+        assert repr([]) == '[]'
+        assert repr([1, 2, 3]) == '[1, 2, 3]'
+        assert repr([1.1, 2.1, 3.1]) == '[1.1, 2.1, 3.1]'
+        assert repr([1.1, 2.1, 3.1, float('nan')]) == '[1.1, 2.1, 3.1, nan]'
+        # mixed
+        assert repr([100, 23, -1.1, 2.1, 3.1, float('nan')]) == '[100, 23, -1.1, 2.1, 3.1, nan]'
+        # bytes
+        assert repr([b'a', b'b', b'c']) == "[b'a', b'b', b'c']"
+        # asciilist
+        assert repr([u'a', u'b', u'c']) == "['a', 'b', 'c']"
+        # some objects
+        o1 = object()
+        d = {1: 2, None: False}
+        assert repr([o1, d]) == '[%r, %r]' % (o1, d)
+
     def test_copy(self):
         # test that empty list copies the empty list
         l = []
@@ -1286,6 +1320,42 @@ class AppTestListObject(object):
         assert c.count('h') == 1
         assert c.count('w') == 0
 
+        c = [0, 1, 2, 2, 3]
+        assert c.count(2) == 2
+        assert c.count(1) == 1
+        assert c.count(0) == 1
+        assert c.count(10) == 0
+
+        c = [0., 1., 2., 2., 3., 0.0, -0.0]
+        assert c.count(2) == 2
+        assert c.count(1) == 1
+        assert c.count(0) == 3
+        assert c.count(10) == 0
+        assert c.count('abc') == 0
+
+        c = [0, 1, 2, 2., 3, 0.0, -0.0]
+        assert c.count(2) == 2
+        assert c.count(1) == 1
+        assert c.count(0) == 3
+        assert c.count(10) == 0
+        assert c.count(2.) == 2
+        assert c.count(1.) == 1
+        assert c.count(0.) == 3
+        assert c.count(-0.) == 3
+        assert c.count(10.) == 0
+        assert c.count('abc') == 0
+
+        c = [0, "abc", 1, 2, 2., 3, 0.0, -0.0, None]
+        assert c.count(2) == 2
+        assert c.count(2.) == 2
+        assert c.count(1) == 1
+        assert c.count(1.) == 1
+        assert c.count(0) == 3
+        assert c.count(0.0) == 3
+        assert c.count(-0.0) == 3
+        assert c.count(10) == 0
+        assert c.count('abc') == 1
+
     def test_insert(self):
         c = list('hello world')
         c.insert(0, 'X')
@@ -1741,6 +1811,12 @@ class AppTestListObject(object):
         with raises(TypeError):
             list(sequence=[])
 
+    def test_generic_alias(self):
+        ga = list[int]
+        assert ga.__origin__ is list
+        assert ga.__args__ == (int, )
+
+
 class AppTestWithoutStrategies:
     spaceconfig = {"objspace.std.withliststrategies": False}
 
