@@ -142,6 +142,14 @@ class TestW_SetObject:
         assert sorted(self.space.listview_int(w_b)) == [1,2,3,4,5]
         assert self.space.listview_bytes(w_b) is None
 
+    def test_cpyext_add_frozen(self, space):
+        t1 = W_FrozensetObject(space)
+        assert space.len_w(t1) == 0
+        res = t1.cpyext_add_frozen(space.newint(1))
+        assert res
+        assert space.len_w(t1) == 1
+
+
 class AppTestAppSetTest:
 
     def setup_class(self):
@@ -886,6 +894,7 @@ class AppTestAppSetTest:
 
     def test_empty_unhashable(self):
         s = set()
+        raises(TypeError, s.__contains__, [[]])
         raises(TypeError, s.difference, [[]])
         raises(TypeError, s.difference_update, [[]])
         raises(TypeError, s.intersection, [[]])
@@ -1177,3 +1186,16 @@ class AppTestAppSetTest:
         x = frozenset()
         raises(TypeError, set.add, x, 1)
         raises(TypeError, set.__ior__, x, set([2]))
+
+    def test_class_getitem(self):
+        for cls in set, frozenset:
+            assert set[int, str].__origin__ is set
+            assert set[int, str].__args__ == (int, str)
+
+    def test_frozenset_hash_like_cpython(self):
+        import sys
+        if sys.maxsize != 2**63 - 1:
+            skip("64 bit only")
+        assert hash(frozenset()) == 133146708735736
+        h = hash(frozenset([1, 2, 9]))
+        assert h == (-5390384031640186368)
