@@ -93,6 +93,10 @@ class AppTestTypeObject:
         x = X()
         exc = raises(AttributeError, "x.a")
         assert str(exc.value) == "'X' object has no attribute 'a'"
+        assert exc.value.name == "a"
+        exc = raises(AttributeError, "X.a")
+        assert str(exc.value) == "type object 'X' has no attribute 'a'"
+        assert exc.value.name == "a"
 
     def test_call_type(self):
         assert type(42) is int
@@ -1020,6 +1024,23 @@ class AppTestTypeObject:
         assert not A == B
         assert not A != A
 
+    def test_compare_overridden(self):
+        """
+        class AMeta(type):
+            def __eq__(self, other):
+                return True
+
+        class A(metaclass=AMeta):
+            pass
+
+        assert (A == 1) == True
+        assert (1 == A) == True
+
+        assert (A != 1) == False
+        assert (1 != A) == False
+        """
+
+
     def test_class_variations(self):
         class A(object):
             pass
@@ -1498,6 +1519,24 @@ class AppTestTypeObject:
         assert not hasattr(TestNoOrigBases, '__orig_bases__')
         """
 
+    def test_mro_entries_bug(self):
+        class A:
+            pass
+
+        def f(): pass
+        f.__mro_entries__ = lambda bases: (A, )
+        class BuggyMcBugFace(f):
+            pass
+        assert BuggyMcBugFace.__base__ is A
+
+    def test_mro_entries_type_ignored(self):
+        class A:
+            def __mro_entries__(self, bases):
+                assert 0
+        class B(A):
+            pass
+        assert B.__base__ is A
+
     def test_classcell_missing(self):
         """
         # Some metaclasses may not pass the original namespace to type.__new__
@@ -1521,6 +1560,12 @@ class AppTestTypeObject:
                     return __class__
         """
 
+    def test_class_getitem(self):
+        ga = type[int]
+        assert ga.__origin__ is type
+        assert ga.__args__ == (int, )
+
+
 class AppTestWithMethodCacheCounter:
     spaceconfig = {"objspace.std.withmethodcachecounter": True}
 
