@@ -134,6 +134,21 @@ class TestUnicodeObject:
         w_b = encode_object(self.space, self.space.newutf8("abc", 3), "ascii", "strict")
         assert self.space.bytes_w(w_b) == "abc"
 
+    def test_utf8_ascii_encode_shortcut_ascii(self, monkeypatch):
+        from rpython.rlib import rutf8
+        from pypy.objspace.std.unicodeobject import encode_object
+        monkeypatch.setattr(rutf8, "check_utf8", None)
+        for enc in ["utf-8", "UTF-8", "utf8"]:
+            w_b = encode_object(self.space, self.space.newutf8("abc", 3), enc, "strict")
+            assert self.space.bytes_w(w_b) == "abc"
+
+    def test_split_shortcut_ascii(self, monkeypatch):
+        from rpython.rlib import rutf8
+        monkeypatch.setattr(rutf8, "isspace", None)
+        w_s = self.space.newutf8("a b c", 5)
+        w_l = w_s.descr_split(self.space) # no crash
+        assert self.space.len_w(w_l) == 3
+
 
 class AppTestUnicodeStringStdOnly:
     def test_compares(self):
@@ -486,6 +501,11 @@ class AppTestUnicodeString:
         s = "a" * (2**16)
         raises(OverflowError, s.replace, "", s)
 
+    def test_empty_replace_empty(self):
+        assert "".replace("", "a", 0) == ""
+        assert "".replace("", "a", 1) == "a"
+        assert "".replace("", "a", 121344) == "a"
+
     def test_strip(self):
         s = " a b "
         assert s.strip() == "a b"
@@ -816,6 +836,10 @@ class AppTestUnicodeString:
         # CESU-8
         raises(UnicodeDecodeError, b'\xed\xa0\xbc\xed\xb2\xb1'.decode, 'utf-8')
 
+    def test_encode_fast_path_bug_position(self):
+        info = raises(UnicodeEncodeError, "ää\ud800".encode, "utf-8")
+        assert info.value.start == 2 # used to be 4
+
     def test_codecs_errors(self):
         # Error handling (encoding)
         raises(UnicodeError, 'Andr\202 x'.encode, 'ascii')
@@ -1354,3 +1378,13 @@ class AppTestUnicodeString:
     def test_replace_no_occurrence(self):
         x = u"xyz"
         assert x.replace(u"a", u"b") is x
+
+    def test_removeprefix(self):
+        assert u'abc'.removeprefix(u'x') == u'abc'
+        assert u'abc'.removeprefix(u'ab') == u'c'
+
+    def test_removesuffix(self):
+        assert u'abc'.removesuffix(u'x') == u'abc'
+        assert u'abc'.removesuffix(u'bc') == u'a'
+        assert u'abc'.removesuffix(u'') == u'abc'
+        assert u'spam'.removesuffix(u'am') == u'sp'
