@@ -42,8 +42,11 @@ def codepoint_at_pos_dont_look_inside(utf8, p):
     return rutf8.codepoint_at_pos(utf8, p)
 
 
-joindriver = jit.JitDriver(greens = ['selfisnotempty', 'tpfirst', 'tplist'], reds = 'auto',
-                           name='joindriver')
+def get_printable_location(selfisnotempty, tpfirst, greenkey):
+    return "unicode.join [selfisnotempty=%s, tpfirst=%s, %s]" % (
+            selfisnotempty, tpfirst, greenkey.iterator_greenkey_printable())
+joindriver = jit.JitDriver(greens = ['selfisnotempty', 'tpfirst', 'greenkey'], reds = 'auto',
+                           name='joindriver', get_printable_location=get_printable_location)
 
 class BadUtf8(Exception):
     pass
@@ -131,6 +134,7 @@ class W_UnicodeObject(W_Root):
         return self._utf8
 
     def utf8_w(self, space):
+        jit.record_known_result(self._length, rutf8._check_utf8, self._utf8, True, 0, -1)
         return self._utf8
 
     def listview_ascii(self):
@@ -433,6 +437,9 @@ class W_UnicodeObject(W_Root):
         return mod_format(space, self, w_values, fmt_type=FORMAT_UNICODE)
 
     def descr_rmod(self, space, w_values):
+        # this should never be reachable for valid invocations
+        if not space.isinstance_w(w_values, space.w_unicode):
+            return space.w_NotImplemented
         return mod_format(space, w_values, self, fmt_type=FORMAT_UNICODE)
 
     def descr_swapcase(self, space):
@@ -689,9 +696,9 @@ class W_UnicodeObject(W_Root):
         size = 1
         selfisnotempty = self._length != 0
         tpfirst = space.type(w_first)
-        tplist = space.type(w_iterable)
+        greenkey = space.iterator_greenkey(w_iter)
         while 1:
-            joindriver.jit_merge_point(tpfirst=tpfirst, tplist=tplist, selfisnotempty=selfisnotempty)
+            joindriver.jit_merge_point(tpfirst=tpfirst, greenkey=greenkey, selfisnotempty=selfisnotempty)
             try:
                 w_element = space.next(w_iter)
             except OperationError as e:
@@ -957,9 +964,14 @@ class W_UnicodeObject(W_Root):
     def descr_split(self, space, w_sep=None, maxsplit=-1):
         res = []
         value = self._utf8
+        is_ascii = self.is_ascii()
         if space.is_none(w_sep):
-            res = split(value, maxsplit=maxsplit, isutf8=True)
-            return space.newlist_utf8(res, self.is_ascii())
+            # need two calls, due to the specialization
+            if is_ascii:
+                res = split(value, maxsplit=maxsplit, isutf8=False)
+            else:
+                res = split(value, maxsplit=maxsplit, isutf8=True)
+            return space.newlist_utf8(res, is_ascii)
 
         by = self.convert_arg_to_w_unicode(space, w_sep)._utf8
         if len(by) == 0:
@@ -1142,10 +1154,6 @@ class W_UnicodeObject(W_Root):
 
         w_sub = self.convert_arg_to_w_unicode(space, w_old)
         w_by = self.convert_arg_to_w_unicode(space, w_new)
-        # the following two lines are for being bug-to-bug compatible
-        # with CPython: see issue #2448
-        if count >= 0 and len(input) == 0:
-            return self._empty()
         try:
             res, replacements = replace_count(input, w_sub._utf8, w_by._utf8,
                                               count, isutf8=True)
@@ -1422,6 +1430,28 @@ class W_UnicodeObject(W_Root):
         assert isinstance(self, W_UnicodeObject)
         return self._getitem_result(space, index)
 
+    def descr_removeprefix(self, space, w_prefix):
+        w_prefix = self.convert_arg_to_w_unicode(space, w_prefix)
+        prefix = w_prefix._utf8
+        selfval = self._utf8
+        if startswith(selfval, prefix):
+            return W_UnicodeObject(selfval[len(prefix):], self._length - w_prefix._length)
+        if type(self) is W_UnicodeObject:
+            return self
+        return W_UnicodeObject(selfval, self._length)
+
+    def descr_removesuffix(self, space, w_suffix):
+        w_suffix = self.convert_arg_to_w_unicode(space, w_suffix)
+        suffix = w_suffix._utf8
+        selfval = self._utf8
+        if suffix and endswith(selfval, suffix):
+            end = len(selfval) - len(suffix)
+            assert end >= 0
+            return W_UnicodeObject(selfval[:end], self._length - w_suffix._length)
+        if type(self) is W_UnicodeObject:
+            return self
+        return W_UnicodeObject(selfval, self._length)
+
 
 def _isidentifier(u):
     if not u:
@@ -1460,35 +1490,23 @@ def get_encoding_and_errors(space, w_encoding, w_errors):
     return encoding, errors
 
 def encode_object(space, w_obj, encoding, errors):
-    from pypy.module._codecs.interp_codecs import _call_codec, lookup_text_codec
-    if errors is None or errors == 'strict':
-        # fast paths
-        utf8 = space.utf8_w(w_obj)
-        if encoding is None or encoding == 'utf-8':
-            try:
-                rutf8.check_utf8(utf8, False)
-            except rutf8.CheckError as a:
-                eh = unicodehelper.encode_error_handler(space)
-                eh(None, "utf-8", "surrogates not allowed", utf8,
-                    a.pos, a.pos + 1)
-                assert False, "always raises"
-            return space.newbytes(utf8)
-        if ((encoding == "latin1" or encoding == "latin-1") and
-                isinstance(w_obj, W_UnicodeObject) and w_obj.is_ascii()):
-            return space.newbytes(w_obj._utf8)
-        elif encoding == 'ascii':
-            try:
-                if not (isinstance(w_obj, W_UnicodeObject) and w_obj.is_ascii()):
-                    rutf8.check_ascii(utf8)
-            except rutf8.CheckError as a:
-                eh = unicodehelper.encode_error_handler(space)
-                eh(None, "ascii", "ordinal not in range(128)", utf8,
-                    a.pos, a.pos + 1)
-                assert False, "always raises"
+    from pypy.module._codecs.interp_codecs import(
+            _call_codec, lookup_text_codec, lookup_error)
+    # fast paths, only ascii, in the other cases it's too easy to mess up
+    # the errors
+    if ((errors is None or errors == 'strict') and
+            isinstance(w_obj, W_UnicodeObject) and w_obj.is_ascii()):
+        utf8 = w_obj.utf8_w(space)
+        if (encoding is None or encoding == 'utf-8' or
+                encoding == 'utf8' or encoding == 'UTF-8' or
+                encoding == "latin1" or encoding == "latin-1" or
+                encoding == 'ascii'):
             return space.newbytes(utf8)
     if encoding is None:
         encoding = space.sys.defaultencoding
     w_codec_info = lookup_text_codec(space, 'encode', encoding)
+    if errors is not None and space.sys.get_flag('dev_mode'):
+        lookup_error(space, errors)
     w_encfunc = space.getitem(w_codec_info, space.newint(0))
     w_retval = _call_codec(space, w_encfunc, w_obj, "encoding", encoding, errors)
     if not space.isinstance_w(w_retval, space.w_bytes):
@@ -1501,7 +1519,8 @@ def encode_object(space, w_obj, encoding, errors):
 
 
 def decode_object(space, w_obj, encoding, errors=None):
-    from pypy.module._codecs.interp_codecs import _call_codec, lookup_text_codec
+    from pypy.module._codecs.interp_codecs import(
+            _call_codec, lookup_text_codec, lookup_error)
     # in all cases, call space.charbuf_w() first.  This will fail with a
     # TypeError if w_obj is of a random type.  Do this even if we're not
     # going to use 's'
@@ -1523,6 +1542,8 @@ def decode_object(space, w_obj, encoding, errors=None):
             return space.newutf8(s, lgt)
     if encoding is None:
         encoding = space.sys.defaultencoding
+    if errors is not None and space.sys.get_flag('dev_mode'):
+        lookup_error(space, errors)
     w_codec_info = lookup_text_codec(space, 'decode', encoding)
     w_encfunc = space.getitem(w_codec_info, space.newint(1))
     w_retval = _call_codec(space, w_encfunc, w_obj, "decoding", encoding, errors)
@@ -2139,6 +2160,9 @@ W_UnicodeObject.typedef = TypeDef(
     maketrans = interp2app(W_UnicodeObject.descr_maketrans,
                            as_classmethod=True,
                            doc=UnicodeDocstrings.maketrans.__doc__),
+
+    removeprefix = interp2app(W_UnicodeObject.descr_removeprefix),
+    removesuffix = interp2app(W_UnicodeObject.descr_removesuffix),
 )
 W_UnicodeObject.typedef.flag_sequence_bug_compat = True
 
