@@ -55,7 +55,7 @@ class Base:
 def test_all_opcodes_defined():
     opnames = set(host_bytecode_spec.method_names)
     methods = set([name for name in dir(FlowContext) if name.upper() == name])
-    handled_elsewhere = set(['EXTENDED_ARG'])
+    handled_elsewhere = set(['EXTENDED_ARG', 'LOAD_REVDB_VAR'])
     missing = opnames - methods - handled_elsewhere
     assert not missing
 
@@ -1015,7 +1015,7 @@ class TestFlowObjSpace(Base):
 
             # this code is generated by pypy-c when compiling above f
             pypy_code = 't\x00\x00\x83\x00\x00}\x00\x00|\x00\x00\xc9\x01\x00\xca\x00\x00S'
-            new_c = self.monkey_patch_code(f.func_code, 3, 3, pypy_code, ('X', 'x', 'm'), ('x',))
+            new_c = self.monkey_patch_code(f.__code__, 3, 3, pypy_code, ('X', 'x', 'm'), ('x',))
             f2 = types.FunctionType(new_c, locals(), 'f')
 
             graph = self.codetest(f2)
@@ -1034,7 +1034,7 @@ class TestFlowObjSpace(Base):
 
             # this code is generated by pypy-c when compiling above f
             pypy_code = 'd\x01\x00\xcb\x00\x00D]\x0c\x00}\x00\x00|\x00\x00^\x02\x00q\x07\x00S'
-            new_c = self.monkey_patch_code(f.func_code, 3, 67, pypy_code, (),
+            new_c = self.monkey_patch_code(f.__code__, 3, 67, pypy_code, (),
                                            ('i',))
             f2 = types.FunctionType(new_c, locals(), 'f')
 
@@ -1156,7 +1156,7 @@ class TestFlowObjSpace(Base):
         def f():
             x = 5
             return x
-            exec "None"
+            exec("None")
         graph = self.codetest(f)
         assert len(graph.startblock.exits) == 1
         assert graph.startblock.exits[0].target == graph.returnblock
