@@ -5,7 +5,7 @@ from rpython.jit.metainterp.optimizeopt.intutils import IntBound
 from rpython.jit.metainterp.optimizeopt.optimizer import (Optimization, CONST_1,
     CONST_0)
 from rpython.jit.metainterp.optimizeopt.util import (
-    make_dispatcher_method, get_box_replacement)
+    make_dispatcher_method, have_dispatcher_method, get_box_replacement)
 from .info import getptrinfo
 from rpython.jit.metainterp.resoperation import rop
 from rpython.jit.metainterp.optimizeopt import vstring
@@ -34,32 +34,22 @@ class OptIntBounds(Optimization):
     def propagate_forward(self, op):
         return dispatch_opt(self, op)
 
-    def propagate_postprocess(self, op):
-        return dispatch_postprocess(self, op)
-
     def propagate_bounds_backward(self, box):
         # FIXME: This takes care of the instruction where box is the result
         #        but the bounds produced by all instructions where box is
         #        an argument might also be tighten
         b = self.getintbound(box)
         if b.is_constant():
-            self.make_constant_int(box, b.getint())
+            self.make_constant_int(box, b.get_constant_int())
 
         box1 = self.optimizer.as_operation(box)
         if box1 is not None:
             dispatch_bounds_ops(self, box1)
 
-    def _optimize_guard_true_false_value(self, op):
-        return self.emit(op)
-
     def _postprocess_guard_true_false_value(self, op):
         if op.getarg(0).type == 'i':
             self.propagate_bounds_backward(op.getarg(0))
 
-    optimize_GUARD_TRUE = _optimize_guard_true_false_value
-    optimize_GUARD_FALSE = _optimize_guard_true_false_value
-    optimize_GUARD_VALUE = _optimize_guard_true_false_value
-
     postprocess_GUARD_TRUE = _postprocess_guard_true_false_value
     postprocess_GUARD_FALSE = _postprocess_guard_true_false_value
     postprocess_GUARD_VALUE = _postprocess_guard_true_false_value
@@ -76,10 +66,8 @@ class OptIntBounds(Optimization):
         return self.emit(op)
 
     def postprocess_INT_OR_or_XOR(self, op):
-        v1 = get_box_replacement(op.getarg(0))
-        b1 = self.getintbound(v1)
-        v2 = get_box_replacement(op.getarg(1))
-        b2 = self.getintbound(v2)
+        b1 = self.getintbound(op.getarg(0))
+        b2 = self.getintbound(op.getarg(1))
         b = b1.or_bound(b2)
         self.getintbound(op).intersect(b)
 
@@ -89,24 +77,17 @@ class OptIntBounds(Optimization):
     postprocess_INT_OR = postprocess_INT_OR_or_XOR
     postprocess_INT_XOR = postprocess_INT_OR_or_XOR
 
-    def optimize_INT_AND(self, op):
-        return self.emit(op)
-
     def postprocess_INT_AND(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
         b = b1.and_bound(b2)
         self.getintbound(op).intersect(b)
 
-    def optimize_INT_SUB(self, op):
-        return self.emit(op)
-
     def postprocess_INT_SUB(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
         b = b1.sub_bound(b2)
-        if b.bounded():
-            self.getintbound(op).intersect(b)
+        self.getintbound(op).intersect(b)
 
     def optimize_INT_ADD(self, op):
         arg1 = get_box_replacement(op.getarg(0))
@@ -142,7 +123,7 @@ class OptIntBounds(Optimization):
                         prod_arg1, prod_arg2 = prod_arg2, prod_arg1
                         prod_v1, prod_v2 = prod_v2, prod_v1
                     if prod_v2.is_constant():
-                        sum = intmask(arg2.getint() + prod_arg2.getint())
+                        sum = intmask(v2.get_constant_int() + prod_v2.get_constant_int())
                         arg1 = prod_arg1
                         arg2 = ConstInt(sum)
                         op = self.replace_op_with(op, rop.INT_ADD, args=[arg1, arg2])
@@ -154,28 +135,14 @@ class OptIntBounds(Optimization):
         b2 = self.getintbound(op.getarg(1))
         r = self.getintbound(op)
         b = b1.add_bound(b2)
-        # NB: the result only gets its bound updated if b has an upper and a
-        # lower bound. This is important, to do the right thing in the presence
-        # of overflow. Example:
-        # y = x + 1 where x >= 0
-        # here it's tempting to give a bound of y >= 1, but that would be
-        # wrong, due to wraparound
-        if b.bounded():
-            r.intersect(b)
-
-    def optimize_INT_MUL(self, op):
-        return self.emit(op)
+        r.intersect(b)
 
     def postprocess_INT_MUL(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
         r = self.getintbound(op)
         b = b1.mul_bound(b2)
-        if b.bounded():
-            r.intersect(b)
-
-    def optimize_CALL_PURE_I(self, op):
-        return self.emit(op)
+        r.intersect(b)
 
     def postprocess_CALL_PURE_I(self, op):
         # dispatch based on 'oopspecindex' to a method that handles
@@ -199,9 +166,6 @@ class OptIntBounds(Optimization):
         r = self.getintbound(op)
         r.intersect(b1.mod_bound(b2))
 
-    def optimize_INT_LSHIFT(self, op):
-        return self.emit(op)
-
     def postprocess_INT_LSHIFT(self, op):
         arg0 = get_box_replacement(op.getarg(0))
         b1 = self.getintbound(arg0)
@@ -210,10 +174,7 @@ class OptIntBounds(Optimization):
         r = self.getintbound(op)
         b = b1.lshift_bound(b2)
         r.intersect(b)
-        # intbound.lshift_bound checks for an overflow and if the
-        # lshift can be proven not to overflow sets b.has_upper and
-        # b.has_lower
-        if b.bounded():
+        if b1.lshift_bound_cannot_overflow(b2):
             # Synthesize the reverse op for optimize_default to reuse
             self.pure_from_args(rop.INT_RSHIFT,
                                 [op, arg1], arg0)
@@ -277,22 +238,25 @@ class OptIntBounds(Optimization):
     def optimize_INT_ADD_OVF(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
-        resbound = b1.add_bound(b2)
-        if resbound.bounded():
+        if b1.add_bound_cannot_overflow(b2):
             # Transform into INT_ADD.  The following guard will be killed
             # by optimize_GUARD_NO_OVERFLOW; if we see instead an
             # optimize_GUARD_OVERFLOW, then InvalidLoop.
 
-            # NB: this case also takes care of int_add_ovf with 0 as on of the
-            # arguments: the result will be bounded, and then the optimization
-            # for int_add with 0 as argument will remove the op.
+            # NB: this case also takes care of int_add_ovf with 0 as one of the
+            # arguments
             op = self.replace_op_with(op, rop.INT_ADD)
         return self.emit(op)
 
     def postprocess_INT_ADD_OVF(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
-        resbound = b1.add_bound(b2)
+        # we can always give the result a bound. if the int_add_ovf is followed
+        # by a guard_no_overflow, then we know no overflow occurred, and the
+        # bound is correct. Otherwise, it must be followed by a guard_overflow
+        # and it is also fine to give the result a bound, because the result
+        # box must never be used in the rest of the trace
+        resbound = b1.add_bound_no_overflow(b2)
         r = self.getintbound(op)
         r.intersect(resbound)
 
@@ -304,8 +268,7 @@ class OptIntBounds(Optimization):
         if arg0.same_box(arg1):
             self.make_constant_int(op, 0)
             return None
-        resbound = b0.sub_bound(b1)
-        if resbound.bounded():
+        if b0.sub_bound_cannot_overflow(b1):
             # this case takes care of int_sub_ovf(x, 0) as well
             op = self.replace_op_with(op, rop.INT_SUB)
         return self.emit(op)
@@ -315,15 +278,14 @@ class OptIntBounds(Optimization):
         arg1 = get_box_replacement(op.getarg(1))
         b0 = self.getintbound(arg0)
         b1 = self.getintbound(arg1)
-        resbound = b0.sub_bound(b1)
+        resbound = b0.sub_bound_no_overflow(b1)
         r = self.getintbound(op)
         r.intersect(resbound)
 
     def optimize_INT_MUL_OVF(self, op):
-        b1 = self.getintbound(op.getarg(0))
-        b2 = self.getintbound(op.getarg(1))
-        resbound = b1.mul_bound(b2)
-        if resbound.bounded():
+        b0 = self.getintbound(op.getarg(0))
+        b1 = self.getintbound(op.getarg(1))
+        if b0.mul_bound_cannot_overflow(b1):
             # this case also takes care of multiplication with 0 and 1
             op = self.replace_op_with(op, rop.INT_MUL)
         return self.emit(op)
@@ -331,7 +293,7 @@ class OptIntBounds(Optimization):
     def postprocess_INT_MUL_OVF(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
-        resbound = b1.mul_bound(b2)
+        resbound = b1.mul_bound_no_overflow(b2)
         r = self.getintbound(op)
         r.intersect(resbound)
 
@@ -515,99 +477,63 @@ class OptIntBounds(Optimization):
         bres = self.getintbound(op)
         bres.intersect(bounds)
 
-    def optimize_ARRAYLEN_GC(self, op):
-        return self.emit(op)
-
     def postprocess_ARRAYLEN_GC(self, op):
         array = self.ensure_ptr_info_arg0(op)
         self.optimizer.setintbound(op, array.getlenbound(None))
 
-    def optimize_STRLEN(self, op):
-        return self.emit(op)
-
     def postprocess_STRLEN(self, op):
         self.make_nonnull_str(op.getarg(0), vstring.mode_string)
         array = getptrinfo(op.getarg(0))
         self.optimizer.setintbound(op, array.getlenbound(vstring.mode_string))
 
-    def optimize_UNICODELEN(self, op):
-        return self.emit(op)
-
     def postprocess_UNICODELEN(self, op):
         self.make_nonnull_str(op.getarg(0), vstring.mode_unicode)
         array = getptrinfo(op.getarg(0))
         self.optimizer.setintbound(op, array.getlenbound(vstring.mode_unicode))
 
-    def optimize_STRGETITEM(self, op):
-        return self.emit(op)
-
     def postprocess_STRGETITEM(self, op):
         v1 = self.getintbound(op)
         v2 = getptrinfo(op.getarg(0))
         intbound = self.getintbound(op.getarg(1))
-        if intbound.has_lower and v2 is not None:
+        if v2 is not None:
             lenbound = v2.getlenbound(vstring.mode_string)
             if lenbound is not None:
                 lenbound.make_gt_const(intbound.lower)
         v1.intersect_const(0, 255)
 
-    def optimize_GETFIELD_RAW_I(self, op):
-        return self.emit(op)
-
     def postprocess_GETFIELD_RAW_I(self, op):
         descr = op.getdescr()
         if descr.is_integer_bounded():
             b1 = self.getintbound(op)
             b1.intersect_const(descr.get_integer_min(), descr.get_integer_max())
 
-    optimize_GETFIELD_RAW_F = optimize_GETFIELD_RAW_I
-    optimize_GETFIELD_RAW_R = optimize_GETFIELD_RAW_I
-    optimize_GETFIELD_GC_I = optimize_GETFIELD_RAW_I
-    optimize_GETFIELD_GC_R = optimize_GETFIELD_RAW_I
-    optimize_GETFIELD_GC_F = optimize_GETFIELD_RAW_I
-
     postprocess_GETFIELD_RAW_F = postprocess_GETFIELD_RAW_I
     postprocess_GETFIELD_RAW_R = postprocess_GETFIELD_RAW_I
     postprocess_GETFIELD_GC_I = postprocess_GETFIELD_RAW_I
     postprocess_GETFIELD_GC_R = postprocess_GETFIELD_RAW_I
     postprocess_GETFIELD_GC_F = postprocess_GETFIELD_RAW_I
 
-    optimize_GETINTERIORFIELD_GC_I = optimize_GETFIELD_RAW_I
-    optimize_GETINTERIORFIELD_GC_R = optimize_GETFIELD_RAW_I
-    optimize_GETINTERIORFIELD_GC_F = optimize_GETFIELD_RAW_I
-
     postprocess_GETINTERIORFIELD_GC_I = postprocess_GETFIELD_RAW_I
     postprocess_GETINTERIORFIELD_GC_R = postprocess_GETFIELD_RAW_I
     postprocess_GETINTERIORFIELD_GC_F = postprocess_GETFIELD_RAW_I
 
-    def optimize_GETARRAYITEM_RAW_I(self, op):
-        return self.emit(op)
-
     def postprocess_GETARRAYITEM_RAW_I(self, op):
         descr = op.getdescr()
         if descr and descr.is_item_integer_bounded():
             intbound = self.getintbound(op)
             intbound.intersect_const(descr.get_item_integer_min(), descr.get_item_integer_max())
 
-    optimize_GETARRAYITEM_RAW_F = optimize_GETARRAYITEM_RAW_I
-    optimize_GETARRAYITEM_GC_I = optimize_GETARRAYITEM_RAW_I
-    optimize_GETARRAYITEM_GC_F = optimize_GETARRAYITEM_RAW_I
-    optimize_GETARRAYITEM_GC_R = optimize_GETARRAYITEM_RAW_I
-
     postprocess_GETARRAYITEM_RAW_F = postprocess_GETARRAYITEM_RAW_I
     postprocess_GETARRAYITEM_GC_I = postprocess_GETARRAYITEM_RAW_I
     postprocess_GETARRAYITEM_GC_F = postprocess_GETARRAYITEM_RAW_I
     postprocess_GETARRAYITEM_GC_R = postprocess_GETARRAYITEM_RAW_I
 
-    def optimize_UNICODEGETITEM(self, op):
-        return self.emit(op)
-
     def postprocess_UNICODEGETITEM(self, op):
         b1 = self.getintbound(op)
         b1.make_ge_const(0)
         v2 = getptrinfo(op.getarg(0))
         intbound = self.getintbound(op.getarg(1))
-        if intbound.has_lower and v2 is not None:
+        if v2 is not None:
             lenbound = v2.getlenbound(vstring.mode_unicode)
             if lenbound is not None:
                 lenbound.make_gt_const(intbound.lower)
@@ -637,37 +563,37 @@ class OptIntBounds(Optimization):
     def propagate_bounds_INT_LT(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_int_lt(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_int_ge(op.getarg(0), op.getarg(1))
 
     def propagate_bounds_INT_GT(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_int_gt(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_int_le(op.getarg(0), op.getarg(1))
 
     def propagate_bounds_INT_LE(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_int_le(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_int_gt(op.getarg(0), op.getarg(1))
 
     def propagate_bounds_INT_GE(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_int_ge(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_int_lt(op.getarg(0), op.getarg(1))
 
     def make_uint_lt(self, box1, box2):
@@ -702,70 +628,84 @@ class OptIntBounds(Optimization):
     def DISABLED_propagate_bounds_UINT_LT(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_uint_lt(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_uint_ge(op.getarg(0), op.getarg(1))
 
     def DISABLED_propagate_bounds_UINT_GT(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_uint_gt(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_uint_le(op.getarg(0), op.getarg(1))
 
     def DISABLED_propagate_bounds_UINT_LE(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_uint_le(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_uint_gt(op.getarg(0), op.getarg(1))
 
     def DISABLED_propagate_bounds_UINT_GE(self, op):
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == 1:
+            if r.get_constant_int() == 1:
                 self.make_uint_ge(op.getarg(0), op.getarg(1))
             else:
-                assert r.getint() == 0
+                assert r.get_constant_int() == 0
                 self.make_uint_lt(op.getarg(0), op.getarg(1))
 
     def propagate_bounds_INT_EQ(self, op):
         r = self.getintbound(op)
         if r.equal(1):
-            b1 = self.getintbound(op.getarg(0))
-            b2 = self.getintbound(op.getarg(1))
-            if b1.intersect(b2):
-                self.propagate_bounds_backward(op.getarg(0))
-            if b2.intersect(b1):
-                self.propagate_bounds_backward(op.getarg(1))
+            self.make_eq(op.getarg(0), op.getarg(1))
+        elif r.equal(0):
+            self.make_ne(op.getarg(0), op.getarg(1))
 
     def propagate_bounds_INT_NE(self, op):
         r = self.getintbound(op)
         if r.equal(0):
-            b1 = self.getintbound(op.getarg(0))
-            b2 = self.getintbound(op.getarg(1))
-            if b1.intersect(b2):
-                self.propagate_bounds_backward(op.getarg(0))
-            if b2.intersect(b1):
-                self.propagate_bounds_backward(op.getarg(1))
+            self.make_eq(op.getarg(0), op.getarg(1))
+        elif r.equal(0):
+            self.make_ne(op.getarg(0), op.getarg(1))
+
+    def make_eq(self, arg0, arg1):
+        b0 = self.getintbound(arg0)
+        b1 = self.getintbound(arg1)
+        if b0.intersect(b1):
+            self.propagate_bounds_backward(arg0)
+        if b1.intersect(b0):
+            self.propagate_bounds_backward(arg1)
+
+    def make_ne(self, arg0, arg1):
+        b0 = self.getintbound(arg0)
+        b1 = self.getintbound(arg1)
+        if b1.is_constant():
+            v1 = b1.get_constant_int()
+            if b0.make_ne_const(v1):
+                self.propagate_bounds_backward(arg0)
+        elif b0.is_constant():
+            v0 = b0.get_constant_int()
+            if b1.make_ne_const(v0):
+                self.propagate_bounds_backward(arg1)
 
     def _propagate_int_is_true_or_zero(self, op, valnonzero, valzero):
         if self.is_raw_ptr(op.getarg(0)):
             return
         r = self.getintbound(op)
         if r.is_constant():
-            if r.getint() == valnonzero:
+            if r.get_constant_int() == valnonzero:
                 b1 = self.getintbound(op.getarg(0))
                 if b1.known_nonnegative():
                     b1.make_gt_const(0)
                     self.propagate_bounds_backward(op.getarg(0))
-            elif r.getint() == valzero:
+            elif r.get_constant_int() == valzero:
                 self.make_constant_int(op.getarg(0), 0)
                 self.propagate_bounds_backward(op.getarg(0))
 
@@ -795,13 +735,16 @@ class OptIntBounds(Optimization):
         b = r.add_bound(b2)
         if b1.intersect(b):
             self.propagate_bounds_backward(op.getarg(0))
-        b = r.sub_bound(b1).mul(-1)
+        b = r.sub_bound(b1).neg_bound()
         if b2.intersect(b):
             self.propagate_bounds_backward(op.getarg(1))
 
     def propagate_bounds_INT_MUL(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
+        if op.opnum != rop.INT_MUL_OVF and not b1.mul_bound_cannot_overflow(b2):
+            # we can only do divide if the operation didn't overflow
+            return
         r = self.getintbound(op)
         b = r.py_div_bound(b2)
         if b1.intersect(b):
@@ -813,6 +756,8 @@ class OptIntBounds(Optimization):
     def propagate_bounds_INT_LSHIFT(self, op):
         b1 = self.getintbound(op.getarg(0))
         b2 = self.getintbound(op.getarg(1))
+        if not b1.lshift_bound_cannot_overflow(b2):
+            return
         r = self.getintbound(op)
         b = r.rshift_bound(b2)
         if b1.intersect(b):
@@ -826,4 +771,5 @@ class OptIntBounds(Optimization):
 dispatch_opt = make_dispatcher_method(OptIntBounds, 'optimize_',
                                       default=OptIntBounds.emit)
 dispatch_bounds_ops = make_dispatcher_method(OptIntBounds, 'propagate_bounds_')
-dispatch_postprocess = make_dispatcher_method(OptIntBounds, 'postprocess_')
+OptIntBounds.propagate_postprocess = make_dispatcher_method(OptIntBounds, 'postprocess_')
+OptIntBounds.have_postprocess_op = have_dispatcher_method(OptIntBounds, 'postprocess_')
