@@ -90,16 +90,11 @@ class UnrollOptimizer(Optimizer):
             if preamble_info.is_nonnull():
                 self.make_nonnull(op)
         elif isinstance(preamble_info, intutils.IntBound):
-            fix_lo = preamble_info.has_lower and preamble_info.lower >= MININT/2
-            fix_up = preamble_info.has_upper and preamble_info.upper <= MAXINT/2
-            if fix_lo or fix_up:
-                intbound = self.getintbound(op)
-                if fix_lo:
-                    intbound.has_lower = True
-                    intbound.lower = preamble_info.lower
-                if fix_up:
-                    intbound.has_upper = True
-                    intbound.upper = preamble_info.upper
+            loop_info = preamble_info.widen()
+            fix_lo = preamble_info.lower >= MININT/2
+            fix_up = preamble_info.upper <= MAXINT/2
+            intbound = self.getintbound(op)
+            intbound.intersect(loop_info)
         elif isinstance(preamble_info, info.FloatConstInfo):
             op.set_forwarded(preamble_info._const)
 
