@@ -357,29 +357,29 @@ class TestHeapCache(object):
         h.setfield(box1, box2, descr1)
         h.setarrayitem(box1, index1, box2, descr1)
         h.setarrayitem(box1, index2, box4, descr1)
-        h.invalidate_caches(rop.INT_ADD, None, [])
-        h.invalidate_caches(rop.INT_ADD_OVF, None, [])
-        h.invalidate_caches(rop.SETFIELD_RAW, None, [])
-        h.invalidate_caches(rop.SETARRAYITEM_RAW, None, [])
+        h.invalidate_caches(rop.INT_ADD, None)
+        h.invalidate_caches(rop.INT_ADD_OVF, None)
+        h.invalidate_caches(rop.SETFIELD_RAW, None)
+        h.invalidate_caches(rop.SETARRAYITEM_RAW, None)
         assert h.getfield(box1, descr1) is box2
         assert h.getarrayitem(box1, index1, descr1) is box2
         assert h.getarrayitem(box1, index2, descr1) is box4
 
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_ELIDABLE_CANNOT_RAISE), [])
         assert h.getfield(box1, descr1) is box2
         assert h.getarrayitem(box1, index1, descr1) is box2
         assert h.getarrayitem(box1, index2, descr1) is box4
 
-        h.invalidate_caches(rop.GUARD_TRUE, None, [])
+        h.invalidate_caches(rop.GUARD_TRUE, None)
         assert h.getfield(box1, descr1) is box2
         assert h.getarrayitem(box1, index1, descr1) is box2
         assert h.getarrayitem(box1, index2, descr1) is box4
 
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_LOOPINVARIANT_N, FakeCallDescr(FakeEffectinfo.EF_LOOPINVARIANT), [])
 
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
         assert h.getfield(box1, descr1) is None
         assert h.getarrayitem(box1, index1, descr1) is None
@@ -514,13 +514,13 @@ class TestHeapCache(object):
         h.setarrayitem(box1, index1, box2, descr1)
         h.new_array(box2, lengthbox1)
         # Just need the destination box for this call
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box5, box2, index1, index1, index1]
         )
         assert h.getarrayitem(box1, index1, descr1) is box2
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box5, box3, index1, index1, index1]
@@ -529,7 +529,7 @@ class TestHeapCache(object):
 
         h.setarrayitem(box4, index1, box2, descr1)
         assert h.getarrayitem(box4, index1, descr1) is box2
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box3, box5, index1, index1, index2]
@@ -545,7 +545,7 @@ class TestHeapCache(object):
         h.setarrayitem(box1, index1, box2, descr2)
         assert h.getarrayitem(box1, index1, descr2) is box2
         h.new_array(box2, lengthbox2)
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box3, box2, index1, index1, index2]
@@ -559,7 +559,7 @@ class TestHeapCache(object):
         box3 = RefFrontendOp(3)
         h.setarrayitem(box1, index1, box2, descr2)
         assert h.getarrayitem(box1, index1, descr2) is box2
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box3, box2, index1, index1, InputArgInt()]
@@ -572,7 +572,7 @@ class TestHeapCache(object):
         box2 = RefFrontendOp(2)
         box3 = RefFrontendOp(3)
         h.setarrayitem(box1, index1, box2, descr1)
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box1, box3, index1, index1, index2]
@@ -588,7 +588,7 @@ class TestHeapCache(object):
         lengthbox1 = IntFrontendOp(11)
         h.new_array(box1, lengthbox1)
         h.setarrayitem(box3, index1, box4, descr1)
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, index2]
@@ -600,16 +600,18 @@ class TestHeapCache(object):
         box2 = RefFrontendOp(2)
         lengthbox1 = IntFrontendOp(11)
         lengthbox2 = IntFrontendOp(12)
-        h.new_array(box1, lengthbox1)
-        h.new_array(box2, lengthbox2)
-        h.invalidate_caches(
+        h.new_array(box1, index2)
+        h.new_array(box2, index2)
+        assert h.is_unescaped(box1)
+        assert h.is_unescaped(box2)
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, index2]
         )
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N,
             arraycopydescr1,
             [None, box2, box1, index1, index1, InputArgInt()]
@@ -624,9 +626,9 @@ class TestHeapCache(object):
         assert not h.is_unescaped(box1)
         h.new(box2)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box2, box1])
+        h.invalidate_caches(rop.SETFIELD_GC, None, box2, box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
+        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
         assert not h.is_unescaped(box2)
 
     def test_unescaped_testing(self):
@@ -639,14 +641,14 @@ class TestHeapCache(object):
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         # Putting a virtual inside of another virtual doesn't escape it.
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
+        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
         assert h.is_unescaped(box2)
         # Reading a field from a virtual doesn't escape it.
-        h.invalidate_caches(rop.GETFIELD_GC_I, None, [box1])
+        h.invalidate_caches(rop.GETFIELD_GC_I, None, box1)
         assert h.is_unescaped(box1)
         # Escaping a virtual transitively escapes anything inside of it.
         assert not h.is_unescaped(box3)
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box3, box1])
+        h.invalidate_caches(rop.SETFIELD_GC, None, box3, box1)
         assert not h.is_unescaped(box1)
         assert not h.is_unescaped(box2)
 
@@ -658,10 +660,10 @@ class TestHeapCache(object):
         h.new(box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.INSTANCE_PTR_EQ, None, [box1, box2])
+        h.invalidate_caches(rop.INSTANCE_PTR_EQ, None, box1, box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.INSTANCE_PTR_NE, None, [box1, box2])
+        h.invalidate_caches(rop.INSTANCE_PTR_NE, None, box1, box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
 
@@ -672,9 +674,9 @@ class TestHeapCache(object):
         box3 = RefFrontendOp(3)
         h.new(box1)
         h.new(box2)
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box2, box1])
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box3, box1]) # does not crash
+        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
+        h.invalidate_caches(rop.SETFIELD_GC, None, box2, box1)
+        h.invalidate_caches(rop.SETFIELD_GC, None, box3, box1) # does not crash
 
     def test_unescaped_array(self):
         h = HeapCache()
@@ -682,22 +684,22 @@ class TestHeapCache(object):
         box2 = RefFrontendOp(2)
         lengthbox1 = IntFrontendOp(11)
         lengthbox2 = IntFrontendOp(12)
-        h.new_array(box1, lengthbox1)
+        h.new_array(box1, index2)
         assert h.is_unescaped(box1)
-        h.invalidate_caches(rop.SETARRAYITEM_GC, None, [box1, index1, box2])
+        h.invalidate_caches(rop.SETARRAYITEM_GC, None, box1, index1, box2)
         assert h.is_unescaped(box1)
-        h.invalidate_caches(rop.SETARRAYITEM_GC, None, [box2, index1, box1])
+        h.invalidate_caches(rop.SETARRAYITEM_GC, None, box2, index1, box1)
         assert not h.is_unescaped(box1)
 
         h = HeapCache()
-        h.new_array(box1, lengthbox1)
+        h.new_array(box1, index2)
         h.new(box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(rop.SETARRAYITEM_GC, None, [box1, lengthbox2, box2])
+        h.invalidate_caches(rop.SETARRAYITEM_GC, None, box1, lengthbox2, box2)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [box1]
         )
         assert not h.is_unescaped(box1)
@@ -710,7 +712,7 @@ class TestHeapCache(object):
         h.new(box1)
         assert h.is_unescaped(box1)
         h.setfield(box1, box2, descr1)
-        h.invalidate_caches(rop.CALL_N,
+        h.invalidate_caches_varargs(rop.CALL_N,
             FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE),
             []
         )
@@ -721,10 +723,10 @@ class TestHeapCache(object):
         box1 = RefFrontendOp(1)
         box3 = RefFrontendOp(3)
         lengthbox1 = IntFrontendOp(11)
-        h.new_array(box1, lengthbox1)
+        h.new_array(box1, index2)
         assert h.is_unescaped(box1)
         h.setarrayitem(box1, index1, box3, descr1)
-        h.invalidate_caches(rop.CALL_N,
+        h.invalidate_caches_varargs(rop.CALL_N,
             FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE),
             []
         )
@@ -738,12 +740,12 @@ class TestHeapCache(object):
         h.new(box2)
         h.setfield(box1, box2, descr1)
         assert h.getfield(box1, descr1) is box2
-        h.invalidate_caches(rop.STRSETITEM, None, [])
-        h.invalidate_caches(rop.UNICODESETITEM, None, [])
-        h.invalidate_caches(rop.SETFIELD_RAW, None, [])
-        h.invalidate_caches(rop.SETARRAYITEM_RAW, None, [])
-        h.invalidate_caches(rop.SETINTERIORFIELD_RAW, None, [])
-        h.invalidate_caches(rop.RAW_STORE, None, [])
+        h.invalidate_caches(rop.STRSETITEM, None)
+        h.invalidate_caches(rop.UNICODESETITEM, None)
+        h.invalidate_caches(rop.SETFIELD_RAW, None)
+        h.invalidate_caches(rop.SETARRAYITEM_RAW, None)
+        h.invalidate_caches(rop.SETINTERIORFIELD_RAW, None)
+        h.invalidate_caches(rop.RAW_STORE, None)
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is box2
@@ -761,9 +763,9 @@ class TestHeapCache(object):
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
+        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
         assert h.getfield(box1, descr1) is box2
-        h.invalidate_caches(rop.CALL_MAY_FORCE_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
+        h.invalidate_caches_varargs(rop.CALL_MAY_FORCE_N, FakeCallDescr(FakeEffectinfo.EF_RANDOM_EFFECTS), [])
         assert not h.is_unescaped(box1)
         assert not h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is None
@@ -775,7 +777,7 @@ class TestHeapCache(object):
         h.new(box1)
         h.new(box2)
         h.setfield(box1, box2, descr1)
-        h.invalidate_caches(rop.SETFIELD_GC, None, [box1, box2])
+        h.invalidate_caches(rop.SETFIELD_GC, None, box1, box2)
         assert h.getfield(box1, descr1) is box2
         descr = BasicFailDescr()
         class XTra:
@@ -788,7 +790,7 @@ class TestHeapCache(object):
             EF_ELIDABLE_OR_MEMORYERROR = 3
             EF_ELIDABLE_CAN_RAISE = 4
         descr.get_extra_info = XTra
-        h.invalidate_caches(rop.CALL_N, descr, [])
+        h.invalidate_caches_varargs(rop.CALL_N, descr, [])
         assert h.is_unescaped(box1)
         assert h.is_unescaped(box2)
         assert h.getfield(box1, descr1) is box2
@@ -832,6 +834,17 @@ class TestHeapCache(object):
         assert not h.is_unescaped(box1)
         assert h.is_likely_virtual(box1)
 
+    def test_is_likely_virtual_array(self):
+        h = HeapCache()
+        box1 = RefFrontendOp(1)
+        h.new_array(box1, index2)
+        assert h.is_likely_virtual(box1)
+        box2 = RefFrontendOp(2)
+        lengthbox = IntFrontendOp(11)
+        # arrays are only virtual if the length is constant
+        h.new_array(box2, lengthbox)
+        assert not h.is_likely_virtual(box2)
+
     def test_quasiimmut_seen(self):
         h = HeapCache()
         box1 = RefFrontendOp(1)
@@ -870,7 +883,7 @@ class TestHeapCache(object):
         assert not h.is_quasi_immut_known(descr1, box2)
 
         # a call invalidates everything
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE), [])
         assert not h.is_quasi_immut_known(descr2, box3)
         assert not h.is_quasi_immut_known(descr2, box4)
@@ -906,7 +919,29 @@ class TestHeapCache(object):
         assert not h.is_quasi_immut_known(descr1, box2)
 
         # a call invalidates everything
-        h.invalidate_caches(
+        h.invalidate_caches_varargs(
             rop.CALL_N, FakeCallDescr(FakeEffectinfo.EF_CAN_RAISE), [])
         assert not h.is_quasi_immut_known(descr2, box3)
         assert not h.is_quasi_immut_known(descr2, box4)
+
+    def test_heapcache_on_const(self):
+        h = HeapCache()
+        # two different boxes of the same value
+        box1 = ConstPtr(ConstPtr.value)
+        box2 = ConstPtr(ConstPtr.value)
+        box3 = RefFrontendOp(3)
+        box4 = RefFrontendOp(4)
+
+        # the cache unifies a bit
+        assert h.getfield(box1, descr1) is None
+        assert h.getfield(box2, descr1) is None
+        h.getfield_now_known(box1, descr1, box3)
+        assert h.heap_cache[descr1].last_const_box is box1
+        assert h.heap_cache[descr1]._unique_const_heuristic(box2) is box1
+        assert h.getfield(box1, descr1) is box3
+        assert h.getfield(box2, descr1) is box3
+
+        h.reset()
+        h.setfield(box2, box4, descr1)
+        assert h.getfield(box1, descr1) is box4
+        assert h.getfield(box2, descr1) is box4
