@@ -1,12 +1,13 @@
 import py
 
 from rpython.jit.codewriter import heaptracker
-from rpython.jit.codewriter.policy import StopAtXPolicy
 from rpython.jit.metainterp.optimizeopt.test.test_util import LLtypeMixin
 from rpython.jit.metainterp.test.support import LLJitMixin
 from rpython.jit.metainterp.warmspot import get_translator, get_stats
 from rpython.jit.metainterp.resoperation import rop
-from rpython.rlib.jit import JitDriver, hint, dont_look_inside, promote, virtual_ref
+from rpython.jit.metainterp.jitprof import Profiler
+from rpython.rlib.jit import JitDriver, hint, dont_look_inside, promote, \
+        virtual_ref, Counters
 from rpython.rlib.rarithmetic import intmask
 from rpython.rtyper.annlowlevel import hlstr
 from rpython.rtyper.llannotation import lltype_to_annotation
@@ -542,6 +543,41 @@ class ImplicitVirtualizableTests(object):
         self.check_resops(getfield_gc_r=1, getarrayitem_gc_i=4,
                           getfield_gc_i=1)
 
+    @py.test.mark.xfail
+    def test_virtualizable_with_array_huge(self):
+        myjitdriver = JitDriver(greens = [], reds = ['n', 'x', 'frame'],
+                                virtualizables = ['frame'])
+
+        class Frame(object):
+            _virtualizable_ = ['l[*]', 's']
+
+            def __init__(self, l, s):
+                self.l = l
+                self.s = s
+
+        def f(n, a):
+            frame = Frame([a] * 1024, 0) # make a huge frame
+            x = 0
+            while n > 0:
+                myjitdriver.can_enter_jit(frame=frame, n=n, x=x)
+                myjitdriver.jit_merge_point(frame=frame, n=n, x=x)
+                frame.s = promote(frame.s)
+                n -= 1
+                s = frame.s
+                assert s >= 0
+                x += frame.l[s]
+                frame.s += 1
+                s = frame.s
+                assert s >= 0
+                x += frame.l[s]
+                x += len(frame.l)
+                frame.s -= 1
+            return x
+
+        res = self.meta_interp(f, [50, 1], listops=True)
+        # should stop giving up to compile, eventually
+        assert get_stats().aborted_count < 6
+
     def test_subclass_of_virtualizable(self):
         myjitdriver = JitDriver(greens = [], reds = ['frame'],
                                 virtualizables = ['frame'])
@@ -584,6 +620,7 @@ class ImplicitVirtualizableTests(object):
         class Frame(BaseFrame):
             pass
 
+        @dont_look_inside
         def g(frame):
             return frame.x[1] == 1
 
@@ -598,7 +635,7 @@ class ImplicitVirtualizableTests(object):
                 n -= 1
             return z
 
-        res = self.meta_interp(f, [10], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [10])
         assert res == f(10)
 
 
@@ -613,6 +650,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             result = somewhere_else.top_frame.y     # external read
             debug_print(lltype.Void, '-+-+-+-+- external read:', result)
@@ -630,7 +668,7 @@ class ImplicitVirtualizableTests(object):
                 frame.y += 1
             return frame.x
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123])
         assert res == f(123)
         self.check_aborted_count(2)
         self.check_jitcell_token_count(0)
@@ -650,6 +688,7 @@ class ImplicitVirtualizableTests(object):
             def __init__(self, value):
                 self.value = value
 
+        @dont_look_inside
         def g():
             result = somewhere_else.top_frame.y     # external read
             debug_print(lltype.Void, '-+-+-+-+- external read:', result)
@@ -670,7 +709,7 @@ class ImplicitVirtualizableTests(object):
                 frame.y += 1
             return frame.x
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123])
         assert res == f(123)
         self.check_aborted_count(2)
         self.check_jitcell_token_count(0)
@@ -686,6 +725,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             result = somewhere_else.top_frame.y + 1
             debug_print(lltype.Void, '-+-+-+-+- external write:', result)
@@ -703,7 +743,7 @@ class ImplicitVirtualizableTests(object):
                 frame.x -= frame.y
             return frame.y
 
-        res = self.meta_interp(f, [240], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [240])
         assert res == f(240)
         self.check_aborted_count(3)
         self.check_jitcell_token_count(0)
@@ -719,6 +759,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             somewhere_else.counter += 1
             if somewhere_else.counter == 70:
@@ -742,8 +783,11 @@ class ImplicitVirtualizableTests(object):
                 frame.y += 1
             return frame.x
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123], ProfilerClass=Profiler)
         assert res == f(123)
+        stats = get_stats()
+        assert stats.metainterp_sd.profiler.counters[
+            Counters.FORCE_VIRTUALIZABLES] == 1
 
     def test_external_read_sometimes_with_virtuals(self):
         jitdriver = JitDriver(greens = [], reds = ['frame'],
@@ -759,6 +803,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             somewhere_else.counter += 1
             if somewhere_else.counter == 70:
@@ -787,7 +832,7 @@ class ImplicitVirtualizableTests(object):
                 frame.x -= 1
             return frame.x
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123])
         assert res == f(123)
 
     def test_external_read_sometimes_changing_virtuals(self):
@@ -804,6 +849,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             somewhere_else.counter += 1
             if somewhere_else.counter == 70:
@@ -837,7 +883,7 @@ class ImplicitVirtualizableTests(object):
                 frame.x -= 1
             return frame.x
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123])
         assert res == f(123)
 
     def test_external_read_sometimes_with_exception(self):
@@ -854,6 +900,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             somewhere_else.counter += 1
             if somewhere_else.counter == 70:
@@ -881,7 +928,7 @@ class ImplicitVirtualizableTests(object):
                 pass
             return frame.x
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123])
         assert res == f(123)
 
     def test_external_read_sometimes_dont_compile_guard(self):
@@ -895,6 +942,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             somewhere_else.counter += 1
             if somewhere_else.counter == 70:
@@ -918,7 +966,7 @@ class ImplicitVirtualizableTests(object):
                 frame.y += 1
             return frame.x
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g), repeat=7)
+        res = self.meta_interp(f, [123], repeat=7)
         assert res == f(123)
 
     def test_external_read_sometimes_recursive(self):
@@ -932,6 +980,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g(rec):
             somewhere_else.counter += 1
             if somewhere_else.counter == 70:
@@ -970,7 +1019,7 @@ class ImplicitVirtualizableTests(object):
             somewhere_else.top_frame = None
             return f(n, True)
 
-        res = self.meta_interp(main, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(main, [123])
         assert res == main(123)
 
     def test_external_write_sometimes(self):
@@ -984,6 +1033,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             somewhere_else.counter += 1
             if somewhere_else.counter == 70:
@@ -1007,7 +1057,7 @@ class ImplicitVirtualizableTests(object):
                 frame.y += 1
             return frame.y
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123])
         assert res == f(123)
 
     def test_bridge_forces(self):
@@ -1021,6 +1071,7 @@ class ImplicitVirtualizableTests(object):
             pass
         somewhere_else = SomewhereElse()
 
+        @dont_look_inside
         def g():
             n = somewhere_else.top_frame.y + 700
             debug_print(lltype.Void, '-+-+-+-+- external write:', n)
@@ -1041,7 +1092,7 @@ class ImplicitVirtualizableTests(object):
                 frame.y += 1
             return frame.y
 
-        res = self.meta_interp(f, [123], policy=StopAtXPolicy(g))
+        res = self.meta_interp(f, [123])
         assert res == f(123)
 
     def test_promote_index_in_virtualizable_list(self):
@@ -1115,7 +1166,7 @@ class ImplicitVirtualizableTests(object):
         init_graph = t._graphof(Frame.__init__.im_func)
 
         def direct_calls(graph):
-            return [op.args[0].value._obj._callable.func_name
+            return [op.args[0].value._obj._callable.__name__
                     for block, op in graph.iterblockops()
                         if op.opname == 'direct_call']
 
@@ -1191,7 +1242,7 @@ class ImplicitVirtualizableTests(object):
         res = self.meta_interp(f, [10])
         assert res == 55
         self.check_resops(new_with_vtable=0, ptr_eq=1)
-        self.check_history(ptr_eq=2)
+        self.check_history(ptr_eq=1)
 
     def test_virtual_child_frame_with_arrays(self):
         myjitdriver = JitDriver(greens = [], reds = ['frame'],
