@@ -1,7 +1,7 @@
 from rpython.rtyper.lltypesystem import rffi
 from rpython.rtyper.lltypesystem import lltype
 from rpython.rtyper.tool import rffi_platform as platform
-from rpython.rtyper.lltypesystem.rffi import CCHARP
+from rpython.rtyper.lltypesystem.rffi import CCHARP, CONST_CCHARP
 from rpython.rlib import jit
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
 from rpython.translator.platform import platform as target_platform
@@ -1164,7 +1164,7 @@ def external_c(name, args, result, **kwargs):
 
 if _POSIX:
     dup = external('dup', [socketfd_type], socketfd_type, save_err=SAVE_ERR)
-    gai_strerror = external('gai_strerror', [rffi.INT], CCHARP)
+    gai_strerror = external('gai_strerror', [rffi.INT], CONST_CCHARP)
 
 #h_errno = c_int.in_dll(socketdll, 'h_errno')
 #
@@ -1218,7 +1218,7 @@ inet_pton = external('inet_pton', [rffi.INT, rffi.CCHARP,
                      save_err=SAVE_ERR)
 
 inet_ntop = external('inet_ntop', [rffi.INT, rffi.VOIDP, CCHARP,
-                                   socklen_t], CCHARP,
+                                   socklen_t], CONST_CCHARP,
                      save_err=SAVE_ERR)
 
 inet_addr = external('inet_addr', [rffi.CCHARP], rffi.UINT)
@@ -1297,7 +1297,7 @@ getprotobyname = external('getprotobyname', [rffi.CCHARP], lltype.Ptr(cConfig.pr
 
 if _POSIX:
     fcntl = external('fcntl', [socketfd_type, rffi.INT, rffi.INT], rffi.INT,
-                     save_err=SAVE_ERR)
+                     save_err=SAVE_ERR, natural_arity=2)
     socketpair_t = rffi.CArray(socketfd_type)
     socketpair = external('socketpair', [rffi.INT, rffi.INT, rffi.INT,
                           lltype.Ptr(socketpair_t)], rffi.INT,
@@ -1432,7 +1432,7 @@ else:
 
     socket_strerror_str = os.strerror
     def gai_strerror_str(errno):
-        return rffi.charp2str(gai_strerror(errno))
+        return rffi.constcharp2str(gai_strerror(errno))
 
     def socket_strerror_unicode(errno):
         return socket_strerror_str(errno).decode('latin-1')
