@@ -344,7 +344,7 @@ def build_new_ctypes_type(T, delayed_builders):
     if isinstance(T, lltype.Ptr):
         if isinstance(T.TO, lltype.FuncType):
             functype = ctypes.CFUNCTYPE
-            if sys.platform == 'win32':
+            if sys.platform == 'win32' and not _64BIT:
                 from rpython.rlib.clibffi import FFI_STDCALL, FFI_DEFAULT_ABI
                 if getattr(T.TO, 'ABI', FFI_DEFAULT_ABI) == FFI_STDCALL:
                     # for win32 system call
@@ -374,9 +374,9 @@ def build_new_ctypes_type(T, delayed_builders):
     elif isinstance(T, lltype.OpaqueType):
         if T is lltype.RuntimeTypeInfo:
             return ctypes.c_char * 2
-        if T.hints.get('external', None) != 'C':
+        if T._hints.get('external', None) != 'C':
             raise TypeError("%s is not external" % T)
-        return ctypes.c_char * T.hints['getsize']()
+        return ctypes.c_char * T._hints['getsize']()
     else:
         _setup_ctypes_cache()
         if T in _ctypes_cache:
@@ -894,9 +894,11 @@ def lltype2ctypes(llobj, normalize=True):
                         #    import pdb; pdb.post_mortem(sys.exc_traceback)
                         global _callback_exc_info
                         _callback_exc_info = sys.exc_info()
-                        if hasattr(getattr(container, '_callable', None),
-                                   '_llhelper_can_raise_'):
-                            llres = T.TO.RESULT._defl()
+                        _callable = getattr(container, '_callable', None)
+                        if hasattr(_callable, '_llhelper_error_value_'):
+                            # see rlib.objectmodel.llhelper_error_value
+                            llres = _callable._llhelper_error_value_
+                            assert lltype.typeOf(llres) == T.TO.RESULT
                             return ctypes_return_value(llres)
                         else:
                             raise
@@ -932,7 +934,7 @@ def lltype2ctypes(llobj, normalize=True):
                     convert_array(container)
                 elif isinstance(T.TO, lltype.OpaqueType):
                     if T.TO != lltype.RuntimeTypeInfo:
-                        cbuf = ctypes.create_string_buffer(T.TO.hints['getsize']())
+                        cbuf = ctypes.create_string_buffer(T.TO._hints['getsize']())
                     else:
                         cbuf = ctypes.create_string_buffer("\x00")
                     cbuf = ctypes.cast(cbuf, ctypes.c_void_p)
@@ -1169,6 +1171,8 @@ if ctypes:
         rtld_default_lib = ctypes.CDLL("ld-elf.so.1", handle=RTLD_DEFAULT, **load_library_kwargs)
     # XXX is this always correct???
     standard_c_lib = ctypes.CDLL(libc_name, **load_library_kwargs)
+else:
+    libc_name = 'no ctypes'
 
 # ____________________________________________
 
@@ -1181,7 +1185,7 @@ if sys.platform == 'darwin':
         expr = r'[^\(\)\s]*lib%s\.[^\(\)\s]*' % re.escape(name)
         fdout, ccout = tempfile.mkstemp()
         os.close(fdout)
-        cmd = 'if type gcc >/dev/null 2>&1; then CC=gcc; else CC=cc; fi;' \
+        cmd = 'if type gcc >/dev/null 2>&1; then : ${CC:=gcc}; else : ${CC:=cc}; fi;' \
               '$CC -Wl,-t -o ' + ccout + ' 2>&1 -l' + name
         try:
             f = os.popen(cmd)
@@ -1200,7 +1204,7 @@ if sys.platform == 'darwin':
 else:
     _findLib_gcc_fallback = lambda name: None
 
-def get_ctypes_callable(funcptr, calling_conv):
+def get_ctypes_callable(funcptr, calling_conv, natural_arity):
     if not ctypes:
         raise ImportError("ctypes is needed to use ll2ctypes")
 
@@ -1293,10 +1297,16 @@ def get_ctypes_callable(funcptr, calling_conv):
 
     # get_ctypes_type() can raise NotImplementedError too
     from rpython.rtyper.lltypesystem import rffi
-    cfunc.argtypes = [get_ctypes_type(T) if T is not rffi.VOIDP
+    argtypes = [get_ctypes_type(T) if T is not rffi.VOIDP
                                          else ctypes.c_void_p
                       for T in FUNCTYPE.ARGS
                       if not T is lltype.Void]
+    if natural_arity != -1:
+        cfunc.argtypes = argtypes[:natural_arity]
+        cfunc.extraargs = argtypes[natural_arity:]
+    else:
+        cfunc.argtypes = argtypes
+        cfunc.extraargs = []
     if FUNCTYPE.RESULT is lltype.Void:
         cfunc.restype = None
     else:
@@ -1306,26 +1316,29 @@ def get_ctypes_callable(funcptr, calling_conv):
 class LL2CtypesCallable(object):
     # a special '_callable' object that invokes ctypes
 
-    def __init__(self, FUNCTYPE, calling_conv):
+    def __init__(self, FUNCTYPE, calling_conv, natural_arity):
         self.FUNCTYPE = FUNCTYPE
         self.calling_conv = calling_conv
         self.trampoline = None
+        self.natural_arity = natural_arity
         #self.funcptr = ...  set later
 
     def __call__(self, *argvalues):
         with rlock:
             if self.trampoline is None:
                 # lazily build the corresponding ctypes function object
-                cfunc = get_ctypes_callable(self.funcptr, self.calling_conv)
-                self.trampoline = get_ctypes_trampoline(self.FUNCTYPE, cfunc)
+                cfunc = get_ctypes_callable(self.funcptr, self.calling_conv,
+                                            self.natural_arity)
+                self.trampoline = get_ctypes_trampoline(self.FUNCTYPE, cfunc, self.natural_arity)
         # perform the call
         return self.trampoline(*argvalues)
 
     def get_real_address(self):
-        cfunc = get_ctypes_callable(self.funcptr, self.calling_conv)
+        cfunc = get_ctypes_callable(self.funcptr, self.calling_conv,
+                                    self.natural_arity)
         return ctypes.cast(cfunc, ctypes.c_void_p).value
 
-def get_ctypes_trampoline(FUNCTYPE, cfunc):
+def get_ctypes_trampoline(FUNCTYPE, cfunc, natural_arity=-1):
     RESULT = FUNCTYPE.RESULT
     container_arguments = []
     for i in range(len(FUNCTYPE.ARGS)):
@@ -1345,6 +1358,8 @@ def get_ctypes_trampoline(FUNCTYPE, cfunc):
                 cvalue = lltype2ctypes(argvalues[i])
                 if i in container_arguments:
                     cvalue = cvalue.contents
+                if natural_arity > 0 and i >= natural_arity:
+                    cvalue = cfunc.extraargs[i - natural_arity](cvalue)
                 cargs.append(cvalue)
         _callback_exc_info = None
         _restore_c_errno()
@@ -1352,6 +1367,10 @@ def get_ctypes_trampoline(FUNCTYPE, cfunc):
         _save_c_errno()
         if _callback_exc_info:
             etype, evalue, etb = _callback_exc_info
+            # cres is the actual C result returned by the function. Stick it
+            # into the exception so that we can check it inside tests (see
+            # e.g. test_llhelper_error_value)
+            evalue._ll2ctypes_c_result = cres
             _callback_exc_info = None
             raise etype, evalue, etb
         return ctypes2lltype(RESULT, cres)
