@@ -1,15 +1,17 @@
 from rpython.annotator import model as annmodel
 from rpython.flowspace.model import Constant
 from rpython.rlib import rgc, jit, types
-from rpython.rlib.debug import ll_assert
-from rpython.rlib.objectmodel import malloc_zero_filled
+from rpython.rtyper.debug import ll_assert
+from rpython.rlib.objectmodel import malloc_zero_filled, enforceargs, specialize
 from rpython.rlib.signature import signature
 from rpython.rlib.rarithmetic import ovfcheck, widen, r_uint, intmask
+from rpython.rlib.rarithmetic import int_force_ge_zero
 from rpython.rtyper.annlowlevel import ADTInterface
 from rpython.rtyper.error import TyperError
 from rpython.rtyper.lltypesystem.lltype import typeOf, Ptr, Void, Signed, Bool
 from rpython.rtyper.lltypesystem.lltype import nullptr, Char, UniChar, Number
-from rpython.rtyper.rmodel import Repr, IteratorRepr, IntegerRepr
+from rpython.rtyper.rmodel import Repr, IteratorRepr
+from rpython.rtyper.rint import IntegerRepr
 from rpython.rtyper.rstr import AbstractStringRepr, AbstractCharRepr
 from rpython.tool.pairtype import pairtype, pair
 
@@ -93,27 +95,25 @@ class AbstractBaseListRepr(Repr):
         raise NotImplementedError
 
     def ll_str(self, l):
-        constant = self.rstr_ll.ll_constant
-        start    = self.rstr_ll.ll_build_start
-        push     = self.rstr_ll.ll_build_push
-        finish   = self.rstr_ll.ll_build_finish
+        from rpython.rtyper.annlowlevel import llstr, hlstr
+        from rpython.rlib.rstring import StringBuilder
 
         length = l.ll_length()
         if length == 0:
-            return constant("[]")
+            return llstr("[]")
 
-        buf = start(2 * length + 1)
-        push(buf, constant("["), 0)
+        buf = StringBuilder(3 * length + 2)
+        buf.append("[")
         item_repr = self.item_repr
         i = 0
         while i < length:
             if i > 0:
-                push(buf, constant(", "), 2 * i)
+                buf.append(", ")
             item = l.ll_getitem_fast(i)
-            push(buf, item_repr.ll_str(item), 2 * i + 1)
+            buf.append(hlstr(item_repr.ll_str(item)))
             i += 1
-        push(buf, constant("]"), 2 * length)
-        return finish(buf)
+        buf.append("]")
+        return llstr(buf.build())
 
     def rtype_bltn_list(self, hop):
         v_lst = hop.inputarg(self, 0)
@@ -266,13 +266,9 @@ class __extend__(pairtype(AbstractBaseListRepr, IntegerRepr)):
         v_res = hop.gendirectcall(llfn, c_func_marker, c_basegetitem, v_lst, v_index)
         return r_lst.recast(hop.llops, v_res)
 
-    rtype_getitem_key = rtype_getitem
-
     def rtype_getitem_idx((r_lst, r_int), hop):
         return pair(r_lst, r_int).rtype_getitem(hop, checkidx=True)
 
-    rtype_getitem_idx_key = rtype_getitem_idx
-
     def rtype_setitem((r_lst, r_int), hop):
         if hop.has_implicit_exception(IndexError):
             spec = dum_checkidx
@@ -292,6 +288,11 @@ class __extend__(pairtype(AbstractBaseListRepr, IntegerRepr)):
         v_lst, v_factor = hop.inputargs(r_lst, Signed)
         return hop.gendirectcall(ll_mul, cRESLIST, v_lst, v_factor)
 
+class __extend__(pairtype(IntegerRepr, AbstractBaseListRepr)):
+    def rtype_mul((r_int, r_lst), hop):
+        cRESLIST = hop.inputconst(Void, hop.r_result.LIST)
+        v_factor, v_lst = hop.inputargs(Signed, r_lst)
+        return hop.gendirectcall(ll_mul, cRESLIST, v_lst, v_factor)
 
 class __extend__(pairtype(AbstractListRepr, IntegerRepr)):
 
@@ -467,12 +468,9 @@ class AbstractListIteratorRepr(IteratorRepr):
 #  done with it.  So in the sequel we don't bother checking for overflow
 #  when we compute "ll_length() + 1".
 
-@jit.look_inside_iff(lambda LIST, count, item: jit.isconstant(count) and count < 15)
-@jit.oopspec("newlist(count, item)")
-def ll_alloc_and_set(LIST, count, item):
-    if count < 0:
-        count = 0
-    l = LIST.ll_newlist(count)
+
+def _ll_zero_or_null(item):
+    # Check if 'item' is zero/null, or not.
     T = typeOf(item)
     if T is Char or T is UniChar:
         check = ord(item)
@@ -480,13 +478,60 @@ def ll_alloc_and_set(LIST, count, item):
         check = widen(item)
     else:
         check = item
-    # as long as malloc is known to zero the allocated memory avoid zeroing
-    # twice
-    if (not malloc_zero_filled) or check:
-        i = 0
-        while i < count:
-            l.ll_setitem_fast(i, item)
-            i += 1
+    return not check
+
+@specialize.memo()
+def _null_of_type(T):
+    return T._defl()
+
+def ll_alloc_and_set(LIST, count, item):
+    count = int_force_ge_zero(count)
+    if jit.we_are_jitted():
+        return _ll_alloc_and_set_jit(LIST, count, item)
+    else:
+        return _ll_alloc_and_set_nojit(LIST, count, item)
+
+def _ll_alloc_and_set_nojit(LIST, count, item):
+    l = LIST.ll_newlist(count)
+    if malloc_zero_filled and _ll_zero_or_null(item):
+        return l
+    i = 0
+    while i < count:
+        l.ll_setitem_fast(i, item)
+        i += 1
+    return l
+
+def _ll_alloc_and_set_jit(LIST, count, item):
+    if _ll_zero_or_null(item):
+        # 'item' is zero/null.  Do the list allocation with the
+        # function _ll_alloc_and_clear(), which the JIT knows about.
+        return _ll_alloc_and_clear(LIST, count)
+    else:
+        # 'item' is not zero/null.  Do the list allocation with the
+        # function _ll_alloc_and_set_nonnull().  That function has
+        # a JIT marker to unroll it, but only if the 'count' is
+        # a not-too-large constant.
+        return _ll_alloc_and_set_nonnull(LIST, count, item)
+
+@jit.oopspec("newlist_clear(count)")
+def _ll_alloc_and_clear(LIST, count):
+    l = LIST.ll_newlist(count)
+    if malloc_zero_filled:
+        return l
+    zeroitem = _null_of_type(LIST.ITEM)
+    i = 0
+    while i < count:
+        l.ll_setitem_fast(i, zeroitem)
+        i += 1
+    return l
+
+@jit.look_inside_iff(lambda LIST, count, item: jit.isconstant(count) and count < 137)
+def _ll_alloc_and_set_nonnull(LIST, count, item):
+    l = LIST.ll_newlist(count)
+    i = 0
+    while i < count:
+        l.ll_setitem_fast(i, item)
+        i += 1
     return l
 
 
@@ -511,6 +556,11 @@ def ll_arraycopy(source, dest, source_start, dest_start, length):
     rgc.ll_arraycopy(source.ll_items(), dest.ll_items(),
                      source_start, dest_start, length)
 
+@signature(types.any(), types.int(), types.int(), types.int(), returns=types.none())
+def ll_arraymove(lst, source_start, dest_start, length):
+    # copy the slice [source_start:source_stop] to the slice [dest_start:..]
+    rgc.ll_arraymove(lst.ll_items(), source_start, dest_start, length)
+
 
 def ll_copy(RESLIST, l):
     length = l.ll_length()
@@ -544,13 +594,8 @@ def ll_append(l, newitem):
 def ll_prepend(l, newitem):
     length = l.ll_length()
     l._ll_resize_ge(length+1)           # see "a note about overflows" above
-    dst = length
-    while dst > 0:
-        src = dst - 1
-        l.ll_setitem_fast(dst, l.ll_getitem_fast(src))
-        dst = src
+    ll_arraymove(l, 0, 1, length)
     l.ll_setitem_fast(0, newitem)
-ll_prepend.oopspec = 'list.insert(l, 0, newitem)'
 
 def ll_concat(RESLIST, l1, l2):
     len1 = l1.ll_length()
@@ -570,13 +615,8 @@ def ll_insert_nonneg(l, index, newitem):
     ll_assert(0 <= index, "negative list insertion index")
     ll_assert(index <= length, "list insertion index out of bound")
     l._ll_resize_ge(length+1)           # see "a note about overflows" above
-    dst = length
-    while dst > index:
-        src = dst - 1
-        l.ll_setitem_fast(dst, l.ll_getitem_fast(src))
-        dst = src
+    ll_arraymove(l, index, index + 1, length - index)
     l.ll_setitem_fast(index, newitem)
-ll_insert_nonneg.oopspec = 'list.insert(l, index, newitem)'
 
 def ll_pop_nonneg(func, l, index):
     ll_assert(index >= 0, "unexpectedly negative list pop index")
@@ -609,14 +649,9 @@ def ll_pop_zero(func, l):
     if func is dum_checkidx and (length == 0):
         raise IndexError
     ll_assert(length > 0, "pop(0) from empty list")
-    newlength = length - 1
     res = l.ll_getitem_fast(0)
-    j = 0
-    j1 = j+1
-    while j < newlength:
-        l.ll_setitem_fast(j, l.ll_getitem_fast(j1))
-        j = j1
-        j1 += 1
+    newlength = length - 1
+    ll_arraymove(l, 1, 0, newlength)
     null = ll_null_item(l)
     if null is not None:
         l.ll_setitem_fast(newlength, null)
@@ -712,6 +747,7 @@ def ll_setitem(func, l, index, newitem):
     l.ll_setitem_fast(index, newitem)
 # no oopspec -- the function is inlined by the JIT
 
+@enforceargs(None, None, int)
 def ll_delitem_nonneg(func, l, index):
     ll_assert(index >= 0, "unexpectedly negative list delitem index")
     length = l.ll_length()
@@ -721,18 +757,12 @@ def ll_delitem_nonneg(func, l, index):
     else:
         ll_assert(index < length, "list delitem index out of bound")
     newlength = length - 1
-    j = index
-    j1 = j+1
-    while j < newlength:
-        l.ll_setitem_fast(j, l.ll_getitem_fast(j1))
-        j = j1
-        j1 += 1
-
+    ll_arraymove(l, index + 1, index, newlength - index)
     null = ll_null_item(l)
     if null is not None:
         l.ll_setitem_fast(newlength, null)
     l._ll_resize_le(newlength)
-ll_delitem_nonneg.oopspec = 'list.delitem(l, index)'
+# no oopspec -- the function is inlined by the JIT
 
 def ll_delitem(func, l, index):
     if func is dum_checkidx:
@@ -900,13 +930,8 @@ def ll_listdelslice_startstop(l, start, stop):
     ll_assert(stop >= start, "del l[x:y] with x > y")
     if stop > length:
         stop = length
+    ll_arraymove(l, stop, start, length - stop)
     newlength = length - (stop-start)
-    j = start
-    i = stop
-    while j < newlength:
-        l.ll_setitem_fast(j, l.ll_getitem_fast(i))
-        i += 1
-        j += 1
     null = ll_null_item(l)
     if null is not None:
         j = length - 1
@@ -917,20 +942,39 @@ def ll_listdelslice_startstop(l, start, stop):
 ll_listdelslice_startstop.oopspec = 'list.delslice_startstop(l, start, stop)'
 
 def ll_listsetslice(l1, start, stop, l2):
-    count = l2.ll_length()
+    len1 = l1.ll_length()
+    len2 = l2.ll_length()
     ll_assert(start >= 0, "l[start:x] = l with unexpectedly negative start")
-    ll_assert(start <= l1.ll_length(), "l[start:x] = l with start > len(l)")
-    ll_assert(stop <= l1.ll_length(), "stop cannot be past the end of l1")
-    ll_assert(count == stop - start,
-                 "setslice cannot resize lists in RPython")
-    # XXX ...but it would be easy enough to support if really needed
-    ll_arraycopy(l2, l1, 0, start, count)
+    ll_assert(start <= len1, "l[start:x] = l with start > len(l)")
+    ll_assert(stop <= len1, "stop cannot be past the end of l1")
+    if len2 == stop - start:
+        ll_arraycopy(l2, l1, 0, start, len2)
+    elif len2 < stop - start:
+        ll_arraycopy(l2, l1, 0, start, len2)
+        ll_arraycopy(l1, l1, stop, start + len2, len1 - stop)
+        l1._ll_resize_le(len1 + len2 - (stop - start))
+    else: # len2 > stop - start:
+        try:
+            newlength = ovfcheck(len1 + len2)
+        except OverflowError:
+            raise MemoryError
+        l1._ll_resize_ge(newlength)
+        ll_arraycopy(l1, l1, stop, start + len2, len1 - stop)
+        ll_arraycopy(l2, l1, 0, start, len2)
 
 
 # ____________________________________________________________
 #
 #  Comparison.
 
+def listeq_unroll_case(l1, l2, eqfn):
+    if jit.isvirtual(l1) and l1.ll_length() < 10:
+        return True
+    if jit.isvirtual(l2) and l2.ll_length() < 10:
+        return True
+    return False
+
+@jit.look_inside_iff(listeq_unroll_case)
 def ll_listeq(l1, l2, eqfn):
     if not l1 and not l2:
         return True
@@ -1003,7 +1047,8 @@ def ll_inplace_mul(l, factor):
     return res
 ll_inplace_mul.oopspec = 'list.inplace_mul(l, factor)'
 
-
+@jit.look_inside_iff(lambda _, l, factor: jit.isvirtual(l) and
+                     jit.isconstant(factor) and factor < 10)
 def ll_mul(RESLIST, l, factor):
     length = l.ll_length()
     if factor < 0:
