@@ -19,6 +19,8 @@
 #include "src/address.h"
 #include "src/unichar.h"
 #include "src/llgroup.h"
+#include "src/stack.h"
+#include "src/threadlocal.h"
 
 #include "src/instrument.h"
 #include "src/asm.h"
@@ -33,7 +35,6 @@
 #  include "src/debug_traceback.h"
 #endif
 
-#  include "src/allocator.h"
 #ifdef PYPY_STANDALONE
 #  include "src/entrypoint.h"
 #endif
@@ -49,3 +50,7 @@
 #ifdef __CYGWIN__
 #include "src/cygwin_wait.h"
 #endif
+
+#ifdef RPY_REVERSE_DEBUGGER
+#include "src-revdb/revdb_include.h"
+#endif
