@@ -1,27 +1,41 @@
 from rpython.translator.simplify import join_blocks, cleanup_graph
-from rpython.translator.unsimplify import copyvar, varoftype
+from rpython.translator.unsimplify import varoftype
 from rpython.translator.unsimplify import insert_empty_block, split_block
 from rpython.translator.backendopt import canraise, inline
 from rpython.flowspace.model import Block, Constant, Variable, Link, \
-    c_last_exception, SpaceOperation, FunctionGraph, mkentrymap
+    SpaceOperation, FunctionGraph, mkentrymap
 from rpython.rtyper.lltypesystem import lltype, llmemory, rffi
 from rpython.rtyper.lltypesystem import lloperation
-from rpython.rtyper.lltypesystem.rclass import ll_inst_type
+from rpython.rtyper.rclass import ll_inst_type
 from rpython.rtyper import rtyper
 from rpython.rtyper.rmodel import inputconst
 from rpython.rlib.rarithmetic import r_uint, r_longlong, r_ulonglong
-from rpython.rlib.rarithmetic import r_singlefloat
+from rpython.rlib.rarithmetic import r_singlefloat, r_longfloat
 from rpython.rlib.debug import ll_assert
-from rpython.annotator import model as annmodel
+from rpython.rtyper.llannotation import lltype_to_annotation
 from rpython.rtyper.annlowlevel import MixLevelHelperAnnotator
 from rpython.tool.sourcetools import func_with_new_name
 
+# ~~~ NOTE ~~~
+# The exact value returned by a function is NOT DEFINED. The returned value is
+# IGNORED EVERYWHERE in RPython and the question "was_an_exception_raised()"
+# is implemented by checking the content of the global exc_data.
+#
+# The only case in which the returned error value is significant is for
+# llhelpers which can be called by 3rd-party C functions, such as e.g. the HPy
+# API. In that case, the error value is specified by @llhelper_error_value.
+#
+# The following table defines the default error values to return, but in
+# general the returned value is not guaranteed. The only case in which it is
+# guaranteed is for functions decorated with @llhelper_error_value
+
 PrimitiveErrorValue = {lltype.Signed: -1,
                        lltype.Unsigned: r_uint(-1),
                        lltype.SignedLongLong: r_longlong(-1),
                        lltype.UnsignedLongLong: r_ulonglong(-1),
                        lltype.Float: -1.0,
                        lltype.SingleFloat: r_singlefloat(-1.0),
+                       lltype.LongFloat: r_longfloat(-1.0),
                        lltype.Char: chr(255),
                        lltype.UniChar: unichr(0xFFFF), # XXX is this always right?
                        lltype.Bool: True,
@@ -32,15 +46,33 @@ for TYPE in rffi.NUMBER_TYPES:
     PrimitiveErrorValue[TYPE] = lltype.cast_primitive(TYPE, -1)
 del TYPE
 
-def error_value(T):
+def default_error_value(T):
     if isinstance(T, lltype.Primitive):
         return PrimitiveErrorValue[T]
     elif isinstance(T, lltype.Ptr):
         return lltype.nullptr(T.TO)
     assert 0, "not implemented yet"
 
-def error_constant(T):
-    return Constant(error_value(T), T)
+def has_llhelper_error_value(graph):
+    return hasattr(graph, 'func') and hasattr(graph.func, '_llhelper_error_value_')
+
+def error_value(graph):
+    assert isinstance(graph, FunctionGraph)
+    T = graph.returnblock.inputargs[0].concretetype
+    if has_llhelper_error_value(graph):
+        # custom case
+        errval = graph.func._llhelper_error_value_
+        if lltype.typeOf(errval) != T:
+            raise TypeError('Wrong type for @llhelper_error_value: expected %s '
+                            'but got %s' % (T, lltype.typeOf(errval)))
+        return errval
+    # default case
+    return default_error_value(T)
+
+def error_constant(graph):
+    assert isinstance(graph, FunctionGraph)
+    T = graph.returnblock.inputargs[0].concretetype
+    return Constant(error_value(graph), T)
 
 def constant_value(llvalue):
     return Constant(llvalue, lltype.typeOf(llvalue))
@@ -65,7 +97,7 @@ class ExceptionTransformer(object):
             assertion_error_ll_exc_type)
         self.c_n_i_error_ll_exc_type = constant_value(n_i_error_ll_exc_type)
 
-        def rpyexc_occured():
+        def rpyexc_occurred():
             exc_type = exc_data.exc_type
             return bool(exc_type)
 
@@ -108,9 +140,9 @@ class ExceptionTransformer(object):
                 exc_data.exc_type = ll_inst_type(evalue)
                 exc_data.exc_value = evalue
 
-        self.rpyexc_occured_ptr = self.build_func(
+        self.rpyexc_occurred_ptr = self.build_func(
             "RPyExceptionOccurred",
-            rpyexc_occured,
+            rpyexc_occurred,
             [], lltype.Bool)
 
         self.rpyexc_fetch_type_ptr = self.build_func(
@@ -163,7 +195,7 @@ class ExceptionTransformer(object):
         return fn
 
     def build_func(self, name, fn, inputtypes, rettype, **kwds):
-        l2a = annmodel.lltype_to_annotation
+        l2a = lltype_to_annotation
         graph = self.mixlevelannotator.getgraph(fn, map(l2a, inputtypes), l2a(rettype))
         return self.constant_func(name, inputtypes, rettype, graph,
                                   exception_policy="exc_helper", **kwds)
@@ -242,7 +274,7 @@ class ExceptionTransformer(object):
         elif block is graph.returnblock:
             return need_exc_matching, n_gen_exc_checks
         last_operation = len(block.operations) - 1
-        if block.exitswitch == c_last_exception:
+        if block.canraise:
             need_exc_matching = True
             last_operation -= 1
         elif (len(block.exits) == 1 and
@@ -250,8 +282,8 @@ class ExceptionTransformer(object):
               len(block.operations) and
               (block.exits[0].args[0].concretetype is lltype.Void or
                block.exits[0].args[0] is block.operations[-1].result) and
-              block.operations[-1].opname not in ('malloc',     # special cases
-                                                  'malloc_nonmovable')):
+              block.operations[-1].opname not in ('malloc', 'malloc_varsize') and
+              not has_llhelper_error_value(graph)):  # special cases
             last_operation -= 1
         lastblock = block
         for i in range(last_operation, -1, -1):
@@ -259,19 +291,16 @@ class ExceptionTransformer(object):
             if not self.raise_analyzer.can_raise(op):
                 continue
 
-            splitlink = split_block(None, block, i+1)
+            splitlink = split_block(block, i+1)
             afterblock = splitlink.target
             if lastblock is block:
                 lastblock = afterblock
 
-            self.gen_exc_check(block, graph.returnblock, afterblock)
+            self.gen_exc_check(graph, block, graph.returnblock, afterblock)
             n_gen_exc_checks += 1
         if need_exc_matching:
-            assert lastblock.exitswitch == c_last_exception
+            assert lastblock.canraise
             if not self.raise_analyzer.can_raise(lastblock.operations[-1]):
-                #print ("operation %s cannot raise, but has exception"
-                #       " guarding in graph %s" % (lastblock.operations[-1],
-                #                                  graph))
                 lastblock.exitswitch = None
                 lastblock.recloseblock(lastblock.exits[0])
                 lastblock.exits[0].exitcase = None
@@ -308,8 +337,7 @@ class ExceptionTransformer(object):
         reraise = self.comes_from_last_exception(entrymap, link)
         result = Variable()
         result.concretetype = lltype.Void
-        block = Block([copyvar(None, v)
-                       for v in graph.exceptblock.inputargs])
+        block = Block([v.copy() for v in graph.exceptblock.inputargs])
         if reraise:
             block.operations = [
                 SpaceOperation("direct_call",
@@ -325,8 +353,7 @@ class ExceptionTransformer(object):
                                varoftype(lltype.Void)),
                 ]
         link.target = block
-        RETTYPE = graph.returnblock.inputargs[0].concretetype
-        l = Link([error_constant(RETTYPE)], graph.returnblock)
+        l = Link([error_constant(graph)], graph.returnblock)
         block.recloseblock(l)
 
     def insert_matching(self, block, graph):
@@ -348,7 +375,7 @@ class ExceptionTransformer(object):
         inlined, the correct exception matching blocks are produced."""
         # XXX slightly annoying: construct a graph by hand
         # but better than the alternative
-        result = copyvar(None, op.result)
+        result = op.result.copy()
         opargs = []
         inputargs = []
         callargs = []
@@ -369,7 +396,7 @@ class ExceptionTransformer(object):
         newgraph = FunctionGraph("dummy_exc1", startblock)
         startblock.closeblock(Link([result], newgraph.returnblock))
         newgraph.returnblock.inputargs[0].concretetype = op.result.concretetype
-        self.gen_exc_check(startblock, newgraph.returnblock)
+        self.gen_exc_check(newgraph, startblock, newgraph.returnblock)
         excblock = Block([])
 
         llops = rtyper.LowLevelOpList(None)
@@ -391,11 +418,7 @@ class ExceptionTransformer(object):
         fptr = self.constant_func("dummy_exc1", ARGTYPES, op.result.concretetype, newgraph)
         return newgraph, SpaceOperation("direct_call", [fptr] + callargs, op.result)
 
-    def gen_exc_check(self, block, returnblock, normalafterblock=None):
-        #var_exc_occured = Variable()
-        #var_exc_occured.concretetype = lltype.Bool
-        #block.operations.append(SpaceOperation("safe_call", [self.rpyexc_occured_ptr], var_exc_occured))
-
+    def gen_exc_check(self, graph, block, returnblock, normalafterblock=None):
         llops = rtyper.LowLevelOpList(None)
 
         spaceop = block.operations[-1]
@@ -406,6 +429,10 @@ class ExceptionTransformer(object):
         else:
             v_exc_type = self.gen_getfield('exc_type', llops)
             var_no_exc = self.gen_isnull(v_exc_type, llops)
+        #
+        # We could add a "var_no_exc is likely true" hint, but it seems
+        # not to help, so it was commented out again.
+        #var_no_exc = llops.genop('likely', [var_no_exc], lltype.Bool)
 
         block.operations.extend(llops)
 
@@ -414,7 +441,7 @@ class ExceptionTransformer(object):
         b = Block([])
         b.operations = [SpaceOperation('debug_record_traceback', [],
                                        varoftype(lltype.Void))]
-        l = Link([error_constant(returnblock.inputargs[0].concretetype)], returnblock)
+        l = Link([error_constant(graph)], returnblock)
         b.closeblock(l)
         l = Link([], b)
         l.exitcase = l.llexitcase = False
@@ -424,44 +451,40 @@ class ExceptionTransformer(object):
         l0.exitcase = l0.llexitcase = True
 
         block.recloseblock(l0, l)
-
         insert_zeroing_op = False
-        if spaceop.opname == 'malloc':
+        if spaceop.opname in ['malloc','malloc_varsize']:
             flavor = spaceop.args[1].value['flavor']
             if flavor == 'gc':
                 insert_zeroing_op = True
-        elif spaceop.opname == 'malloc_nonmovable':
-            # xxx we cannot insert zero_gc_pointers_inside after
-            # malloc_nonmovable, because it can return null.  For now
-            # we simply always force the zero=True flag on
-            # malloc_nonmovable.
-            c_flags = spaceop.args[1]
-            c_flags.value = c_flags.value.copy()
-            spaceop.args[1].value['zero'] = True
+            true_zero = spaceop.args[1].value.get('zero', False)
         # NB. when inserting more special-cases here, keep in mind that
         # you also need to list the opnames in transform_block()
         # (see "special cases")
 
         if insert_zeroing_op:
             if normalafterblock is None:
-                normalafterblock = insert_empty_block(None, l0)
+                normalafterblock = insert_empty_block(l0)
             v_result = spaceop.result
             if v_result in l0.args:
                 result_i = l0.args.index(v_result)
                 v_result_after = normalafterblock.inputargs[result_i]
             else:
-                v_result_after = copyvar(None, v_result)
+                v_result_after = v_result.copy()
                 l0.args.append(v_result)
                 normalafterblock.inputargs.append(v_result_after)
+            if true_zero:
+                opname = "zero_everything_inside"
+            else:
+                opname = "zero_gc_pointers_inside"
             normalafterblock.operations.insert(
-                0, SpaceOperation('zero_gc_pointers_inside',
-                                  [v_result_after],
+                0, SpaceOperation(opname, [v_result_after],
                                   varoftype(lltype.Void)))
 
     def setup_excdata(self):
         EXCDATA = lltype.Struct('ExcData',
             ('exc_type',  self.lltype_of_exception_type),
-            ('exc_value', self.lltype_of_exception_value))
+            ('exc_value', self.lltype_of_exception_value),
+            hints={'is_excdata': True})
         self.EXCDATA = EXCDATA
 
         exc_data = lltype.malloc(EXCDATA, immortal=True)
